c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: John R. Barker and Thanh Lam Nguyen
c      jrbarker@umich.edu     nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c                                                                 
c    PROGRAM adensum    
c
c    by      
c
c    John R. Barker   and   Thanh Lam Nguyen                       
c    jrbarker@umich.edu     nguyenlt@umich.edu
c                                                                  
c    ***Wang-Landau Algorithm for Densities of States***
c
c
c@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c    Citations:                                                    
c                                                                  
c    1. F. Wang and D. P. Landau, Phys. Rev. Letters 86, 2050-2053 (2001).           
c                                                                  
c    2. M. Basire, P. Parneix, and F. Calvo, J. Chem.Phys. 129, 081101 (2008).    
c                                                                  
c    3. T. L. Nguyen and J. R. Barker, to be published (2010).     
c                                                                  
c@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

      PROGRAM adensum

      IMPLICIT NONE

      CHARACTER AVERSION*6 , ADATE*8 , cut*46 , FNAME*10
      PARAMETER ( AVERSION='2010', ADATE='Jan 2010' )
      
      PARAMETER (cut='**************INPUT DATA SUMMARY**************')

      INTEGER index(100), ns, nv(100), i, j, k, MaxTrials, nsteps,
     & idum, nvm, m, ntrials, jtest, l , nstart , Nacc, Nrej, Nout,
     & iv , kk , nvold(100), iters, ngrains , nold , nnew , ndm ,
     & ndmax , nh , ig , nnt, nvmax , nntmin , KIN
      PARAMETER(KIN=2)                                                  ! input UNIT=2

      DOUBLE PRECISION wa , w0 , wf , xa , ya , za ,
     &                 Etot, Eu, Eb, Percent, TrialMax, 
     &                 Beta , Sum(10000), Error(10000), arg,
     &                 Emin, Emax, Estart, energy, 
     &                 RAN1, fac, succ, succ2, D(100),
     &                 anumb, enrg, dens(10000), eff(10000), zpe ,
     &                 an ,  ave , av0, avf , Etop
     
      DOUBLE PRECISION H(10000), g(10000), p , p0 , f , fo , test , 
     &                 acc , hmax , hmin , have, hav2, y , var
 
      CHARACTER TITLE1*100, TITLE2*100, WW*2 , KEYWORD*6

      CHARACTER IDOF(100)*3, Vhr(100)*5, Bhr(100)*5
      DOUBLE PRECISION HRfreq(100), HRVo(100), HRI(100), 
     &     HRB(100), HRzpe(100), DUM1, DUM2
      DOUBLE PRECISION CV(100,100), CB(100,100), CVt(100), CBt(100) 
      DOUBLE PRECISION Phav(100), Phab(100)
      INTEGER Nsep, NG(100), IMAX(100), NVV(100), NBB(100), 
     &     NGV(100), NGB(100)
      DOUBLE PRECISION T(10000), AT(10000), DS(10000), SS(10000)

      INTEGER I1, J1, K1, L1, NY, NZ , MODE(100)
      INTEGER nvk, nvmaxxyz, JMAX 
      DOUBLE PRECISION energyxyz
     
      DOUBLE PRECISION BB(3), B, temp
      INTEGER JB
      INTEGER lenstr

      DOUBLE PRECISION RI(100)
      INTEGER NRSN(100), IDIM(100)
      INTEGER NDIM, NSYMM, JK
 
      DOUBLE PRECISION Egrain1 , Egrain2 , Emax1 , Emax2 , Viblo
      INTEGER          imax1 , imax2 , Isize , NMAX
      PARAMETER (NMAX=50001)  

      INTEGER ntest
      
      COMMON/I/ idum
      COMMON/wxyz/ wa(100) , w0(100) , wf(100) , xa(100,100) ,
     &   ya(100,100,100) , za(100,100,100,100) , zpe , ave ,
     &   av0 , avf , Viblo

      EXTERNAL energy , energyxyz, RAN1 , ndmax, nvmax, nvmaxxyz
      
      SAVE
      
      OPEN (UNIT=2,FILE='adensum.dat',STATUS='OLD')
      OPEN (UNIT=4,FILE='adensum.out',STATUS='UNKNOWN')

      READ (2,*) FNAME

      WRITE(4,9001) AVERSION , ADATE , AVERSION , ADATE

      READ(2,9002)  TITLE1
      READ(2,9002)  TITLE2

      WRITE (4,99023) FNAME
      WRITE(4,9002) TITLE1
      WRITE(4,9002) TITLE2 

      CALL DateTime(4)

      WRITE(4,9005)

      OPEN(UNIT=3,STATUS='UNKNOWN',FILE=FNAME(1:lenstr(fname))//'.dens')  ! Succinct output for MultiWell input

      WRITE (3,99030) cut                                       ! Start of data summary block in ____.dens file
      WRITE(3,9001) AVERSION , ADATE , AVERSION , ADATE
      WRITE (3,99023) FNAME
      WRITE(3,9002) TITLE1 
      WRITE(3,9002) TITLE2 

      CALL DateTime(3)


      READ(2,*)  ns , NY, NZ, WW

      CALL READ_WXYZ(KIN , ns , NY, NZ, WW)                            ! read frequencies and anharmonicities
      
      READ(2,*) NSEP                                           ! No. of separable DOF

      DO I=ns+1, ns+NSEP
         READ(2,*) MODE(I), IDOF(I), DUM1, DUM2, NG(I)
         IF (IDOF(I).EQ.'hra') THEN
            HRfreq(I)=DUM1
            HRI(I)=DUM2
            HRB(I)=16.85763d0/HRI(I)
            HRVo(I)=((HRfreq(I)/NG(I))**2)/HRB(I)
         ELSEIF (IDOF(I).EQ.'hrb') THEN
            HRfreq(I)=DUM1
            HRVo(I)=DUM2
            HRB(I)=((HRfreq(I)/NG(I))**2)/HRVo(I)
            HRI(I)=16.85763d0/HRB(I)
         ELSEIF (IDOF(I).EQ.'hrc') THEN
            HRI(I)=DUM1
            HRVo(I)=DUM2
            HRB(I)=16.85763d0/HRI(I)
            HRfreq(I)=NG(I)*sqrt(HRVo(I)*HRB(I))
         ELSEIF (IDOF(I).EQ.'hrd') THEN
            NVV(I)=INT(DUM1)
            NBB(I)=INT(DUM2)
            READ(2,*) Vhr(I), NGV(I), Phav(I), (CV(I,J), J=1, NVV(I))
            READ(2,*) Bhr(I), NGB(I), Phab(I), (CB(I,J), J=1, NBB(I))
         ELSEIF (IDOF(I).EQ.'qro') THEN
            RI(I) = DUM1
            NRSN(I) = DUM2
            IDIM(I) = NG(I)
         ELSEIF (IDOF(I).EQ.'kro') THEN
            RI(I) = DUM1
            NRSN(I) = DUM2
            IDIM(I) = NG(I)
         ELSEIF (IDOF(I).EQ.'box') THEN
            RI(I) = DUM1
            NRSN(I) = DUM2
            IDIM(I) = NG(I)
        ENDIF
      END DO
 
      READ (2,*) Egrain1 , imax1 , Isize , Emax2 , KEYWORD
      IF (Emax2/Egrain1 .GT. NMAX) THEN
        WRITE(*,*) '****Grain Size Too Small****'
        STOP
      ENDIF
c
c   ******* END INPUT *******
c
!!!      Emax = Emax2                             ! Top of Wang-Landou energy window

	Emax = Emax2 + Egrain1

      Emin = 0.0                               ! Bottom of Wang-Landou energy window
      Emax1 = Egrain1*(imax1-1)
      Egrain2 = Emax2/(Isize-imax1-1)
      imax2 = Isize - imax1
 
      JMAX = 1 + INT(Emax2/Egrain1)
      JMAX = MIN(JMAX,NMAX)

      CALL convib( ns , WW )                                    ! convert input frequencies to other forms

      WRITE(4,9003) ns
      WRITE(3,9003) ns
      DO i = 1 , ns
        D(i) = 0.0
        IF (xa(i,i) .LT. 0.0) D(i) = -0.25*w0(i)**2/xa(i,i)     ! Bond dissociation energy
        WRITE(4,9004) i, wa(i), w0(i), wf(i), D(i)
        WRITE(3,9004) i, wa(i), w0(i), wf(i), D(i)
      END DO

      WRITE(4, 9013) ave, av0, avf, zpe                         ! zero point energy and average frequencies

      WRITE(4,9006)
      WRITE(3,9006)
      WRITE(4,9043) (i, i=1,ns)
      WRITE(3,9043) (i, i=1,ns)
      DO j = 1 , ns
        WRITE(4,9042) j, (xa(j,i), i=1,ns)
        WRITE(3,9042) j, (xa(j,i), i=1,ns)
      END DO

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c    Write out separable degrees of freedom
c

      write(4,*)
      write(3,*)
      DO I=ns+1, ns+Nsep
            IF (IDOF(I).EQ.'hra') THEN
                write(4,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)
                write(3,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)

            ELSEIF (IDOF(I).EQ.'hrb') THEN
                write(4,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)
                write(3,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)

            ELSEIF (IDOF(I).EQ.'hrc') THEN
                write(4,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)
                write(3,9030) MODE(I), HRfreq(I), HRI(I), NG(I), NG(I), 
     &               HRB(I), HRVo(I)

            ELSEIF (IDOF(I).EQ.'hrd') THEN
                write(4,9031) MODE(I), NG(I)
                write(4,9032) Vhr(I), NGV(I), Phav(I),
     &                  (CV(I,J), J=1, NVV(I))
                write(3,9031) MODE(I), NG(I)
                write(3,9032) Vhr(I), NGV(I), Phav(I),
     &                  (CV(I,J), J=1, NVV(I))

                IF(Bhr(I).EQ.'Bhrd1') THEN
                  write(4,9033) Bhr(I), NGB(I), Phab(I),
     &                  (CB(I,J), J=1, NBB(I))
                  write(3,9033) Bhr(I), NGB(I), Phab(I),
     &                  (CB(I,J), J=1, NBB(I))
                ELSE
                  write(4,9033) Bhr(I), NGB(I), Phab(I),
     &                  (CB(I,J), J=1, NBB(I))
                  write(3,9033) Bhr(I), NGB(I), Phab(I),
     &                  (CB(I,J), J=1, NBB(I))
                ENDIF

            ELSEIF (IDOF(I).EQ.'qro') THEN
                  B = 16.85763d0 / RI(I)
                  write(4,99041) MODE(I), RI(I), NRSN(I), IDIM(I), B
                  write(3,99041) MODE(I), RI(I), NRSN(I), IDIM(I), B

            ELSEIF (IDOF(I).EQ.'kro') THEN
                  B = 16.85763d0 / RI(I)
                  write(4,99042) MODE(I), RI(I), NRSN(I), IDIM(I), B
                  write(3,99042) MODE(I), RI(I), NRSN(I), IDIM(I), B

            ELSEIF (IDOF(I).EQ.'box') THEN
                  write(4,99043) MODE(I), RI(I), IDIM(I)
                  write(3,99043) MODE(I), RI(I), IDIM(I)
                  
            ENDIF

      ENDDO

        write(4,*)
        write(3,*)

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        write(4,*)
        write(3,*)

c-----------------------------------------------------------------------------------------
c
c         WANG-LANDAU PARAMETERS
C
c-----------------------------------------------------------------------------------------
      p0 = 1.0/ns                                       ! Scaled according to number of vibrations
      IF ( p0 .GT. 0.25 ) p0 = 0.25
      iters = 21                                        ! Always
      ngrains = (Emax-Emin)/Egrain1 + 1                 ! should be same as JAMX (when Emin = 0.0)
      Etop = Emax + Egrain1
      fo = 1.0d+00

c-----------------------------------------------------------------------------------------
c   Trials per energy grain
c
      IF    ( KEYWORD .EQ. 'extra' ) THEN
        TrialMax = 1.e+6
      ELSEIF ( KEYWORD .EQ. 'Extra' ) THEN
        TrialMax = 1.e+6
      ELSEIF ( KEYWORD .EQ. 'EXTRA' ) THEN
        TrialMax = 1.e+6
      ELSEIF ( KEYWORD .EQ. 'best' ) THEN
        TrialMax = 1.e+5
      ELSEIF ( KEYWORD .EQ. 'Best' ) THEN
        TrialMax = 1.e+5
      ELSEIF ( KEYWORD .EQ. 'BEST' ) THEN
        TrialMax = 1.e+5
      ELSEIF ( KEYWORD .EQ. 'better' ) THEN
        TrialMax = 1.e+4
      ELSEIF ( KEYWORD .EQ. 'Better' ) THEN
        TrialMax = 1.e+4
      ELSEIF ( KEYWORD .EQ. 'BETTER' ) THEN
        TrialMax = 1.e+4
      ELSEIF ( KEYWORD .EQ. 'good' ) THEN
        TrialMax = 1.e+3
      ELSEIF ( KEYWORD .EQ. 'Good' ) THEN
        TrialMax = 1.e+3
      ELSEIF ( KEYWORD .EQ. 'GOOD' ) THEN
        TrialMax = 1.e+3
      ELSEIF ( KEYWORD .EQ. 'fair' ) THEN
        TrialMax = 1.e+2
      ELSEIF ( KEYWORD .EQ. 'Fair' ) THEN
        TrialMax = 1.e+2
      ELSEIF ( KEYWORD .EQ. 'FAIR' ) THEN
        TrialMax = 1.e+2
      ELSE                             ! KEYWORD not recognized
        TrialMax = 1.e+2               ! default = 'Fair'
        KEYWORD = 'Key???'
      ENDIF

      TrialMax = ngrains * TrialMax   
   
      IF ( TrialMax .GT. 2.d+09 ) TrialMax = 2.d+09       ! 32 bit machine (INTEGER*4) upper limit 

      MaxTrials = INT( TrialMax )
c-----------------------------------------------------------------------------------------

      idum = -1147483647                                  ! Random number seed
      y = RAN1( idum )
      f  = fo
c
c     Initialize quantum numbers
c
 
      ntest = 1
      DO WHILE ( ntest .EQ. 1 )
       Estart = Emin + RAN1( idum )*(Emax-Emin)
        DO l = 1, ns
          DO k=l+1, ns
               nvold(k)=0
          ENDDO
          IF((NY.EQ.0).AND.(NZ.EQ.0)) THEN
            Eu=Estart-energy( ns, wa, xa, nvold )
            IF(Eu .NE. 0.0d0) THEN
               nstart=nvmax( ns, wa, xa, nvold, l, Eu )
               nvold(l) = nstart*RAN1( idum )
            ELSE
               nvold(l) = 0
            ENDIF
          ELSE
            Eu=Estart-energyxyz( ns , wa , xa , ya, za, nvold )       ! initializg quantum numbers
            IF(Eu .NE. 0.0d0) THEN
               nstart = nvmaxxyz( ns, wa, xa, ya, za, nvold, l, Eu )
               nvold(l) = nstart*RAN1( idum )
            ELSE
               nvold(l) = 0
            ENDIF
          ENDIF
        ENDDO

      IF((NY.EQ.0).AND.(NZ.EQ.0)) THEN
         CALL ckderiv( ns, nvold, wa, xa, ntest )               ! check derivatives for X-matrix
      ELSE
         CALL ckderivxyz( ns, nvold, wa, xa, ya, za, ntest )    ! check derivatives for XYZ-matrix
      ENDIF

      END DO                                                    ! END of DO WHILE


      IF((NY.EQ.0).AND.(NZ.EQ.0)) THEN
          enrg = energy( ns, wa, xa, nvold )
      ELSE
             enrg = energyxyz( ns , wa , xa , ya, za, nvold )             
      ENDIF
        nold = IDNINT( (enrg - Emin)/Egrain1 ) + 1              ! energy grain number: nearest integer
            
      DO i = 1 , ngrains                                        ! initialize grains
        g(i) = 0.0d+00
      END DO
      
c-----------------------------------------------------------------------------------------
c-----------------------------------------------------------------------------------------
c     Start Iterations
c-----------------------------------------------------------------------------------------
c-----------------------------------------------------------------------------------------
      p = p0
      DO i = 1 , iters
        WRITE(*,9020) i , iters, f, p
        DO l = 1 , ngrains                                      ! initialize Histogram
          H(l) = 0.0d+00
        END DO

        Nrej = 0
        Nacc = 0
        Nout = 0
        
        nnt = MaxTrials
        DO j = 1 , nnt

          DO k = 1 , ns                                          ! select new quantum numbers
            test = RAN1(idum)
            IF ( test .LE. p  ) THEN                             ! down-step
               nv(k) = nvold(k) - 1
               IF ( nv(k) .LT. 0 ) nv(k) = 0 
             ELSEIF (test .GT. p .AND. test .LE. 2.0d0*p )THEN   ! up-step

                nv(k) = nvold(k) + 1

             ELSE                                                ! sit-tight
               nv(k) = nvold(k)
            ENDIF
          END DO                                                 ! end quantum number selection

      IF((NY.EQ.0).AND.(NZ.EQ.0)) THEN
          enrg = energy( ns, wa, xa, nv)
      ELSE
          enrg = energyxyz( ns , wa , xa , ya, za, nv )             ! new energy above zpe
      ENDIF

	IF ( enrg.GE.Emin .AND. enrg.LE.Emax ) THEN	! check energy boundary

      ntest = 0
      IF((NY.EQ.0).AND.(NZ.EQ.0)) THEN
         CALL ckderiv( ns, nv, wa, xa, ntest )		! check derivatives for X-matrix 
      ELSE
         CALL ckderivxyz( ns, nv, wa, xa, ya, za, ntest )		! check derivatives for XYZ-matrix 
      ENDIF

	ELSE
		ntest = 1				! fail test, ntest = 1
	ENDIF

	IF ( ntest.EQ.0 ) THEN            ! Accept or reject? pass: ntest = 0; fail: ntest = 1 
            nnew = IDNINT( (enrg - Emin)/Egrain1 ) + 1           ! energy grain number: nearest integer
            acc = exp( g(nold)-g(nnew) )                         ! acceptance probability
            test = RAN1(idum)
            If ( test .LE. acc ) THEN                            ! accepted for a move
              Nacc = Nacc + 1        
              H(nnew) = H(nnew) + 1.d+00
              g(nnew) = g(nnew) + f
              nold = nnew                                        ! energy grain
                DO l = 1 , ns
                  nvold(l) = nv(l)                               ! quantum numbers
                END DO
             ELSE                                                ! rejected: sits tight
              Nrej = Nrej + 1
              H(nold) = H(nold) + 1.d+00
              g(nold) = g(nold) + f
            END IF
	ELSE                                                 ! rejected: tried to step out of energy range
              Nout = Nout + 1
              H(nold) = H(nold) + 1.d+00
              g(nold) = g(nold) + f
	ENDIF             
          
        END DO           ! j -> ntrials
        
        f = f/2.0                      ! prepare for next iteration

      END DO             ! i -> iters

      CALL DateTime(4)

      hmax = 0.0
      hmin = 1.e+10
      have = 0.0d+00
      nh = 0
      DO ig = 1 , ngrains    
        hmax = MAX( hmax, H(ig) )      ! Maximum histogram
        IF (H(ig) .GT. 1.e-2) THEN
          hmin = MIN( hmin, H(ig) )    ! minimum non-zero histogram
          have = have + H(ig)          ! sum of non-zero histogram values
          hav2 = hav2 + H(ig)**2       ! sum of squares
          nh = nh + 1
        END IF
      END DO
      have = have/nh                   ! average non-zero histogram
      hav2 = hav2/nh                   ! average non-zero sum of squares
      hmax = (hmax-have)*100./have     ! % positive deviation
      hmin = (hmin-have)*100./have     ! % negative deviation
      var = hav2 - have**2             ! variance

      hav2 = 0.0d+00
      DO ig = 1 , ngrains    
        IF (H(ig) .GT. 1.e-2) hav2 = hav2 + abs( H(ig) - have )       ! sum of un-signed errors
      END DO
      hav2 = hav2/nh                  ! average un-signed deviation from mean
      hav2 = hav2*100.0/have          ! % average un-signed deviation from mean

c    PRINT OUTPUT

      WRITE(4,9008) TrialMax, KEYWORD, iters, p0 , fo
      WRITE(4,9016) 100.*Nacc/nnt , 100.*Nrej/nnt , 
     &       100.*Nout/nnt ,
     &       have , SQRT(var) , hav2 , hmax , hmin
      WRITE(3,9008) TrialMax, KEYWORD, iters, p0 , fo
      WRITE(3,9016) 100.*Nacc/nnt , 100.*Nrej/nnt , 
     &       100.*Nout/nnt ,
     &       have , SQRT(var) , hav2 , hmax , hmin

       WRITE(4,9017) 
       
c --------------------------------------------------------------------------

      DO I=1, ngrains

		temp = g(I)-g(1)
          T(I)=dexp(temp)      ! Normalize and obtain sums of states
          AT(I)=0.0d0

      ENDDO
c --------------------------------------------------------------------------
c
c     Convolution of separable modes with non-separable via
c       the Beyer-Swinehart/Stein-Rabinovitch Algorithm
c
      IF ( Nsep .GT. 0 ) THEN       ! If separable modes are to be included
      DO I=ns+1, ns + Nsep
          IF((IDOF(I).EQ.'hra').OR.(IDOF(I).EQ.'hrb').OR.
     &         (IDOF(I).EQ.'hrc')) THEN
              CALL SHRLEV(T,AT,Egrain1,ngrains,HRB(I),HRVo(I),NG(I),1,
     &        IMAX(I),HRzpe(I),
     &        'Vhrd1','Bhrd1',0.0d0,0.0d0,NG(I))  ! rigid, symmetrical hindered rotor
          ELSEIF (IDOF(I).EQ.'hrd') THEN
           DO J=1, NVV(I)
              CVt(J)=CV(I,J)
           ENDDO
           DO J=1, NBB(I)
              CBt(J)=CB(I,J)
           ENDDO
           CALL UHRLEV(T,AT,Egrain1,ngrains,NBB(I),NVV(I),CBt,CVt,
     &         NGV(I),NGB(I),IMAX(I),HRzpe(I),Vhr(I),Bhr(I),
     &         Phav(I),Phab(I),NG(I))   ! non-rigid, unsymmetrical hindered rotor
         ELSEIF (IDOF(I).EQ.'qro') THEN
            B=16.85763d0/RI(I)
            NSYMM=NRSN(I)
            NDIM=IDIM(I)
            CALL ROTLEV(T,AT,Egrain1,ngrains,B,NDIM,NSYMM)
         ELSEIF (IDOF(I).EQ.'kro') THEN
            B=16.85763d0/RI(I)
            NSYMM=NRSN(I)
            JK=IDIM(I)
            CALL KROTLEV(T,AT,Egrain1,ngrains,B,JK,NSYMM)
         ELSEIF (IDOF(I).EQ.'box') THEN
            B=RI(I)
            NDIM=IDIM(I)
            CALL BOXLEV(T,AT,Egrain1,ngrains,B,NDIM)
         ELSE
            write(*,*) "MISTAKES at combining modes"
         ENDIF
      ENDDO
      ENDIF  ! separable modes
c --------------------------------------------------------------------------
c
c     Compute sums and densities of states
c

      DS(1)=T(1)/Egrain1
      SS(1)=T(1)
      DO I=2, ngrains
         SS(I)=SS(I-1) + T(I)
         DS(I)=T(I)/Egrain1
      ENDDO
c --------------------------------------------------------------------------
c     Write adensum.out output file
c

        ngrains = ngrains - 1   !!! the boundary of interest

      DO I=1, ngrains
       enrg = Emin + (I-1)*Egrain1
       WRITE(4,9010) I , enrg , DS(I), SS(I)
      ENDDO 
c --------------------------------------------------------------------------
c
c      Write to ___.dens file (UNIT=3) 
c 

      WRITE (3,99030) cut                    ! End of data summary block in ____.dens file

      WRITE (3,*) FNAME(1:lenstr(fname))
      WRITE (3,*) TITLE1
      WRITE (3,99001) Egrain1 , imax1 , Emax2 , Isize , Viblo
      WRITE (3,99010)
      DO 200 I = 1 , imax1
         enrg = Emin + (I-1)*Egrain1
         WRITE (3,99014) I , enrg , DS(I), SS(I)
 200  CONTINUE
 
      DO 300 I = imax1 + 1 , Isize
         enrg = Emin + (I-imax1-1)*Egrain2
         j = INT(enrg/Egrain1) + 1
         WRITE (3,99014) I , enrg , DS(j), SS(j)
 300  CONTINUE
c
c       End of writing to ___.dens file
c
c --------------------------------------------------------------------------

9001  FORMAT (
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'//,
     &'                      aDenSum-',A6,//
     &'                       ',A8,//,
     &'              John R. Barker and Thanh Lam Nguyen',/
     &'          jrbarker@umich.edu     nguyenlt@umich.edu',//
     &'                   University of Michigan',/
     &'               Ann Arbor, Michigan 48109-2143',//
     &'           http://aoss.engin.umich.edu/multiwell',//
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%',//,
     &'Suggested Literature Citation:'//8x,
     &'a) MultiWell-',A6,' Software, ',A8,', designed and'/11x,
     &'maintained by J.R. Barker with contributors N.F. Ortiz,'/11x,
     &'J.M. Preses, L.L. Lohr, A. Maranzana, P.J. Stimac, T. L.'/11x,
     &'Nguyen, and T. J. Dhilip Kumar, University of Michigan,'/11x,
     &'Ann Arbor, MI; http://aoss.engin.umich.edu/multiwell/',//8x,
     &'b) F. Wang and D.P. Landau, Phys. Rev. Letters 86, ',/11x,
     &'2050-2053 (2001).',//8x,
     &'c) M. Basire, P. Parneix, and F. Calvo, J. Chem.Phys. 129,',/11x,
     &' 081101 (2008).',//8x,    
     &'d) T.L. Nguyen and J.R. Barker, J. Phys. Chem. A.,',/11x,
     &'submitted (2010).',
     &//,
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'/)
9002  FORMAT(A100)
9003  FORMAT(I3,' Vibrational Frequencies (cm-1)  ',/,
     & '  We; E = We*(v+1/2) +...; Wo: E = W0*v +...;  0<->1 fund.  ',/,
     & '  De: BDE for Separable Morse oscillator [0.0 if non-Morse]',//,
     & '  No.     We         Wo      0-1_Fund.   De(cm-1)' )
9004  FORMAT( I5,3(1x, F10.3),1x,F10.1 )
9041  FORMAT( 100(3x,I3,4x) )
9042  FORMAT( I3, 100( 1x , f9.3 ) )
9043  FORMAT( 100(7x,I3) )
9005  FORMAT(/)
9006  FORMAT('Anharmonicity Matrix (cm-1)')
9007  FORMAT( 100(f9.1,1x) )
9008  FORMAT(/'Monte Carlo trials: ', 1pe8.1,' ("',A6,'")',/,
     &        '  iterations = ',0pI2/,
     &        '          p0 = ',0pf7.3/,
     &        '          f  = ',0pf7.3/)
9010  FORMAT( 2x,I6,2x, F10.1,2(2x,1pe10.3) )
9011  FORMAT(A5)
9013  FORMAT(/,' Ave:', 3(1x, F10.3)/
     &         '(Anharmonic) Zero point energy (cm-1) = ', f10.3,/)
9014  FORMAT(///,'Bond Dissociation Energies (Do, cm-1)  [Do = 0.0 for n
     &on-Morse Oscillator]',/)
9015  FORMAT(10x,'[After conversion from ',A2,' to ',A2,']',/)
9016  FORMAT( '             steps up or down:',f10.3,' %  ',/,
     &        '               steps in place:',f10.3,' %  ',/,
     &        '           steps out of range:',f10.3,' %  ',/,
     &        '   average non-zero histogram: ',f10.0,' +/-', f8.0,/,
     &        '  average un-signed deviation:',0pf10.2,' %  ',/,
     &        '       max positive deviation:',0pf10.2,' %  ',/,
     &        '        max negative devation:',0pf10.2,' %',//)
9017  FORMAT(
     & '     No.        cm-1  states/cm-1  SumStates')
9020  FORMAT(' Iteration =',I3,'/',I2,5x,'f = ',1pe10.2, 5x,
     &                'p = ',1pe10.4)
9030     FORMAT(I3,2X,'Hind.Rot',2x,'Freq(har in cm-1)=',F8.2,2x,
     &     'Mom(amu.A**2)=',F8.3,2x,'fold=',I2,2x,'symm=',I2,2x,
     &     'B=',F8.4,' cm-1',2x,'Uo=',F7.1,' cm-1')

9031     FORMAT (I3,2X,'Non-Rigid HinRotor:',2x,'Rotor symm. =',I2)
9032     FORMAT (8X,A5,1x,' ; symmV =',I2,2x,'; Phase(rad.) = ',
     & F8.4,2x,'; Coeff. =',20(F10.4,1x))

9033     FORMAT (8X,A5,1x,' ; symmB =',I2,2x,'; Phase(rad.) = ',
     & F8.4,2x,'; Coeff. =',20(F10.4,1x))

9034     FORMAT (8X,A5,1x,' ; symmM =',I2,2x,'; Phase(rad.) = ',
     & F8.4,2x,'; Coeff. =',20(F10.4,1x))
9035     FORMAT(I3,2X,'HINDERED INTERNAL ROTORS') 

9100    FORMAT(1X,"External rotational constants (cm-1) ",/,
     &     1x, "B1 = ",F7.4,";",2x,"B2 = ",F7.4,";",2x,"B3 = ",F7.4,/,
     &     1x, "Angular momentum (J) = ",I3)

99001 FORMAT (1x,f10.1,1x,I10,1x,f10.1,1x,I10,1x,f10.1)
99010 FORMAT ('       No.     (cm-1)   Density   Sum')
99014 FORMAT (i10,1x,f10.1,2(1x,1pe10.3))
99023 FORMAT (/A10)
99030 FORMAT (A46)
99031 FORMAT (A6)
99032 FORMAT (A2)

9036     FORMAT(I3,2X,'OTHER (SEPARABLE) DOF')
99041 FORMAT (I3,2X,"Quantized Rotor:",2X,"I(Amu.A**2) = ",
     &	F8.3," ;",2X,"Sym. = ",I1," ;",2X,
     &	"Dim. = ",I2," ;",2X,"B(cm-1) = ",F8.3)
99042 FORMAT (I3,2X,"K-Rotor:",2X,"I(Amu.A**2) = ",
     &  F8.3," ;",2X,"Sym. = ",I2," ;",2X,"Quantum No. J = ",I2," ;",2X,
     &	"B(cm-1) = ",F7.3)
99043 FORMAT (I3,2X,"Particle-in-a-Box:",2X,"Freq(cm-1) = ",
     &  F7.2," ;",2X,"Dim. = ",I2)


      END

      SUBROUTINE BOXLEV(T,AT,DELE,JMAX,WE,NSYMM)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c DenSum: a code for calculating sums and densities of states.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
C
C
C
C	CALCULATES ENERGY LEVELS FOR PARTICLE-IN-A-BOX TO BE USED
C          WITH STERAB
C
c	DELE	= energy grain
c	JMAX	= index of Emax
c	WE	= vibration frequency (cm-1)
c	IR	= vector of energy level indices
c	NMAX	= ceiling of array
c       zpe     = zero point energy (cm-1)
c
c	E = WE*n^2  for n=1,2,3,...
C
C     OUTPUT ENERGIES IN WAVENUMBERS, relative to ZPE = WE
C
c    2/02 counted as vibs in Whitten-Rabinovitch
c         parameter calculations.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION IR(20001)
      DIMENSION T(JMAX), AT(JMAX)

      SAVE 
 
	Emax=(JMAX-1)*DELE
      RMAX = SQRT(Emax/WE)                           ! Maximum bound level
      NMAX = INT(RMAX) 
      IF ( NMAX.GT.20001 ) NMAX = 20001
      zpe = WE                                            ! Zero point energy
      DO 100 I = 1 , NMAX
        v2 = DBLE(I+1)*DBLE(I+1)                          ! "zero" level = 1, start loop at I+1=2
        R = (WE*v2 - zpe)/DELE                            ! Relative to zpe = WE
        IR(I) = NINT(R)                                 ! Nearest integer: number of grains
 100  CONTINUE

            DO 390 L = 1 , NSYMM                                ! Loop for degenerate vibrations
               DO 355 J = 1 , NMAX                              ! NMAX is the number of energy levels
                  DO 365 K = 1 , JMAX
                     KARG = K - IR(J)
                     IF ( KARG.GT.0 .AND. KARG.LE.JMAX )
     &                   AT(K) = AT(K) + T(KARG)
 365              CONTINUE
 355           CONTINUE

               DO 370 J = 1 , JMAX
                  T(J) = AT(J) + T(J)
                  AT(J) = 0.0D+00
 370           CONTINUE
 390        CONTINUE


      RETURN
      END
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          Aug. 9, 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c
c To compute the first derivative of Ev energy function 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	SUBROUTINE CALDERIXYZ(F,NVK,ns,k,nv,w,x,y,z)
        INTEGER ns, k, NVK, No
	PARAMETER (No=100)
        DOUBLE PRECISION A1, A2, A3, A4
        DOUBLE PRECISION x(No,No), y(No,No,No), z(No,No,No,No)
        DOUBLE PRECISION  w(No)
        DOUBLE PRECISION F
	INTEGER nv(No)

C
C Definition: 
C	F   : return a value of partitial derivative of Ev at the mode k
C	NVK : NVK = nv(k) 
C

	CALL CALA1234XYZ(A1,A2,A3,A4,ns,k,nv,w,x,y,z)
	CALL CALFXYZ(F,NVK,A1,A2,A3,A4)

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE CALFXYZ(F,NVK,A1,A2,A3,A4)
	INTEGER NVK
	DOUBLE PRECISION X, F, A1, A2, A3, A4
	
	X = 0.5d0 + NVK
	F = A1*(X**3) + A2*(X**2) + A3*X + A4
	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE CALA1234XYZ(A1,A2,A3,A4,ns,k,nv,w,x,y,z)
	INTEGER ns, i, j, k, l, No
	PARAMETER (No=100)
        DOUBLE PRECISION A1, A2, A3, A4
        DOUBLE PRECISION x(No,No), y(No,No,No), z(No,No,No,No)
        DOUBLE PRECISION  w(No)
	INTEGER nv(No)

        A1=4.0d0*z(k,k,k,k)

        A2=y(k,k,k)
        DO j=1, ns
                IF(j.ne.k) A2=A2+z(k,k,k,j)*(nv(j)+0.5d0)
        ENDDO
        A2=3.0d0*A2

        A3=x(k,k)
        DO j=1, ns
                IF(j.ne.k) A3=A3+y(k,k,j)*(nv(j)+0.5d0)
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN 
                        DO j=1, i
        IF(j.ne.k) A3=A3+z(k,k,i,j)*(nv(i)+0.5d0)*(nv(j)+0.5d0)
                        ENDDO
                ENDIF
        ENDDO
        A3=2.0d0*A3
        
        A4=w(k) 
        DO i=1, ns
                IF(i.ne.k) A4=A4+x(k,i)*(nv(i)+0.5d0)
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN 
                        DO j=1, i
        IF(j.ne.k) A4=A4+y(k,i,j)*(nv(i)+0.5d0)*(nv(j)+0.5d0)
                        ENDDO
                ENDIF
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN 
                DO j=1, i
                	IF(j.ne.k) THEN 
                        	DO l=1, j
       				IF(l.ne.k) THEN
	tp=(nv(i)+0.5d0)*(nv(j)+0.5d0)*(nv(l)+0.5d0)
	A4=A4+z(k,i,j,l)*tp
				ENDIF
                        	ENDDO
                	ENDIF
		ENDDO
                ENDIF
        ENDDO

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: John R. Barker and Thanh Lam Nguyen
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c-----------------------------------------------------------------------------------------
c    Test partial derivatives of Evib with respect to each quantum number
c
c    for bound states, all partial derivatives should be >0
c    ntest = 0 for PASS
c          = 1 for FAIL
c-----------------------------------------------------------------------------------------

      SUBROUTINE ckderiv( ns , nv , wa, xa, ntest ) 
      
      IMPLICIT NONE
      INTEGER ns , i , j , k , ntest , nv(100)
      DOUBLE PRECISION wa(100) , xa(100,100)
      DOUBLE PRECISION deriv, sum

      SAVE
      
      ntest = 0
      k = 1
      DO WHILE ( (k .LE. ns) .AND. (ntest .EQ. 0) )    ! start checking
         sum = 0.0d+00
         DO i = 1 , ns
           sum = sum + xa(i,k)*( nv(i)+0.5d0 ) 
         END DO
         deriv = wa(k) + ( nv(k)+0.5d0 )*xa(k,k) + sum
         IF ( deriv .LE. 0.0d+00 ) ntest = 1           ! failed test
         k = k + 1
      END DO
        
      RETURN
      END


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          Aug. 9, 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c
c To compute the first derivative of Ev energy function 
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	SUBROUTINE CKDERIVXYZ(ns, nv, wa, xa, ya, za, ntest)

      IMPLICIT NONE
      INTEGER ns , i , j , k , l, ntest , nv(100)
      DOUBLE PRECISION wa(100) , xa(100,100), deriv , sum 
      DOUBLE PRECISION ya(100,100,100) , za(100,100,100,100) 

        DOUBLE PRECISION XX, A1, A2, A3, A4, tp

      SAVE


      ntest = 0
      k = 1
      DO WHILE ( (k .LE. ns) .AND. (ntest .EQ. 0) )    ! start checking

        A1=4.0d0*za(k,k,k,k)

        A2=ya(k,k,k)
        DO j=1, ns
                IF(j.ne.k) A2=A2+za(k,k,k,j)*(nv(j)+0.5d0)
        ENDDO
        A2=3.0d0*A2

        A3=xa(k,k)
        DO j=1, ns
                IF(j.ne.k) A3=A3+ya(k,k,j)*(nv(j)+0.5d0)
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN
                        DO j=1, i
        IF(j.ne.k) A3=A3+za(k,k,i,j)*(nv(i)+0.5d0)*(nv(j)+0.5d0)
                        ENDDO
                ENDIF
        ENDDO
        A3=2.0d0*A3

        A4=wa(k)
        DO i=1, ns
                IF(i.ne.k) A4=A4+xa(k,i)*(nv(i)+0.5d0)
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN
                        DO j=1, i
        IF(j.ne.k) A4=A4+ya(k,i,j)*(nv(i)+0.5d0)*(nv(j)+0.5d0)
                        ENDDO
                ENDIF
        ENDDO
        DO i=1, ns
                IF(i.ne.k) THEN
                DO j=1, i
                        IF(j.ne.k) THEN
                                DO l=1, j
                                IF(l.ne.k) THEN
        tp=(nv(i)+0.5d0)*(nv(j)+0.5d0)*(nv(l)+0.5d0)
        A4=A4+za(k,i,j,l)*tp
                                ENDIF
                                ENDDO
                        ENDIF
                ENDDO
                ENDIF
        ENDDO

        XX = 0.5d0 + nv(k) 
        deriv = A1*(XX**3) + A2*(XX**2) + A3*XX + A4

         IF ( deriv .LE. 0.0d+00 ) ntest = 1           ! failed test
         k = k + 1
      END DO

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c-----------------------------------------------------------------------------------------
c     From input as wa, w0, or wf frequencies, convert to the other forms:
c       wa(i): harmonic frequencies with zero of energy at the potential minimum
c          Evib = SUMi( wai*(vi+1/2) ) + SUMi( SUMj( Xij*(vi+1/2)*(vj+1/2) ) )
c       w0(i): frequencies with zero of energy at zpe
c          Evib = SUMi( w0i*vi ) + SUMi( SUMj( Xij*vi*vj ) )
c       wf(i) = fundamental frequencies (for 0-1 transitions with all vj = 0)
c
c     zpe = zero point energy, based on We and X matrix
c
c     Average Frequences:
c       ave = average wa(i)
c       av0 = average w0(i)
c       avf = average wf(i)
c
c     [Herzberg, Infrared and Raman Spectra (D. van Nostrand Co., 1945), p. 206ff]
c     Eq. numbers from Herzberg
c-----------------------------------------------------------------------------------------

      SUBROUTINE convib( ns , WW ) 
      
      IMPLICIT NONE
      INTEGER ns , i , j
      DOUBLE PRECISION wa , w0 , wf , xa , ya , za , zpe, 
     &                 av0 , ave , avf , Viblo
      CHARACTER WW*2
      COMMON/wxyz/ wa(100) , w0(100) , wf(100) , xa(100,100) ,
     &   ya(100,100,100) , za(100,100,100,100) , zpe , ave ,
     &   av0 , avf , Viblo

      SAVE
      
      IF ( WW .EQ. 'We' ) THEN

      DO i = 1 , ns                               ! start conversion
        w0(i) = wa(i) + xa(i,i)                   ! Eq. (II,273)
        wf(i) = wa(i) + 2.0*xa(i,i)               ! fundamental
          DO j = 1, ns
            IF ( i .NE. j ) THEN
            w0(i) = w0(i) + 0.5*xa(i,j)           ! Eq. (II,273)
            wf(i) = wf(i) + 0.5*xa(i,j)           ! fundamental
            ENDIF
          END DO
      END DO                                      ! end conversion

      ELSEIF ( WW .EQ. 'W0' ) THEN

      DO i = 1 , ns                               ! start conversion
        wa(i) = w0(i) - xa(i,i)                   ! Eq. (II,273)
          DO j = 1, ns
            IF ( i .NE. j ) THEN
            wa(i) = wa(i) - 0.5*xa(i,j)           ! Eq. (II,273)
            ENDIF
          END DO
      END DO                                      ! end conversion
      DO i = 1 , ns                               ! start conversion
        wf(i) = wa(i) + 2.0*xa(i,i)               ! fundamental
          DO j = 1, ns
            IF ( i .NE. j ) THEN
            wf(i) = wf(i) + 0.5*xa(i,j)           ! fundamental
            ENDIF
          END DO
      END DO                                      ! end conversion

      ELSEIF ( WW .EQ. 'Wf' ) THEN

      DO i = 1 , ns                               ! start conversion
        wa(i) = wf(i) - 2.*xa(i,i)
          DO j = 1, ns
            IF ( i .NE. j ) THEN
            wa(i) = wa(i) - 0.5*xa(i,j)           ! Eq. (II,273)
            ENDIF
          END DO
      END DO                                      ! end conversion
      DO i = 1 , ns                               ! start conversion
        w0(i) = wa(i) + xa(i,i)                   ! Eq. (II,273)
          DO j = 1, ns
            IF ( i .NE. j ) THEN
            w0(i) = w0(i) + 0.5*xa(i,j)           ! Eq. (II,273)
            ENDIF
          END DO
      END DO                                      ! end conversion

      ENDIF
      
      zpe = 0.0d+00
      ave = 0.0d+00
      av0 = 0.0d+00
      avf = 0.0d+00
      Viblo = 10000.
      DO i = 1 , ns                               ! start ZPE
        zpe = zpe + 0.5*wa(i)                     ! Eq. (II,267)
        ave = ave + wa(i)                     ! summed frequency
        av0 = av0 + w0(i)                     ! summed frequency
        avf = avf + wf(i)                     ! summed frequency
        DO j = 1, i
          zpe = zpe + 0.25*xa(j,i)                ! Eq. (II,267)
        END DO
        Viblo = MIN( Viblo , wa(i) )              ! find lowest frequency
      END DO                                      ! end 

      ave = ave/ns                            ! average frequency
      av0 = av0/ns                            ! average frequency
      avf = avf/ns                            ! average frequency


      RETURN
      END
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
      SUBROUTINE DateTime(KUNIT)
c
c	Date & time subroutine
      CHARACTER*24 DateT
      REAL*4 tarray(2) , total
 
      CALL FDATE(DateT)
      WRITE (KUNIT,99001) DateT
      total = ETIME(tarray)
      WRITE (KUNIT,99002) tarray(1)
c      WRITE (KUNIT,99003) tarray(2)
      WRITE (KUNIT,99004) total/60.0
      RETURN
 
99001 FORMAT (/,10x,A24)
99002 FORMAT (10x,'Elapsed User Time: ',f12.3,' sec')
99003 FORMAT (10x,'Elapsed Sytem Time:',f12.3,' sec')
99004 FORMAT (10x,'Total Elapsed Time:',f12.3,' min',/)
      END
      subroutine rs(nm,n,a,w,matz,z,fv1,fv2,ierr)
c
      integer n,nm,ierr,matz
      double precision a(nm,n),w(n),z(nm,n),fv1(n),fv2(n)
c
c     this subroutine calls the recommended sequence of
c     subroutines from the eigensystem subroutine package (eispack)
c     to find the eigenvalues and eigenvectors (if desired)
c     of a real symmetric matrix.
c
c     on input
c
c        nm  must be set to the row dimension of the two-dimensional
c        array parameters as declared in the calling program
c        dimension statement.
c
c        n  is the order of the matrix  a.
c
c        a  contains the real symmetric matrix.
c
c        matz  is an integer variable set equal to zero if
c        only eigenvalues are desired.  otherwise it is set to
c        any non-zero integer for both eigenvalues and eigenvectors.
c
c     on output
c
c        w  contains the eigenvalues in ascending order.
c
c        z  contains the eigenvectors if matz is not zero.
c
c        ierr  is an integer output variable set equal to an error
c           completion code described in the documentation for tqlrat
c           and tql2.  the normal completion code is zero.
c
c        fv1  and  fv2  are temporary storage arrays.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      if (n .le. nm) go to 10
      ierr = 10 * n
      go to 50
c
   10 if (matz .ne. 0) go to 20
c     .......... find eigenvalues only ..........
      call  tred1(nm,n,a,w,fv1,fv2)
*  tqlrat encounters catastrophic underflow on the Vax
*     call  tqlrat(n,w,fv2,ierr)
      call  tql1(n,w,fv1,ierr)
      go to 50
c     .......... find both eigenvalues and eigenvectors ..........
   20 call  tred2(nm,n,a,w,fv1,z)
      call  tql2(nm,n,w,fv1,z,ierr)
   50 return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine tql1(n,d,e,ierr)
c
      integer i,j,l,m,n,ii,l1,l2,mml,ierr
      double precision d(n),e(n)
      double precision c,c2,c3,dl1,el1,f,g,h,p,r,s,s2,tst1,tst2,pythag
c
c     this subroutine is a translation of the algol procedure tql1,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues of a symmetric
c     tridiagonal matrix by the ql method.
c
c     on input
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct and
c          ordered for indices 1,2,...ierr-1, but may not be
c          the smallest eigenvalues.
c
c        e has been destroyed.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      if (n .eq. 1) go to 1001
c
      do 100 i = 2, n
  100 e(i-1) = e(i)
c
      f = 0.0d0
      tst1 = 0.0d0
      e(n) = 0.0d0
c
      do 290 l = 1, n
         j = 0
         h = dabs(d(l)) + dabs(e(l))
         if (tst1 .lt. h) tst1 = h
c     .......... look for small sub-diagonal element ..........
         do 110 m = l, n
            tst2 = tst1 + dabs(e(m))
            if (tst2 .eq. tst1) go to 120
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
  110    continue
c
  120    if (m .eq. l) go to 210
  130    if (j .eq. 30) go to 1000
         j = j + 1
c     .......... form shift ..........
         l1 = l + 1
         l2 = l1 + 1
         g = d(l)
         p = (d(l1) - g) / (2.0d0 * e(l))
         r = pythag(p,1.0d0)
         d(l) = e(l) / (p + dsign(r,p))
         d(l1) = e(l) * (p + dsign(r,p))
         dl1 = d(l1)
         h = g - d(l)
         if (l2 .gt. n) go to 145
c
         do 140 i = l2, n
  140    d(i) = d(i) - h
c
  145    f = f + h
c     .......... ql transformation ..........
         p = d(m)
         c = 1.0d0
         c2 = c
         el1 = e(l1)
         s = 0.0d0
         mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
         do 200 ii = 1, mml
            c3 = c2
            c2 = c
            s2 = s
            i = m - ii
            g = c * e(i)
            h = c * p
            r = pythag(p,e(i))
            e(i+1) = s * r
            s = e(i) / r
            c = p / r
            p = c * d(i) - s * g
            d(i+1) = h + s * (c * g + s * d(i))
  200    continue
c
         p = -s * s2 * c3 * el1 * e(l) / dl1
         e(l) = s * p
         d(l) = c * p
         tst2 = tst1 + dabs(e(l))
         if (tst2 .gt. tst1) go to 130
  210    p = d(l) + f
c     .......... order eigenvalues ..........
         if (l .eq. 1) go to 250
c     .......... for i=l step -1 until 2 do -- ..........
         do 230 ii = 2, l
            i = l + 2 - ii
            if (p .ge. d(i-1)) go to 270
            d(i) = d(i-1)
  230    continue
c
  250    i = 1
  270    d(i) = p
  290 continue
c
      go to 1001
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 1000 ierr = l
 1001 return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      subroutine tql2(nm,n,d,e,z,ierr)
c
      integer i,j,k,l,m,n,ii,l1,l2,nm,mml,ierr
      double precision d(n),e(n),z(nm,n)
      double precision c,c2,c3,dl1,el1,f,g,h,p,r,s,s2,tst1,tst2,pythag
c
c     this subroutine is a translation of the algol procedure tql2,
c     num. math. 11, 293-306(1968) by bowdler, martin, reinsch, and
c     wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 227-240(1971).
c
c     this subroutine finds the eigenvalues and eigenvectors
c     of a symmetric tridiagonal matrix by the ql method.
c     the eigenvectors of a full symmetric matrix can also
c     be found if  tred2  has been used to reduce this
c     full matrix to tridiagonal form.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c        z contains the transformation matrix produced in the
c          reduction by  tred2, if performed.  if the eigenvectors
c          of the tridiagonal matrix are desired, z must contain
c          the identity matrix.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct but
c          unordered for indices 1,2,...,ierr-1.
c
c        e has been destroyed.
c
c        z contains orthonormal eigenvectors of the symmetric
c          tridiagonal (or full) matrix.  if an error exit is made,
c          z contains the eigenvectors associated with the stored
c          eigenvalues.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls pythag for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      ierr = 0
      if (n .eq. 1) go to 1001
c
      do 100 i = 2, n
  100 e(i-1) = e(i)
c
      f = 0.0d0
      tst1 = 0.0d0
      e(n) = 0.0d0
c
      do 240 l = 1, n
         j = 0
         h = dabs(d(l)) + dabs(e(l))
         if (tst1 .lt. h) tst1 = h
c     .......... look for small sub-diagonal element ..........
         do 110 m = l, n
            tst2 = tst1 + dabs(e(m))
            if (tst2 .eq. tst1) go to 120
c     .......... e(n) is always zero, so there is no exit
c                through the bottom of the loop ..........
  110    continue
c
  120    if (m .eq. l) go to 220
  130    if (j .eq. 30) go to 1000
         j = j + 1
c     .......... form shift ..........
         l1 = l + 1
         l2 = l1 + 1
         g = d(l)
         p = (d(l1) - g) / (2.0d0 * e(l))
         r = pythag(p,1.0d0)
         d(l) = e(l) / (p + dsign(r,p))
         d(l1) = e(l) * (p + dsign(r,p))
         dl1 = d(l1)
         h = g - d(l)
         if (l2 .gt. n) go to 145
c
         do 140 i = l2, n
  140    d(i) = d(i) - h
c
  145    f = f + h
c     .......... ql transformation ..........
         p = d(m)
         c = 1.0d0
         c2 = c
         el1 = e(l1)
         s = 0.0d0
         mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
         do 200 ii = 1, mml
            c3 = c2
            c2 = c
            s2 = s
            i = m - ii
            g = c * e(i)
            h = c * p
            r = pythag(p,e(i))
            e(i+1) = s * r
            s = e(i) / r
            c = p / r
            p = c * d(i) - s * g
            d(i+1) = h + s * (c * g + s * d(i))
c     .......... form vector ..........
            do 180 k = 1, n
               h = z(k,i+1)
               z(k,i+1) = s * z(k,i) + c * h
               z(k,i) = c * z(k,i) - s * h
  180       continue
c
  200    continue
c
         p = -s * s2 * c3 * el1 * e(l) / dl1
         e(l) = s * p
         d(l) = c * p
         tst2 = tst1 + dabs(e(l))
         if (tst2 .gt. tst1) go to 130
  220    d(l) = d(l) + f
  240 continue
c     .......... order eigenvalues and eigenvectors ..........
      do 300 ii = 2, n
         i = ii - 1
         k = i
         p = d(i)
c
         do 260 j = ii, n
            if (d(j) .ge. p) go to 260
            k = j
            p = d(j)
  260    continue
c
         if (k .eq. i) go to 300
         d(k) = d(i)
         d(i) = p
c
         do 280 j = 1, n
            p = z(j,i)
            z(j,i) = z(j,k)
            z(j,k) = p
  280    continue
c
  300 continue
c
      go to 1001
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 1000 ierr = l
 1001 return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



      subroutine tred1(nm,n,a,d,e,e2)
c
      integer i,j,k,l,n,ii,nm,jp1
      double precision a(nm,n),d(n),e(n),e2(n)
      double precision f,g,h,scale
c
c     this subroutine is a translation of the algol procedure tred1,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix
c     to a symmetric tridiagonal matrix using
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        a contains information about the orthogonal trans-
c          formations used in the reduction in its strict lower
c          triangle.  the full upper triangle of a is unaltered.
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        e2 contains the squares of the corresponding elements of e.
c          e2 may coincide with e if the squares are not needed.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      do 100 i = 1, n
         d(i) = a(n,i)
         a(n,i) = a(i,i)
  100 continue
c     .......... for i=n step -1 until 1 do -- ..........
      do 300 ii = 1, n
         i = n + 1 - ii
         l = i - 1
         h = 0.0d0
         scale = 0.0d0
         if (l .lt. 1) go to 130
c     .......... scale row (algol tol then not needed) ..........
         do 120 k = 1, l
  120    scale = scale + dabs(d(k))
c
         if (scale .ne. 0.0d0) go to 140
c
         do 125 j = 1, l
            d(j) = a(l,j)
            a(l,j) = a(i,j)
            a(i,j) = 0.0d0
  125    continue
c
  130    e(i) = 0.0d0
         e2(i) = 0.0d0
         go to 300
c
  140    do 150 k = 1, l
            d(k) = d(k) / scale
            h = h + d(k) * d(k)
  150    continue
c
         e2(i) = scale * scale * h
         f = d(l)
         g = -dsign(dsqrt(h),f)
         e(i) = scale * g
         h = h - f * g
         d(l) = f - g
         if (l .eq. 1) go to 285
c     .......... form a*u ..........
         do 170 j = 1, l
  170    e(j) = 0.0d0
c
         do 240 j = 1, l
            f = d(j)
            g = e(j) + a(j,j) * f
            jp1 = j + 1
            if (l .lt. jp1) go to 220
c
            do 200 k = jp1, l
               g = g + a(k,j) * d(k)
               e(k) = e(k) + a(k,j) * f
  200       continue
c
  220       e(j) = g
  240    continue
c     .......... form p ..........
         f = 0.0d0
c
         do 245 j = 1, l
            e(j) = e(j) / h
            f = f + e(j) * d(j)
  245    continue
c
         h = f / (h + h)
c     .......... form q ..........
         do 250 j = 1, l
  250    e(j) = e(j) - h * d(j)
c     .......... form reduced a ..........
         do 280 j = 1, l
            f = d(j)
            g = e(j)
c
            do 260 k = j, l
  260       a(k,j) = a(k,j) - f * e(k) - g * d(k)
c
  280    continue
c
  285    do 290 j = 1, l
            f = d(j)
            d(j) = a(l,j)
            a(l,j) = a(i,j)
            a(i,j) = f * scale
  290    continue
c
  300 continue
c
      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



      subroutine tred2(nm,n,a,d,e,z)
c
      integer i,j,k,l,n,ii,nm,jp1
      double precision a(nm,n),d(n),e(n),z(nm,n)
      double precision f,g,h,hh,scale
c
c     this subroutine is a translation of the algol procedure tred2,
c     num. math. 11, 181-195(1968) by martin, reinsch, and wilkinson.
c     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
c
c     this subroutine reduces a real symmetric matrix to a
c     symmetric tridiagonal matrix using and accumulating
c     orthogonal similarity transformations.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        a contains the real symmetric input matrix.  only the
c          lower triangle of the matrix need be supplied.
c
c     on output
c
c        d contains the diagonal elements of the tridiagonal matrix.
c
c        e contains the subdiagonal elements of the tridiagonal
c          matrix in its last n-1 positions.  e(1) is set to zero.
c
c        z contains the orthogonal transformation matrix
c          produced in the reduction.
c
c        a and z may coincide.  if distinct, a is unaltered.
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     ------------------------------------------------------------------
c
      do 100 i = 1, n
c
         do 80 j = i, n
   80    z(j,i) = a(j,i)
c
         d(i) = a(n,i)
  100 continue
c
      if (n .eq. 1) go to 510
c     .......... for i=n step -1 until 2 do -- ..........
      do 300 ii = 2, n
         i = n + 2 - ii
         l = i - 1
         h = 0.0d0
         scale = 0.0d0
         if (l .lt. 2) go to 130
c     .......... scale row (algol tol then not needed) ..........
         do 120 k = 1, l
  120    scale = scale + dabs(d(k))
c
         if (scale .ne. 0.0d0) go to 140
  130    e(i) = d(l)
c
         do 135 j = 1, l
            d(j) = z(l,j)
            z(i,j) = 0.0d0
            z(j,i) = 0.0d0
  135    continue
c
         go to 290
c
  140    do 150 k = 1, l
            d(k) = d(k) / scale
            h = h + d(k) * d(k)
  150    continue
c
         f = d(l)
         g = -dsign(dsqrt(h),f)
         e(i) = scale * g
         h = h - f * g
         d(l) = f - g
c     .......... form a*u ..........
         do 170 j = 1, l
  170    e(j) = 0.0d0
c
         do 240 j = 1, l
            f = d(j)
            z(j,i) = f
            g = e(j) + z(j,j) * f
            jp1 = j + 1
            if (l .lt. jp1) go to 220
c
            do 200 k = jp1, l
               g = g + z(k,j) * d(k)
               e(k) = e(k) + z(k,j) * f
  200       continue
c
  220       e(j) = g
  240    continue
c     .......... form p ..........
         f = 0.0d0
c
         do 245 j = 1, l
            e(j) = e(j) / h
            f = f + e(j) * d(j)
  245    continue
c
         hh = f / (h + h)
c     .......... form q ..........
         do 250 j = 1, l
  250    e(j) = e(j) - hh * d(j)
c     .......... form reduced a ..........
         do 280 j = 1, l
            f = d(j)
            g = e(j)
c
            do 260 k = j, l
  260       z(k,j) = z(k,j) - f * e(k) - g * d(k)
c
            d(j) = z(l,j)
            z(i,j) = 0.0d0
  280    continue
c
  290    d(i) = h
  300 continue
c     .......... accumulation of transformation matrices ..........
      do 500 i = 2, n
         l = i - 1
         z(n,l) = z(l,l)
         z(l,l) = 1.0d0
         h = d(i)
         if (h .eq. 0.0d0) go to 380
c
         do 330 k = 1, l
  330    d(k) = z(k,i) / h
c
         do 360 j = 1, l
            g = 0.0d0
c
            do 340 k = 1, l
  340       g = g + z(k,i) * z(k,j)
c
            do 360 k = 1, l
               z(k,j) = z(k,j) - g * d(k)
  360    continue
c
  380    do 400 k = 1, l
  400    z(k,i) = 0.0d0
c
  500 continue
c
  510 do 520 i = 1, n
         d(i) = z(n,i)
         z(n,i) = 0.0d0
  520 continue
c
      z(n,n) = 1.0d0
      e(1) = 0.0d0
      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



      double precision function pythag(a,b)
      double precision a,b
c
c     finds dsqrt(a**2+b**2) without overflow or destructive underflow
c
      double precision p,r,s,t,u
      p = dmax1(dabs(a),dabs(b))
      if (p .eq. 0.0d0) go to 20
      r = (dmin1(dabs(a),dabs(b))/p)**2
   10 continue
         t = 4.0d0 + r
         if (t .eq. 4.0d0) go to 20
         s = r/t
         u = 1.0d0 + 2.0d0*s
         p = u*p
         r = (s/u)**2 * r
      go to 10
   20 pythag = p
      return
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        Double precision FUNCTION energy(N,freq,X,NN)

        INTEGER No
        PARAMETER (No=100)        
        Integer I, J, K, L, N, NN(No)
        Double precision eng, zpe
        Double precision freq(No), X(No,No)

        eng=0.0d0
        zpe=0.0d0
        DO I=1, N
                eng=eng + (NN(I)+0.5d0)*freq(I)
                zpe=zpe + 0.5d0*freq(I)
                DO J=1, I
                        eng=eng + (NN(I)+0.5d0)*(NN(J)+0.5d0)*X(I,J)
                        zpe=zpe + 0.25d0*X(I,J)
                ENDDO
        ENDDO   
        energy = eng - zpe
                
        IF(energy.LT.0.0d0) THEN
        IF(energy.GE.(-1.0d-5)) energy=0.0d0
        ENDIF
        
        RETURN
        END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        Double precision FUNCTION energyxyz(N,freq,X,Y,Z,NN)

        INTEGER No
        PARAMETER (No=100)        
        Integer I, J, K, L, N, NN(No)
        Double precision eng, zpe
        Double precision freq(No), X(No,No)
        Double precision Y(No,No,No), Z(No,No,No,No) 

        eng=0.0D0
        zpe=0.0d0
        DO I=1, N
                eng=eng + (NN(I)+0.5d0)*freq(I)
                zpe=zpe + 0.5d0*freq(I)
                DO J=1, I
                        eng=eng + (NN(I)+0.5d0)*(NN(J)+0.5d0)*X(I,J)
                        zpe=zpe + 0.25d0*X(I,J)
                DO K=1, J
                        eng=eng+(NN(I)+0.5d0)*
     &                  (NN(J)+0.5d0)*(NN(K)+0.5d0)*Y(I,J,K)
                        zpe=zpe + 0.125d0*Y(I,J,K)
                DO L=1, K 
                        eng=eng+(NN(I)+0.5d0)*(NN(J)+0.5d0)*
     &                          (NN(K)+0.5d0)*(NN(L)+0.5d0)*Z(I,J,K,L)
                        zpe=zpe + 0.0625d0*Z(I,J,K,L)
                ENDDO
                ENDDO
                ENDDO
        ENDDO   
        energyxyz = eng - zpe
                
        IF(energyxyz.LT.0.0d0) THEN
        IF(energyxyz.GE.(-1.0d-4)) energyxyz=0.0d0
        ENDIF
        
        RETURN
        END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



	SUBROUTINE GHRLEV(EV,NN,Nmax,Emax,B,NCV,NCF,CV,CF,
     &		Nsiv,Nsif,Vhr,Bhr,Phav,Phab)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c GHRLEV: a code of general purposes to calculate eigenvalues of 1D-hindered internal rotation 
c Copyright (C) 2009 Lam T. Nguyen
c
c	DATE:	Feb. 13, 2009
c
c Lam T. Nguyen
c NGUYENLT@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c
c or contact
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C
C	INPUT:	
C		1) Emax		: maximum energy in Master Equation
C		2) B		: average rotational constant, defined as:
C			B = (Integral of B(x)dx from 0 to 2*PI) / (2*PI)
C		3) NCV, NCF	: number of elements in vectors of CV and CF, respectively
C		4) CV		: vector of coefficients with NCV elements in torsional 
C			potential energy function, which is expressed as = CVo/2 + Sum of CVi*(1-cos(i*x*Nsig))/2
C		5) CF		: vector of coefficients with NCF elements in rotational constant,
C			which is given by a Fourier series = CFo + Sum of CFi*cos(i*x*Nsig)
C		6) Nsiv, Nsif	: symmetry numbers for tortional potential E function and rotational constant, respectively.
C		7) NN		: maximum number of elements in vector EV
C		8) Vhr		: model for torsional potential energy function
C		9) Bhr		: model for rotational constant
C		10) Phav, Phab	: phases of torsional angles in Vhr and Bhr, respectively.  	 
C
C	OUTPUT:	1) EV(Nmax)	: vector of eigenvalues with Nmax elements
C		2) Nmax		: number of elements of vector EV
C
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


      IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
      PARAMETER (NX=201)
C
C.....Declare arrays.
      DIMENSION EV(NN), CV(NCV), CF(NCF)
        CHARACTER Vhr*5, Bhr*5
C
C.....CALVo used to find Vmax and Vmin of torsional potential energy function
	CALL CALVo(Vo,Vm,CV,NCV,Nsiv,Vhr,Phav)

	CALL ODQHR(EV,NN,NCV,NCF,CV,CF,Nsiv,Nsif,Vhr,Bhr,Phav,Phab,Vm)
	Nmax=NX
	CALL CALEI(EV,NN,Nmax,Emax,B)

        write(*,9991) Vo, Vm, EV(1)
9991    FORMAT(11X,'Torsional potential: ',1X,'Vmax (cm-1) = ',F8.1,1X,
     &  '; Vmin (cm-1) = ',F8.1,1X,'; zpe (cm-1) = ',F5.1)


	RETURN 
      END


C****************************************************************************************************************
C	NAME:	Subroutine ODQHR 
C	USE:	To obtain a vector of quantum eigenvalues of 1D-hindered internal rotor, in which  
C		both torsional potential energy function and moment inertia are explicitly treated as 
C		functions of internal rotational angle, i.e. non-rigid rotor, based on Meyer's algorithm:
C		R. Meyer J. Chem. Phys. 52 (1970) 2053-2059.  
C
C	AUTHORS:	Lam T. Nguyen and John R. Barker 
C
C	DATE:	Jan. 26, 2009
C
C****************************************************************************************************************
C     
C	 NX  : Number of grid points
C        XA  : Position on a grid.
C        AR  : Hamiltonian Matrix, i.e. Meyer's symmetrical matrix
C        ER  : Vector of eigenvalues 
C        ZR  : Eigenvectors (X,Y); where
C                                      X : Wavefunction.
C                                      Y : Energy level.       
C        NPR      : 0 for eigenvalue only and 1 for both eigenvalue and eigenvector 
C        RMIN     : Starting point of grid
C        RMAX     : End point of grid
C        ZL       : Grid length
C        DX       : Grid spacings
C
C****************************************************************************************************************
C
C	INPUT:	1) NCV, NCF	= Number of coefficients derived from fitting rotational energy profile and moment inertia
C		2) CV(NCV)	= Vector of coefficients from rotational energy profile, with NCV elements
C		3) CF(NCF)	= Vector of coefficients from moment inertia (rotational constant), with NCF elements
C		4) Nsig		= rotational symmetry number
C	OUTPUT:	1) ER 		= Vector of eigenvalues of 1D-hindered internal rotor, with NX=201 elements 
C
C*****************************************************************************************************************

	SUBROUTINE ODQHR(ER,NN,NCV,NCF,CV,CF,Nsiv,Nsif,
     &	Vhr,Bhr,Phav,Phab,Vm)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
      PARAMETER (NX=201)
C
C.....Declare arrays.
      DIMENSION AR(NX,NX),ER(NN),ZR(NX,NX),XA(NX) 
      DIMENSION D(NX,NX), F(NX), V(NX), DT(NX,NX),
     & T(NX,NX), FT(NX,NX)
	DIMENSION  CF(NCF), CV(NCV), FV1(NX), FV2(NX)
        CHARACTER Vhr*5, Bhr*5

C.....	DIMENSION WORK(NX*3)

C
C.....Variable data input.

      DATA NPR/0/

C.....Now compute Hamiltonian matrix:

	CALL CMM(AR,NCV,NCF,CV,CF,Nsiv,Nsif,Vhr,Bhr,Phav,Phab,Vm)
					
C.....Now call eigenvalue solver using RS subroutine of EISPACK.

	CALL rs(NX,NX,AR,ER,NPR,ZR,FV1,FV2,IERR) 

C...... Alterative option: using subroutine DSYEV from LAPACK 
C......	CALL DSYEV( 'V', 'U', NX, AR, NX, ER, WORK, NX*3, IERR )
C......

	RETURN 
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C.....	CMM stands for Constructing Meyer's Matrix, AR, for 1D-hindered internal rotor using Meyer's method, 
C.....	R. MEYER J. Chem. Phys. 52 (1970) 2053.
C.....	Both rotational energy profile and moment inertia are properly treated as functions of 
C.....	internal rotation angle, i.e. non-rigid rotor
C.....	INPUT:	1) NCF, NCV = Number of coefficients drived from fitting rotational energy profile and moment inertia
C.....		2) CV(NCV) = Vector of coefficients from rotational energy profile, with NCV elements
C.....		3) CF(NCF) = Vector of coefficients from moment inertia, with NCF elements
C.....		4) Nsig = Rotational symmetry number, it can be either one or two or three or whatever 
C.....	OUTPUT: 1) AR(NX,NX) = Meyer's symmetrical matrix, with a size of NX--number of grid points   


	SUBROUTINE CMM(AR,NCV,NCF,CV,CF,Nsiv,Nsif,Vhr,Bhr,Phav,Phab,Vm)
	IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
	PARAMETER (NX=201)
C
C.....Declare arrays.
	DIMENSION AR(NX,NX),XA(NX) 
	DIMENSION D(NX,NX), F(NX), V(NX), DT(NX,NX),
     & T(NX,NX), FT(NX,NX)
	DIMENSION  CF(NCF), CV(NCV)
	CHARACTER Vhr*5, Bhr*5

C
C.....Variable data input.

	PI=acos(-1.D0)
C
C.....Set up grid
	RMIN=0.0D0
	RMAX=PI*2
	ZL=(RMAX-RMIN)
	DX=ZL/NX

C.....Now compute Meyer's symmetrical matrix:

	DO I=1, NX
		XA(I)=RMIN+DX*I
		CALL VX(V(I),XA(I)+Phav,CV,NCV,Nsiv,Vhr,Vm)
		CALL FX(F(I),XA(I)+Phab,CF,NCF,Nsif,Bhr)
		DO J=1, NX
			ImJ=I-J
			IF(ImJ.EQ.0) THEN
				D(I,J)=0.0D0
			ELSE
				tp1=PI*ImJ/dble(NX)
				tp2=(-1.0D0)**ImJ
				D(I,J)=tp2/2.0D0/dsin(tp1)
			ENDIF
			DT(I,J)=-D(I,J)
			FT(I,J)=F(I)*DELTA(I,J)
		ENDDO
	ENDDO

	DO I=1, NX
		DO J=1, NX
			SU=0.0D0
			DO K=1, NX
				SU=SU + FT(I,K)*D(K,J)
			ENDDO
			T(I,J)=SU
		ENDDO
	ENDDO

	DO I=1, NX
		DO J=1, NX
			SU=V(I)*DELTA(I,J)
			DO K=1, NX
				SU=SU + DT(I,K)*T(K,J)
			ENDDO
			AR(I,J)=SU
		ENDDO
	ENDDO	

	RETURN 
      END
C..........
C..........
	SUBROUTINE VX(V,X,CV,N,Nsig,Vhr,Vm)
	IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
	DIMENSION CV(N)
	CHARACTER Vhr*5

	IF((Vhr.EQ.'Vhrd1').OR.(Vhr.EQ.'VHRD1')) THEN
		V=0.0d0
		DO I=1, N
			V=V+CV(I)*(1.0d0-dcos(X*Nsig*I))/2.0d0
		ENDDO
	ELSEIF((Vhr.EQ.'Vhrd2').OR.(Vhr.EQ.'VHRD2')) THEN
		V=CV(1)
		DO I=2, N
			V=V+CV(I)*dcos(X*Nsig*(I-1))
		ENDDO
        ELSEIF((Vhr.EQ.'Vhrd3').OR.(Vhr.EQ.'VHRD3')) THEN
		V=CV(1)
		Nc=(N+1)/2
		DO I=2, N
			IF(I.LE.Nc) THEN
				V=V+CV(I)*dcos(X*Nsig*(I-1))
			ELSE
				V=V+CV(I)*dsin(X*Nsig*(I-Nc))
			ENDIF
		ENDDO
	ELSE
		write(*,*) "ERROR at INPUT for torsional function = Vhrd"
		STOP
	ENDIF

	V=V-Vm

	RETURN
	END
C..........
C..........
	SUBROUTINE FX(F,X,CF,N,Nsig,Bhr)
	IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
	DIMENSION CF(N)
	PARAMETER (CON=16.85763D0)
	CHARACTER Bhr*5

	IF((Bhr.EQ.'Bhrd1').OR.(Bhr.EQ.'BHRD1')) THEN
		F=CF(1)
		DO I=2, N
			F=F+CF(I)*dcos(X*Nsig*(I-1))
		ENDDO
        ELSEIF((Bhr.EQ.'Ihrd1').OR.(Bhr.EQ.'IHRD1')) THEN
                F=CF(1)
                DO I=2, N
                        F=F+CF(I)*dcos(X*Nsig*(I-1))
                ENDDO
		F=CON/F
	ELSE
                write(*,*) "ERROR at INPUT for Bhrd or Ihrd"
		STOP
	ENDIF

	RETURN
	END
C.........
C.........      
	Double Precision Function DELTA(I,J)
	INTEGER I, J

	IF(I.EQ.J) THEN
		DELTA=1.0D0
	ELSE
		DELTA=0.0D0
	ENDIF
	
	RETURN
	END
C.........
C.........

CCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE CALEI(EV,NN,Nmax,Emax,B)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c CALEI: a code to compute/select eigenvalues of torsional motion based on Emax 
c Copyright (C) 2009 Lam T. Nguyen and John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C
C	INPUT:	1) EV(Nmax)	: a vector of eigenvalues with Nmax elements
C		2) Nmax		: number of elements of vector EV
C		3) Emax		: maximum energy in Master Equation
C		4) B		: average rotational constant, defined as:
C			B = (Integral of B(x)dx from 0 to 2*PI) / (2*PI)
C	OUTPUT:	1) EV(Nmax)
C		2) Nmax	
C
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
      DIMENSION EV(NN)

	I=Nmax

C.......IF The largest eigenvalue is larger than the maximum energy, 
C.......THEN searching for an eigenvalue, which is <= Emax
11	E=EV(I)
	IF(E.GT.Emax) THEN
		I=I-1
		goto 11
	ENDIF
	Nmax=I

C.......Looking for higher-lying eigenvalues with double degeneracy
111	NE1=NINT(EV(I))
	NE2=NINT(EV(I-1))
	IF(NE1.GT.NE2) THEN
		I=I-1
		goto 111
	ENDIF

C.......Switching from a hindered rotor to a Pitzer rotor
C.......Pitzer rotor is defined as E = Eo + B*(JP**2)
C.......Two variables Eo and JP will be computed as below
	tp=EV(I-1) - EV(I-2)
	JP=NINT((tp/B - 1.0d0)/2.0d0)
	Eo=EV(I-2)-B*JP*JP

1111	EV(I+1)=Eo + B*(JP+2)*(JP+2)
	EV(I+2)=EV(I+1)
	IF(EV(I+2).LE.Emax) THEN
		I=I+2
		JP=JP+1
		goto 1111
	ENDIF
	Nmax=I

	RETURN
 
      END

CCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCC
C
C CALVo is used to compute Vmax and Vmin 
C of the torsional potential energy function 
C
        SUBROUTINE CALVo(Vo,Vm,CV,NCV,Nsiv,Vhr,Pha)

        IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
        DIMENSION CV(NCV)
        CHARACTER Vhr*5

        PI=acos(-1.0d0)

        RMIN=0.0D0
        RMAX=PI*2
        ZL=(RMAX-RMIN)
        DX=ZL/3600.0d0
        CALL VX(Vo,0.0d0+Pha,CV,NCV,Nsiv,Vhr,0.0d0)
        Vm=Vo
        DO I=1, 3600
                XA = RMIN + DX*I
                CALL VX(Vtp,XA+Pha,CV,NCV,Nsiv,Vhr,0.0d0)
                IF(Vtp.GT.Vo) Vo=Vtp
                IF(Vtp.LT.Vm) Vm=Vtp
        ENDDO
        RETURN
        END

CCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCC









      SUBROUTINE KROTLEV(T,AT,DELE,JMAX,B,JK,NSYMM)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c DenSum: a code for calculating sums and densities of states.
c Copyright (C) 2007 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C
C	CALCULATES ENERGY LEVELS FOR 1-DIMENSIONAL K-ROTOR TO BE USED
C          WITH STERAB
C
c	DELE	= energy grain size
c	JMAX	= index of Emax
c	B	= rotational constant (cm-1)
c	JK	= quantum number for total angular momentum
c	IR	= vector of energy level indices
c	NMAX	= ceiling of array
c
c
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION IR(20001)
	DIMENSION T(JMAX), AT(JMAX)
      SAVE 


	Emax=(JMAX-1)*DELE 
      RMAX = SQRT(Emax/B)          ! Highest rotational quantum number consistent with energy ceiling
      NMAX = INT(RMAX)             ! Integer version
      IF ( NMAX .GT. JK ) NMAX = JK
      IF ( NMAX .GT. 20001 ) NMAX = 20001
       
      IF ( NMAX .GT. 0 ) THEN
      DO 100 K = 1 , NMAX               ! K = quantum number of 1-dimensional free rotor
         R = B*K*K/DELE                 ! index number starting at 0
         IR(K) = NINT(R)              ! Nearest integer: number of grains
 100  CONTINUE

CCCCCCCCCCCCCCCCCCCCCCC

            DO 281 J = 1 , NMAX ! J = quantum number for K-rotor;  NMAX is the number of energy levels
               IF ( J.EQ.0 ) THEN
                  F = 1         ! F is the multiplicity of the rotational level (see J.L. McHale, Molecular Spectroscopy (Prentice-Hall, 1999), 216f.
               ELSE
                  F = 2.0       ! 1-D rotor (e.g. single-axis internal rotor)
               ENDIF
               DO 261 K = 1 , JMAX
                  KARG = K - IR(J)
                  IF ( KARG.GT.0 .AND. KARG.LE.JMAX)
     &                   AT(K) = AT(K) + F*T(KARG)
 261           CONTINUE
 281        CONTINUE

            DO 301 J = 1 , JMAX
               T(J) = AT(J) + T(J)
               T(J) = T(J) / NSYMM
               AT(J) = 0.0D+00
 301        CONTINUE

CCCCCCCCCCCCCCCCCCCCCC

      ENDIF
 
      RETURN
      END
c
      function lenstr(string)
c
      implicit      none
c
      character*(*) string,  !input string
     2              null*1   !null character
c
      integer*4     i,       !string index
     2              iend,    !candidate for string length
     3              j,       !index used for stepping through string
     4              length,  !defined length of input
     5              lenstr   !length of string
c
      length=len(string)
c
c     Look for the existence of nulls; if there
c     are any, the first one marks one character past the
c     end of the defined string. If there are no nulls,
c     pick up the final nonspace character, and call that
c     the defined length of the string.
c
      null=char(0)
c
c     Look for nulls
c
      iend=index(string(1:length),null)
c
c     No nulls, look for the location of the last nonspace.
c
      if(iend.eq.0)then
          do i=1,length
              j=length+1-i
              if(string(j:j).ne.' ')then
                  lenstr=j
                  return
              endif
          enddo
c
c         all spaces
c
          lenstr=0
          return
      else
c
c         first null-1
c
          lenstr=iend-1
          return
      endif
      end
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: John R. Barker and Thanh Lam Nguyen
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       INTEGER FUNCTION nvmax( ns, w, x, nv, k, Eu )

c     Select highest quantum number to be sampled

      IMPLICIT NONE
      INTEGER ns, index(100), nv(100), i, k, nvd
      DOUBLE PRECISION r(100), w(100), x(100,100), Eu, D , vd
      DOUBLE PRECISION vmax, sum, energy, enrg, zpe, tp
      SAVE

      IF ( Eu .LT. 0.0d+00 ) THEN
         nvmax = -1
         RETURN
      ENDIF

      IF ( x(k,k) .NE. 0.0 ) THEN
         sum = 0.0d+00
         DO i = 1, ns
           IF ( i .NE. k ) sum = sum + ( nv(i) + 0.5d0 )*x(i,k)
         END DO
        zpe = x(k,k)/4.0d0 + w(k)/2.0d0 + sum/2.0d0

         vd = -(w(k) + sum)/( 2.0d+00*x(k,k) ) - 0.5d0             ! Eq. 15: quantum no. of highest bound energy level
         D = -( ( w(k) + sum )**2)/( 4.0d+00*x(k,k) ) - zpe       ! Do (not De)


	tp = 1.0d+00 - Eu/D

	IF(tp.LT.0.0d0) THEN
		vmax = vd
	ELSE IF(D.NE.0.0d0) THEN
         vmax = vd*( 1.0d+00 - SQRT( 1.0d+00 - Eu/D) )      ! Eq. 16
	ELSE
		vmax = vd
	ENDIF

         IF ( (w(k) + sum) .GT. 0.0 .AND. x(k,k) .LT. 0.0) THEN
           IF ( Eu .GT. D ) THEN
             nvmax = INT( vd )
           ELSE
             nvmax = INT( vmax )
           ENDIF
         ELSEIF ( (w(k) + sum) .GT. 0.0 .AND. x(k,k) .GT. 0.0) THEN
           nvmax = INT( vmax )
         ELSE                                ! (w(k) + sum) < 0
           nvmax = -1
         ENDIF
      ELSEIF ( x(k,k) .EQ. 0.0 ) THEN
           nvmax = INT( Eu/w(k) )                              ! harmonic when x(k,k) = 0
      ENDIF   ! Eu < 0

      RETURN
      END



c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu   
c          Aug. 9, 2009
c       
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      INTEGER FUNCTION nvmaxxyz( ns, w, x, y, z, nv, k, Eu )

c     Select highest quantum number to be sampled

      IMPLICIT NONE
      INTEGER No
      PARAMETER (No=100)
      INTEGER ns, nv(No), i, j, k, nvd
      DOUBLE PRECISION w(No), Eu, D , vd
      DOUBLE PRECISION x(No,No), y(No,No,No), z(No,No,No,No)
      DOUBLE PRECISION energy, ENER, EGS, energyxyz
      DOUBLE PRECISION Fo	

	IF(Eu.GT.0.0d0) THEN
		D=0.0d0 
        	nv(k)=0
        	EGS=energyxyz( ns, w, x, y, z, nv )
11		nv(k)=nv(k)+1
        	ENER=energyxyz( ns, w, x, y, z, nv ) - EGS
		IF(D.LT.ENER) THEN
			D=ENER
			IF(D.LE.Eu) THEN
                       		goto 11
			ELSE
				nv(k)=nv(k)-1
				nvmaxxyz=nv(k)
                	ENDIF
		ELSE
			nv(k)=nv(k)-1
			CALL CALDERIXYZ(Fo,nv(k),ns,k,nv,w,x,y,z)
			IF(Fo.GE.0.0d0) THEN
				nvmaxxyz=nv(k)
			ELSE
				nvmaxxyz=nv(k)-1
			ENDIF		
		ENDIF
	ELSEIF(Eu.EQ.0.0d0) THEN
		nvmaxxyz=0
	ELSE
		nvmaxxyz=-1
	ENDIF

	nv(k)=0

	RETURN
	END		
C==================================


**==ran1.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
      DOUBLE PRECISION FUNCTION RAN1(idum)
C---------------------------------------------------------------------------------
C	"Minimal" random number generator of Park and Miller with Bays-Durham shuffle
C	and added safeguards.  Returns a uniform random deviate between 0.0 and 1.0
C	(exclusive of the endpoint values).  Call with idum a negative integer to
C	initialize; thereafter do not alter idum between successive deviates in a
C	sequence.  RNMX should approximate the largest floating value that is less
C	than 1.
C
C	W. H. Press and S. A. Teukolsky, Computers in Physics, 6(5), 522-4 (1992).
C---------------------------------------------------------------------------------
      IMPLICIT NONE
      INTEGER idum , IA , IM , IQ , IR , NTAB , NDIV
      DOUBLE PRECISION AM , EPS , RNMX , T
      PARAMETER (IA=16807,IM=2147483647,AM=1.0D+00/IM,IQ=127773,IR=2836,
     &        NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=2.3D-16,RNMX=1.0D+00-EPS)
      INTEGER j , k , iv(NTAB) , iy
      SAVE iv , iy
      DATA iv/NTAB*0/ , iy/0/
      IF ( idum.LE.0 .OR. iy.EQ.0 ) THEN
                                        ! Initialize
         idum = max(-idum,1)            ! Be sure to prevent idum=0
         DO 50 j = NTAB + 8 , 1 , -1    ! Load shuffle table after 8 warm-ups
            k = idum/IQ
            idum = IA*(idum-k*IQ) - IR*k
            IF ( idum.LT.0 ) idum = idum + IM
            IF ( j.LE.NTAB ) iv(j) = idum
 50      CONTINUE
         iy = iv(1)
      ENDIF
      k = idum/IQ                       ! Start here when not initializing
      idum = IA*(idum-k*IQ) - IR*k      ! Compute idum=mod(IA*idum,IM) without
      IF ( idum.LT.0 ) idum = idum + IM !    overflows by Schrage's Method
      j = 1 + iy/NDIV                   ! Will be in range 1:NTAB
      iy = iv(j)                        ! Output previously stored value
      iv(j) = idum                           ! Refill shuffle table
      T = AM*iy
      RAN1 = min(T,RNMX)        ! Because users don't expect endpoint values
11	format(d90.50)
c	write(6,11)RAN1
      RETURN
      END
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker and Thanh Lam Nguyen
c
c Authors: Thanh Lam Nguyen and John R. Barker
c          nguyenlt@umich.edu
c          September 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c----------------------------------------------------------------------
c
c      Read vibrational frequencies and anharmonicities
c
c       KIN = input UNIT for READ statements
c       ns  = number of vibrational frequencies
c       WW  = keyword ('we', 'w0', or 'wf') for the three types of frequencies:
c       wa(i): harmonic frequencies with zero of energy at the potential minimum
c          Evib = SUMi( wai*(vi+1/2) ) + SUMi( SUMj( Xij*(vi+1/2)*(vj+1/2) ) )
c       w0(i): frequencies with zero of energy at zpe
c          Evib = SUMi( w0i*vi ) + SUMi( SUMj( Xij*vi*vj ) )
c       wf(i) = fundamental frequencies (for 0-1 transitions with all vj = 0)
c----------------------------------------------------------------------
      SUBROUTINE READ_WXYZ( KIN , ns , NY, NZ, WW ) 
      
      IMPLICIT NONE
      CHARACTER WW*2 , KEYWORD*5
      INTEGER KIN , ns , i , j , k , l
      INTEGER NY , NZ , I1 , J1 , K1 , L1
      DOUBLE PRECISION wa , w0 , wf , xa , ya , za
      DOUBLE PRECISION ave , av0 , avf , zpe , Viblo
      COMMON/wxyz/ wa(100) , w0(100) , wf(100) , xa(100,100) ,
     &   ya(100,100,100) , za(100,100,100,100) , zpe , ave ,
     &   av0 , avf , Viblo
      SAVE

      IF ( (WW .EQ. 'We') .OR. (WW .EQ. 'we') .OR. (WW .EQ. 'WE') ) THEN
        READ(KIN,*)     (wa(i), i=1,ns)
        WW = 'We'
      ELSEIF ( (WW .EQ. 'W0') .OR. (WW .EQ. 'w0') ) THEN
        READ(KIN,*)     (w0(i), i=1,ns)
        WW = 'W0'
      ELSEIF ( (WW.EQ.'Wf') .OR. (WW.EQ.'wf') .OR. (WW.EQ.'WF') ) THEN
        READ(KIN,*)     (wf(i), i=1,ns)
        WW = 'Wf'
      ENDIF
c                                                        X matrix (vibrational anharmonicity)
      READ(KIN,9011) KEYWORD
      IF ( (KEYWORD .EQ. 'UPPER') .OR. (KEYWORD .EQ. 'upper') 
     &       .OR. (KEYWORD .EQ. 'Upper') ) THEN
        DO j = 1, ns
          READ(KIN,*) (xa(j,i), i=j, ns)
          IF ( j .LT. ns) THEN
            DO k = j+1 , ns
              xa(k,j) = xa(j,k)
            END DO
          ENDIF
        ENDDO
      ELSEIF ( (KEYWORD .EQ. 'LOWER') .OR. (KEYWORD .EQ. 'lower') 
     &           .OR. (KEYWORD .EQ. 'Lower') ) THEN
        DO j = 1, ns
          READ(KIN,*) (xa(j,i), i=1,j)
          IF ( j .GT. 1) THEN
            DO k = 1 , j-1
              xa(k,j) = xa(j,k)
            END DO
          ENDIF
        ENDDO
      ENDIF
c                                                        Y matrix (vibrational anharmonicity)
       DO i=1, ns
         DO j=1, ns
             DO k=1, ns
                ya(i,j,k)=0.0d0
             ENDDO
         ENDDO
       ENDDO

        DO i=1, NY
         READ(KIN,*) I1, J1, K1, ya(I1,J1,K1)
         ya(K1,J1,I1)=ya(I1,J1,K1)
         ya(K1,I1,J1)=ya(I1,J1,K1)
         ya(J1,I1,K1)=ya(I1,J1,K1)
         ya(J1,K1,I1)=ya(I1,J1,K1)
         ya(I1,K1,J1)=ya(I1,J1,K1)
        ENDDO 
c                                                        Z matrix (vibrational anharmonicity)
        DO i=1, ns
          DO j=1, ns
            DO k=1, ns    
              DO l=1, ns
                za(i,j,k,l)=0.0d0
              ENDDO
            ENDDO
          ENDDO
        ENDDO

        DO i=1, NZ
         READ(KIN,*) I1, J1, K1, L1, za(I1,J1,K1,L1)
         za(I1,J1,L1,K1)=za(I1,J1,K1,L1)
         za(I1,K1,J1,L1)=za(I1,J1,K1,L1)
         za(I1,K1,L1,J1)=za(I1,J1,K1,L1)
         za(I1,L1,K1,J1)=za(I1,J1,K1,L1)
         za(I1,L1,J1,K1)=za(I1,J1,K1,L1)
         za(J1,I1,K1,L1)=za(I1,J1,K1,L1)
         za(J1,I1,L1,K1)=za(I1,J1,K1,L1)
         za(J1,K1,I1,L1)=za(I1,J1,K1,L1)
         za(J1,K1,L1,I1)=za(I1,J1,K1,L1)
         za(J1,L1,I1,K1)=za(I1,J1,K1,L1)
         za(J1,L1,K1,I1)=za(I1,J1,K1,L1)
         za(K1,I1,J1,L1)=za(I1,J1,K1,L1)
         za(K1,I1,L1,J1)=za(I1,J1,K1,L1)
         za(K1,J1,I1,L1)=za(I1,J1,K1,L1)
         za(K1,J1,L1,I1)=za(I1,J1,K1,L1)
         za(K1,L1,I1,J1)=za(I1,J1,K1,L1)
         za(K1,L1,J1,I1)=za(I1,J1,K1,L1)
         za(L1,I1,J1,K1)=za(I1,J1,K1,L1)
         za(L1,I1,K1,J1)=za(I1,J1,K1,L1)
         za(L1,J1,I1,K1)=za(I1,J1,K1,L1)
         za(L1,J1,K1,I1)=za(I1,J1,K1,L1)
         za(L1,K1,I1,J1)=za(I1,J1,K1,L1)
         za(L1,K1,J1,I1)=za(I1,J1,K1,L1)
        ENDDO

9011  FORMAT(A5)

      RETURN
      END
      
      
**==rotlev.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
      SUBROUTINE ROTLEV(T,AT,DELE,JMAX,B,NDIM,NSYMM)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c DenSum: a code for calculating sums and densities of states.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c    Last modified:
c
c    8/01 BUG FIX: fixed 1-D rotor levels: replaced B*J*(J+1) with B*J*J
c    5/99 by J. R. Barker
CC
C	CALCULATES ENERGY LEVELS FOR FREE ROTOR TO BE USED
C          WITH STERAB
C
c	DELE	= energy grain size
c	JMAX	= index of Emax
c	B	= rotational constant (cm-1)
c	NDIM	= rotor dimension
c	IR	= vector of energy level indices
c	NMAX	= ceiling of array
c
c
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION IR(20001)
	DIMENSION T(JMAX), AT(JMAX)
      SAVE 

	Emax=(JMAX-1)*DELE 
      RMAX = SQRT(Emax/B)  ! Maximum bound level
      NMAX = INT(RMAX)  
      IF ( NMAX.GT.20001 ) NMAX = 20001
 
      DO 100 J = 1 , NMAX
         IF ( NDIM.EQ.1 ) THEN
             R = B*J*J/DELE
           ELSE
             R = B*J*(J+1)/DELE
         ENDIF
         IR(J) = NINT(R)              ! Nearest integer: number of grains
 100  CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCC

            DO 280 J = 1 , NMAX ! J = rot quantum number;  NMAX is the number of energy levels
               IF ( J.EQ.0 ) THEN
                  F = 1         ! F is the multiplicity of the rotational level (see J.L. McHale, Molecular Spectroscopy (Prentice-Hall, 1999), 216f.
               ELSEIF ( NDIM.EQ.1 ) THEN   ! 1-D rotor (e.g. single-axis internal rotor)
                  F = 2.0
               ELSEIF ( NDIM.EQ.2 ) THEN   ! 2-D rotor
                  F = 2.0*J + 1.0
               ELSEIF ( NDIM.EQ.3 ) THEN   ! 3-D rotor (shperical top)
                  F = (2.0*J+1.0)*(2.0*J+1.0)
               ENDIF
               DO 260 K = 1 , JMAX
                  KARG = K - IR(J)
                  IF ( KARG.GT.0 .AND. KARG.LE.JMAX) THEN
                         AT(K) = AT(K) + F*T(KARG)
                  ENDIF
 260           CONTINUE
 280        CONTINUE

           DO 300 J = 1 , JMAX
               T(J) = AT(J) + T(J)
               T(J) = T(J) / NSYMM
               AT(J) = 0.0D+00
 300        CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCC
 
      RETURN
      END
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c SHRLEV: a code to calculate eigenvalues of symmetrical, rigid 1D-hindered internal rotation
c Copyright (C) 2009 Lam T. Nguyen and John R. Barker
c
c	DATE:	Feb. 13, 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        SUBROUTINE SHRLEV(T,AT,DELE,JMAX,B,V,n1,n2,IMAX,zpe,
     &          Vhr,Bhr,Phav,Phab,Nsig)

        IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
        PARAMETER (NN=2000, No=1)
        DIMENSION EV(NN), CV(No), CF(No), T(JMAX), AT(JMAX)
	   DIMENSION IR(NN)
        CHARACTER Vhr*5, Bhr*5

        Emax=(JMAX-1)*DELE

        NCV=1
        NCF=1
        CV(1)=V
        CF(1)=B

        CALL GHRLEV(EV,NN,IMAX,Emax,B,NCV,NCF,CV,CF,n1,n2,
     &          Vhr,Bhr,Phav,Phab)

	zpe=EV(1)

C......	write(*,*) "Zero-point energy = ", zpe

	DO I=1, IMAX
		tp=EV(I)-zpe
		IR(I)=NINT(tp/DELE)
C......		write(*,*) I, tp

	ENDDO 

	DO J=1, IMAX
		LL=IR(J)
		DO L=1, JMAX - LL
			AT(L+LL) = AT(L+LL) + T(L)
		ENDDO
	ENDDO
	DO J=1, JMAX
		T(J)=AT(J)/Nsig
		AT(J)=0.0d0
	ENDDO 

	RETURN
	END


c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c Copyright (C) 2009 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      SUBROUTINE shuffle ( ns, w0, x0, w, x, nx)
c
c	W0 = Inital frequency vector
c	x0 = Initial anharmonicity coefficient matrix
c	w  = shuffled frequency vector
c	x  = shuffled anharmonicity coefficient matrix
c	nx = shuffled vector of initial index numbers

      IMPLICIT NONE
      INTEGER ns, index(100), idum, i, j, k, m, nx(100), nt(100)
      DOUBLE PRECISION r(100), RAN1, rtemp, 
     &                 w(100), x(100,100), w0(100), x0(100,100)
      COMMON/I/ idum
      EXTERNAL RAN1
      SAVE

      DO i = 1, ns
        r(i) = RAN1(idum)
      END DO

      DO j = 1 , ns
        rtemp = r(1)
        k = 1
        DO i = 2, ns
          IF ( r(i) .gt. rtemp ) THEN
            k = i
            rtemp = r(i)
          ENDIF
        END DO
        index(k) = j
        r(k) = 0.0
      END DO

      DO i = 1, ns
        nt(i) = nx( index(i) )
        w(i)  = w0( index(i) )
        DO j = 1, ns
          x(i,j) = x0( index(i),index(j) )
        END DO
      END DO
      
      DO i = 1, ns
        nx(i) = nt(i)
      END DO

      RETURN
      END

       

      
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c UHRLEV: a code to calculate eigenvalues of unsymmetrical, non-rigid 1D-hindered internal rotation
c Copyright (C) 2009 Lam T. Nguyen and John R. Barker
c
c	DATE:	Feb. 13, 2009
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        SUBROUTINE UHRLEV(T,AT,DELE,JMAX,NCB,NCV,CB,CV,n1,n2,IMAX,
     &          zpe,Vhr,Bhr,Phav,Phab,Nsig)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z), INTEGER(I-N)
      PARAMETER (NN=2000)
      DIMENSION EV(NN), CV(NCV), CB(NCB), T(JMAX), AT(JMAX)
	DIMENSION IR(NN)
        CHARACTER Vhr*5, Bhr*5

        PI=acos(-1.0d0)
        Emax=(JMAX-1)*DELE

        IF((Bhr.EQ.'Bhrd2').OR.(Bhr.EQ.'BHRD2')) THEN   ! Average rotational constant
                B=0.0d0
                DO I=1, n2
                        B=B + (CB(I)/I)*((PI*2)**(I-1))
                ENDDO
        ELSEIF((Bhr.EQ.'Ihrd2').OR.(Bhr.EQ.'IHRD2')) THEN       ! Average moment of inertia
                B=0.0d0
                DO I=1, n2
                        B=B + (CB(I)/I)*((PI*2)**(I-1))
                ENDDO
                B=16.85763d0 / B        ! convert from I to B
        ELSEIF((Bhr.EQ.'Ihrd1').OR.(Bhr.EQ.'IHRD1')) THEN       ! Average moment of inertia
                B=16.85763d0/CB(1)      ! convert from I to B
        ELSEIF((Bhr.EQ.'Bhrd1').OR.(Bhr.EQ.'BHRD1')) THEN       ! Average rotational constant
                B=CB(1)
        ELSE
                write(*,*) "ERROR at INPUT for Bhr or Ihr"
        ENDIF

        CALL GHRLEV(EV,NN,IMAX,Emax,B,NCV,NCB,CV,CB,n1,n2,
     &          Vhr,Bhr,Phav,Phab)

	zpe=EV(1)

C......	write(*,*) "Zero-point energy = ", zpe

	DO I=1, IMAX
		tp=EV(I)-zpe
		IR(I)=NINT(tp/DELE)
C......		write(*,*) I, tp
	ENDDO 

	DO J=1, IMAX
		LL=IR(J)
		DO L=1, JMAX - LL
			AT(L+LL) = AT(L+LL) + T(L)
		ENDDO
	ENDDO
	DO J=1, JMAX
		T(J)=AT(J)/Nsig
		AT(J)=0.0d0
	ENDDO 

	RETURN
	END


