      SUBROUTINE BOOK1(Mol,E,Temp,IP,Time,Delt,Igo)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c	Bookkeeping routine, works in concert with Subroutine STEPPER
c
c	Mol	= index of molecule at time=Time
c	E	= vibrational energy (relative to ZPE of Mol) at t=Time
c	Temp	= translational temperature
c	IP	= index of selected process = rxn channel number,
c			or Nchan+1 if collision
c	Time	= time
c	Delt	= time step
c	Igo	= flag which signals the following:
c		= 0	stop stepping for this trial
c		= 1	continue stepping for this trial
c
c***********************************************************************
c
c Date   Version  Revisions
c
c 08/02  v1.2.3  -Bug Fix: in subroutine book1.f, fixed bug that gave 
c                 erroneously large yields wen the time scale was much shorter
c                 than the inverse of the sum of the rates (k(E) + collision
c                 frequency).
c 05/02  v1.2.2  -Revised averaging of kuni so that only one sample per time bin per
c                 trial is included in the average. Previous algorithm resulted in
c                 over-weighting faster k(E)'s and gave erroneously high average kuni
c                 reported in multiwell.out.
c
c***********************************************************************

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
      INTEGER Mol , IP , Igo , ifin , i , Last , N
      INTEGER ifind , Dlast , k , istart 
     &      , ktime , kstep , klast , kfin
      DOUBLE PRECISION E , Temp , Time , Delt , COLSTEP , Step , 
     &                 DistStep
      SAVE 

      kstep = Ntime                         ! number of time steps for collision sampling

      DistStep = Emax2/(Ndist-1)                ! Energy step for binned distributions
      N = 1 + E/DistStep                        ! Bin energy index for binned distributions
      IF ( N.GT.Ndist ) N = Ndist

c
c	First, record the current step
c
      IF ( Time.EQ.0.0D+00 ) THEN               ! First step of a trial
         Last = 1                               ! index of final bin for last step
         Dlast = 1                              ! index of final bin for vib distribution
         klast = 1
         Num(Mol,1) = Num(Mol,1) + 1.d+00       ! Number of times 1st time bin accessed (for averages)
         Mum(Mol,1) = Mum(Mol,1) + 1.d+00       ! Number of times 1st time bin accessed (for EDIST)
         EMol(Mol,1) = EMol(Mol,1) + E          ! For calculating average vibrational energy
         EDIST(Mol,N,1) = EDIST(Mol,N,1) + 1.d+00   ! For calculating vibrational distribution
         Nuni(Mol,1) = Nuni(Mol,1) + 1.0D+00
         DO 15 k = 1 , Nchan(Mol)                   ! Rab(k)= k(E) for kth channel calculated by STEPPER
            kuni(Mol,k,1) = Rab(k) + kuni(Mol,k,1)  ! average rate constant
 15      CONTINUE
      ENDIF

      ifin = 1 + ((Time+Delt)/Tlim)*(Ntime-1)   ! index of final time-bin for this step
      IF ( ifin.GE.Ntime ) THEN
         ifin = Ntime
         Igo = 0                                ! Stop stepping
      ENDIF
 
      ifind = 1 + (Time+Delt)*(Mtime-1)/Tlim    ! Vib distribution final time-bin index
      IF ( ifind.GT.Mtime ) ifind = Mtime
 
      IF ( Nchan(Mol).GT.0 .AND. ifin.GT.Last ) THEN   ! Sample at end of each time bin
c         IF ( ifin.EQ.Last ) THEN
c            istart = Last
c           ELSE
            istart = Last + 1
c         ENDIF
         DO 250 i = istart , MIN(ifin,Ntime)
            Nuni(Mol,i) = Nuni(Mol,i) + 1.0D+00
            DO 150 k = 1 , Nchan(Mol)                  ! Rab(k)= k(E) for kth channel calculated by STEPPER [includes IVR effects]
               kuni(Mol,k,i) = Rab(k) + kuni(Mol,k,i)  ! average rate constant [includes IVR effects]
 150        CONTINUE
 250     CONTINUE
      ENDIF
 
      IF ( ifin.GT.Last ) THEN                  ! snapshot at end of each time interval
         DO 50 i = Last + 1 , MIN(ifin,Ntime)
            Num(Mol,i) = Num(Mol,i) + 1.d+00    ! Number of times ith bin accessed
            EMol(Mol,i) = EMol(Mol,i) + E       ! Average vibrational energy
 50      CONTINUE
         Last = ifin
      ENDIF

      IF ( ifind.GT.Dlast ) THEN                ! snapshot at end of each time interval
         DO 100 i = Dlast + 1 , MIN(ifind,Mtime)
            Mum(Mol,i) = Mum(Mol,i) + 1.d+00    ! Number of times ith time bin accessed
            
            EDIST(Mol,N,i) = EDIST(Mol,N,i) + 1.d+00 ! vibrational distribution
 100     CONTINUE
         Dlast = ifind
      ENDIF


c--------This block for sampling molecular energies    <------------------------- FOR ENERGIES
c      *** Normally, commented-out ***
c         kfin = 1 + (Time+Delt)*(kstep-1)/Tlim          ! time-step index for collisional sampling
c         IF ( kfin.GT.klast ) THEN                      ! snapshot at end of each time interval
c            DO i = klast + 1 , MIN(kfin,kstep)
c               WRITE (20,*) E
c            END DO
c            klast = kfin
c         ENDIF
c------End energy sampling block-------------

c
c	Now make changes
c
 
      Time = Time + Delt
 
      IF ( Igo.EQ.0 ) THEN                              ! Stop stepping
         RETURN
      ELSEIF ( IP.EQ.(1+Nchan(Mol)) ) THEN              ! COLLISION
         Step = COLSTEP(Mol,E,Temp)                     ! Collision step size
         E = E + Step                                   ! Energy after collision

c      write(21,*) Step   !         <------------------------- FOR COLLISION STEP-SIZES

      ELSE                                              ! REACTION
         IF ( Jto(Mol,IP).LE.NWells ) E = E + HMol(Mol)
     &        - HMol(Jto(Mol,IP))                       ! Vib energy of new molecule
         Mol = Jto(Mol,IP)                              ! New molecule index
      ENDIF
 
      IF ( Mol.GT.NWells .AND. Last.LT.Ntime ) THEN     ! If irreversible product
         Igo = 0                                        ! Stop stepping
         DO 200 i = Last + 1 , Ntime
            Num(Mol,i) = Num(Mol,i) + 1.d+00            ! Number of times ith bin accessed
 200     CONTINUE
      ENDIF
 
      RETURN
      END
      SUBROUTINE COLNORM(Mol,Temp)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c      REVISIONS
c
c      06/2008  Revised algorithm to minimize instabilities and improve 
c               convergence; Still some instability related to absent states
c               at low energies.
c
c      04/2006  General overhaul to improve accuracy, especially at 
c               low energies.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c	Normalization for a down-step model given
c		in FUNCTION PDOWN(E,EE,Mol,Temp,Hs)
c
c		Mol	= index of molecule
c		Temp	= translational temperature
c	 	E	= initial energy
c		EE	= final energy
c		Hs	= characteristic stepsize at energy E (calculated by PDOWN)
c
c	Note that COLLUP and CNORM are 'double arrays' with the first
c	imax1+1 elements starting at E = 0 with Egrain1 spacing; the remaining
c	elements are at Egrain2 spacing, starting at E = 0.
c
c	Coefficients stored in array DC
c	Probability of up-steps stored in COLLUP(MaxWell,Isize)
c	Normalization factor (up + down) stored in CNORM(MaxWell,Isize)
c
c       ERRo    = Convergence criterion declared in Declare3.inc include file
c	EDIFF 	= Minimum integration interval (cm-1)
c	FRACT  	= Step-size relative to 'Hs'
c	ITERS  	= Iterations on up-steps
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare3.inc'
 
      DOUBLE PRECISION PDOWN , DENVR , Temp , CNORMI , CNORMJ , 
     &                 RATIO , DELMIN , RATT , RATT0 , DUP

 
      DOUBLE PRECISION B , DENI , E , EE , Hs , PROB , H , DENJ , TERM , 
     &                 SPAN , TLAST , TEST , DOUBNTERP , a , BL ,
     &                 AA , TESTH , CC , Elow , arg , Ebare
 
      INTEGER I , J , kk , Mol , Igg , Ilow , jc ,
     &        IE , Istart

      DOUBLE PRECISION CN , PUP , PUPmax , denslo , Etherm
      DIMENSION CN(Imax) , denslo(Imax)
 
      EXTERNAL PDOWN , DENVR , RAN1 , DOUBNTERP , Etherm
 
      SAVE 

      Statest = 0.5d+00/Egrain1      ! Criterion for whether a state is present within an energy grain

      WRITE (*,99001) Mol
 
c	Integrate over UP steps (temporarily store up-step integrals in COLLUP)
c
       DO 100 I = 1 , Isize
 
         IF ( I.LE.imax1 ) THEN
            E = (I-1)*Egrain1
           ELSE
            E = (I-imax1-1)*Egrain2
         ENDIF

         DENI = DENVR(Mol,E)                                 ! Density at E(I)
         IF ( I. EQ. 1 ) DENI = 1.0                          ! Special case for E=0

         IF ( E.GE.0.0d+00 .AND. DENI.GE.Statest ) THEN      !  ONLY IF STATE, OTHERWISE SKIP
            TEST = 1.0
            COLLUP(Mol,I) = 0.0D+00     ! Temporary storage of UP-normalization
            EE = E
            PROB = PDOWN(E,EE,Mol,Temp,Hs)
            TLAST = 1.0d+00                                     ! Initial Hs and 'old' TERM
            DELMIN = Viblo(Mol) + 0.5*Hs*(Temp/hor)/(Hs + (Temp/hor))

            DO WHILE ( TEST.GT.ERRo )                            ! ********** UP STEPS **********
               IF ( EE.GT.Emax1 ) THEN
                  H = FRACT*Hs*(Temp/hor)/(Hs + (Temp/hor))    
                ELSE
                  H = Egrain1
               ENDIF

               EE = EE + H                                     ! UP-steps

               DENJ = DENVR(Mol,EE)
               IF ( DENJ .GE. Statest ) THEN
                  RATIO = DENJ/DENI
                  B = EXP(-(EE-E)*hor/Temp)                     ! EE > E
                  PROB = PDOWN(EE,E,Mol,Temp,Hs)               
                  TERM = PROB*B*RATIO
               ELSE
                  TERM = 0.0D+00
               ENDIF
 
               IF ( EE .GT. Emax1 ) THEN
                  a = -LOG(TERM/TLAST)/H                        ! Exponential form
                  AA = TLAST                                    ! Exponential form
                  SPAN = (AA/a)*(1.0D+00-EXP(-a*H))             ! Exponential form: integral
               ELSE
                  SPAN = 0.5D+00*H*( TERM + TLAST )             ! Trapezoidal Rule: integral
               ENDIF
 
               COLLUP(Mol,I) = COLLUP(Mol,I) + SPAN
 
               IF ( SPAN.GT.0.0 .AND. ABS(E-EE).GT.DELMIN )
     &              TEST = ABS(SPAN/COLLUP(Mol,I))
               TLAST = TERM                                     ! 'old' TERM
            ENDDO
         ENDIF
 100  CONTINUE

      WRITE (*,99002)
 
c		! ********** DOWN STEPS ***********
c*************************************************************
c 
      Ilow = Emax1/Egrain2 + 2 + imax1                 ! index of lowest element in upper half not overlapped with lower half

      CNORM(Mol,1) = COLLUP(Mol,1)
      CN(1) = 0.0d+00
 
        DO 250 I = 2 , Isize

        IF ( I.LE.imax1 ) THEN
           E = (I-1)*Egrain1
          ELSE
           E = (I-imax1-1)*Egrain2
        ENDIF

        DENI = EXP( Dens(Mol,I) )                   ! Density at E(I)

	IF ( DENI.LT.Statest ) THEN
          CNORM(Mol,I) = 0.0d+00
	ELSE                                        ! PROCEED ONLY IF STATE PRESENT

            EE = E
            PROB = PDOWN(E,E,Mol,Temp,Hs)                    ! to get Hs
            DELMIN = Viblo(Mol) + 0.5*Hs
            TLAST = 1.0d+00                                  ! Initial Hs and 'old' TERM
            TEST = 1.0
            CN(I) = 0.0d+00

            DO WHILE ( TEST.GT.ERRo .AND. EE .GE. Egrain1 )
              IF ( EE.GT.Emax1 ) THEN
                H = FRACT*Hs
                H = MIN( H , EE-Emax1 )
               ELSE
                H = Egrain1
              ENDIF
              IF ( H.GT.EE ) H = EE

              EE = EE - H                                  ! DOWN-steps
              DENJ = DENVR(Mol,EE)
              IF ( DENJ.GE.Statest ) THEN
                PROB = PDOWN(E,EE,Mol,Temp,Hs)                  
                TERM = PROB
               ELSE
                TERM = 0.0D+00
              ENDIF
 
              IF ( EE-H .GE. Emax1 ) THEN
                a = -LOG(TERM/TLAST)/H                     ! Exponential form
                AA = TLAST                                 ! Exponential form
                SPAN = (AA/a)*(1.0D+00-EXP(-a*H))          ! Exponential form
               ELSE
                SPAN = 0.5D+00*H*( TERM + TLAST )          ! Trapezoidal Rule
              ENDIF
              TLAST = TERM                          ! 'old' value of TERM
              CN(I) = CN(I) + SPAN
 
              IF ( SPAN.GT.0.0 .AND. ABS(E-EE).GT.DELMIN ) 
     &             TEST = ABS(SPAN/CN(I))

            ENDDO

            IF ( ( COLLUP(Mol,I)+CN(I) ) .GT. 0.0 ) THEN
              CNORM(Mol,I) = ( COLLUP(Mol,I) + CN(I) )         
             ELSE
              CNORM(Mol,I) = 0.0d+00
            ENDIF

        ENDIF           ! End test for state present
 
 250  CONTINUE          ! end energy steps

      CNORM(Mol,1) = COLLUP(Mol,1) 
      CNORM(Mol,imax1+1) = CNORM(Mol,1)
 
c	Now calculate final COLLUP array

      DO 500 I = 1 , Isize
        IF ( CNORM(Mol,I) .GT. 0.d+00 ) THEN
          COLLUP(Mol,I) = COLLUP(Mol,I)/CNORM(Mol,I)             ! Evaluate up-probability array
         ELSE
          COLLUP(Mol,I) = 0.0d+00
        ENDIF
 500  CONTINUE

c      Now use values for lower half of double array to replace overlapped values in upper half

      DO I = imax1+2 , Ilow-1
         EE = (I-imax1-1)*Egrain2
         CNORM(Mol,I)  = DOUBNTERP(EE,Mol,CNORM)
         COLLUP(Mol,I) = DOUBNTERP(EE,Mol,COLLUP)
      END DO

      COLLUP(Mol,1) = 1.0d+00
      COLLUP(Mol,imax1+1) = COLLUP(Mol,1)
      CNORM(Mol,imax1+1) = CNORM(Mol,1)

      WRITE (*,99003)
 
      RETURN
 
99001 FORMAT ('Entering Subroutine COLNORM: Molecule #',I2)
 
99002 FORMAT ('    Finished up steps, starting down-steps')
 
99003 FORMAT ('Leaving Subroutine COLNORM'/)
      END

      DOUBLE PRECISION FUNCTION COLSTEP(Mol,E,Temp)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c      REVISIONS
c      12/2004  General overhaul to improve speed and accuracy, especially at 
c               low energies.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
C
C     FROM RANDOM NUMBER RON AND ENERGY E COMPUTE A COLLISIONAL
C     STEP SIZE FROM THE NORMALIZED CUMULATIVE DISTRIBUTION 
C     FUNCTION CORRESPONDING TO THE COLLISIONAL DOWN-STEP MODEL:
C
C	IFLAG2 = -1 FOR DOWN-STEPS
C	IFLAG2 = +1 FOR UP-STEPS
C
c	Step-size distribution given in FUNCTION PDOWN(E,EE,Mol,Temp,Hs)
c
c		Mol	= index of molecule
c		Temp	= translational temperature
c	 	E	= initial energy
c		EE	= final energy
c		DC	= coefficients for stepsize distribution
c		Hs	= characteristic stepsize at energy E (calculated by PDOWN
c
c	Note that COLLUP and CNORM are 'double arrays' with the first
c	imax1+1 elements starting at E = 0 with Egrain1 spacing; the remaining
c	elements are at Egrain2 spacing, starting at E = 0.
c
c	Coefficients stored in array DC
c	Probability of up-steps stored in COLLUP(MaxWell,Isize)
c	Log of Normalization factor (up + down) stored in CNORM(MaxWell,Isize)
c
c	ERRo	= Convergence criterion (declared in "declare.inc")
c	EDIFF 	= Minimum integration interval (cm-1)
c	FRACT  	= Step-size relative to 'Hs'
c	ITERS  	= Iterations on up-steps
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare3.inc'
 
      DOUBLE PRECISION PDOWN , DENVR , Temp , CNORMI , CNORMJ , 
     &                 DOUBNTERP , RATIO , CD , X 
 
      DOUBLE PRECISION UP , RON , TOT , ADOWN , AUP , B , RAN1 , a , 
     &                 AA , Ht , arg , DELMIN , TEST , 
     &                 TESTH , CC
 
      INTEGER          Mol , IFLAG2 , IFLAG3 , Igg , Ntrap , JK , II ,
     &                 IEE
 
      DOUBLE PRECISION DENI , E , EE , Hs , PROB , H , DENJ , TERM , 
     &                 SPAN , TLAST , Tiggp , Tiggm
 
      EXTERNAL         DENVR , PDOWN , RAN1
 
      SAVE 

      IF ( E .GT. Emax2 ) THEN
        WRITE(*,*) '**********************************************'
        WRITE(*,*) ' FATAL ERROR at point #1 in colstep.f!'
        WRITE(*,*) ' E is greater than Emax2'
        WRITE(*,*) ' Suggest you increase Emax'
        WRITE(*,*) '**********************************************'
        STOP 
      ENDIF

      Statest = 0.5d+00/Egrain1

      DENI = DENVR(Mol,E)                               ! DENI = Density at E
      CNORMI = DOUBNTERP(E,Mol,CNORM)                   ! CNORMI = Normalization at E 
 
      PROB = PDOWN(E,E,Mol,Temp,Hs)     ! Evaluate to get initial Hs

      UP = DOUBNTERP(E,Mol,COLLUP)      ! UP = COLLUP at E = probability of an up-step
      IF ( UP .GT. 1.0d+00 ) UP = 1.0d+00

      RON = RAN1(IDUM)
      IF ( RON.LE.UP ) THEN             ! Select up-steps, or down-steps
         IFLAG2 = 1                     ! Up-step
        ELSE
         IFLAG2 = -1                    ! Down-step
      ENDIF
      IF ( E .LE. 0.0 ) IFLAG2 = 1
 

c ******************** Down steps ***************
      IF ( IFLAG2.EQ.-1 ) THEN                  !  Integrate over down steps
 
         DELMIN = Viblo(Mol) + 0.5*Hs
         CD = CNORMI*(1.d+00-UP)
 50      RON = RAN1(IDUM)
         IF ( RON .GT. (1.0d+00-ERRo) ) RON = 1.00D+00 - ERRo
 
         ADOWN = RON*CD !  Random-number-selected integral of down-steps

         TLAST = 1.0d+00                        ! 'old' TERM
         TOT = 0.0
         EE = E
         TEST = 1.0 
 
         DO WHILE (TOT.LT.ADOWN .AND. EE.GE.Egrain1 .AND. TEST.GT.ERRo)
              IF ( EE.GT.Emax1 ) THEN
                H = FRACT*Hs
                H = MIN( H , EE-Emax1 )
              ELSE
                H = Egrain1
              ENDIF
              IF ( H.GT.EE ) H = EE                        ! for down-steps

              EE = EE - H                                  ! DOWN-steps

              DENJ = DENVR(Mol,EE)
              IF ( DENJ.GE.Statest ) THEN
                PROB = PDOWN(E,EE,Mol,Temp,Hs)                  
                TERM = PROB
               ELSE
                TERM = 0.0D+00
              ENDIF
 
              IF ( EE-H .GE. Emax1 ) THEN
                Ntrap = 0
                a = -LOG(TERM/TLAST)/H                     ! Exponential form
                AA = TLAST                                 ! Exponential form
                SPAN = (AA/a)*(1.0D+00-EXP(-a*H))          ! Exponential form
               ELSE
                Ntrap = 1
                SPAN = 0.5D+00*H*( TERM + TLAST )          ! Trapezoidal Rule
              ENDIF
              TLAST = TERM                          ! 'old' value of TERM
              TOT = TOT + SPAN                      ! Down-step normalization integral
 
            IF ( DENJ .GE. Statest ) THEN
               IF ( (E-EE) .GE. DELMIN ) TEST = SPAN/TOT
               IF ( TOT .GE. ADOWN ) TEST = 0.0
            ENDIF
            
         ENDDO

         IF ( TOT.LT.ADOWN ) THEN       ! Invoked when estimated CNORMI is too large
            CD = TOT*(1.0d+00-ERRo)     ! Use new cumulative sum
            GOTO 50                     ! Repeat search with a fresh random number
         ENDIF
c                                       Begin correction for overshoot ( TOT > ADOWN )
         IF ( Ntrap .EQ. 1) THEN               ! Trapezoidal rule
             EE = EE + (TOT - ADOWN)*H/SPAN
           ELSE                                ! Exponential form
             arg = 1.0D+00 - a*( ADOWN - TOT + SPAN )/AA
             Ht = -LOG( arg )/a
             EE = EE + H - Ht           ! Correction for overshoot
          ENDIF

c ******************** Up steps ***************
      ELSE                                              ! Up-collisions
 
         CD = CNORMI*UP
 100     RON = RAN1(IDUM)
         IF ( RON .GT. (1.0d+00-ERRo) ) RON = 1.00D+00 - ERRo
         AUP = RON*CD                   ! Random number-selected
         PROB = PDOWN(E,E,Mol,Temp,Hs)  ! Evaluate to get initial Hs
         DELMIN = Viblo(Mol) + 0.5*Hs*(Temp/hor)/(Hs + (Temp/hor))
         TLAST = 1.0d+00                                ! 'old' TERM
         TOT = 0.0d+00
         EE = E
         TEST = 1.0
 
         DO WHILE ( TOT.LT.AUP .AND. TEST.GT.ERRo )
               IF ( EE.GE.Emax1 ) THEN
                  H = FRACT*Hs*(Temp/hor)/(Hs + (Temp/hor))
                ELSE
                  H = Egrain1
               ENDIF

               EE = EE + H                                     ! UP-steps

               DENJ = DENVR(Mol,EE)
               IF ( DENJ .GE. Statest ) THEN
                  RATIO = DENJ/DENI
                  B = EXP(-(EE-E)*hor/Temp)                     ! EE > E
                  PROB = PDOWN(EE,E,Mol,Temp,Hs)               
                  TERM = PROB*B*RATIO
                ELSE
                  TERM = 0.0D+00
               ENDIF
 
               IF ( EE .GT. Emax1 ) THEN
                  Ntrap = 0
                  a = -LOG(TERM/TLAST)/H                        ! Exponential form
                  AA = TLAST                                    ! Exponential form
                  SPAN = (AA/a)*(1.0D+00-EXP(-a*H))             ! Exponential form: integral
               ELSE
                  Ntrap = 1
                  SPAN = 0.5D+00*H*( TERM + TLAST )             ! Trapezoidal Rule: integral
               ENDIF
 
            TLAST = TERM                        ! Old value for TERM
            TOT = TOT + SPAN                    ! Up-step normalization integral
 
            IF ( DENJ .GE. Statest ) THEN
               IF ( (EE-E) .GE. DELMIN ) TEST = SPAN/TOT
               IF ( TOT .GE. AUP ) TEST = 0.0
            ENDIF

      IF ( EE .GT. Emax2 ) THEN
        WRITE(*,*) '**********************************************'
        WRITE(*,*) ' FATAL ERROR at point #2 in colstep.f!'
        WRITE(*,*) ' EE is greater than Emax2'
        WRITE(*,*) ' Suggest you increase Emax2'
        WRITE(*,*) '**********************************************'
        STOP 
      ENDIF

         ENDDO
 
         IF ( TOT.LT.AUP ) THEN                 ! Invoked when estimated CNORMI was too large
            CD = TOT*(1.0-ERRo)                 ! Use new cumulative sum
            GOTO 100                            ! Repeat search
         ENDIF

c                                       Begin correction for overshoot ( TOT > AUP )

         IF ( Ntrap .EQ. 1) THEN               ! Trapezpidal rule
             EE = EE - (TOT - AUP)*H/SPAN
           ELSE                                ! Exponential form
             arg = 1.0D+00 - a*( AUP - TOT + SPAN )/AA
             Ht = -LOG( arg )/a
             EE = EE - H +  Ht           ! Correction for overshoot
          ENDIF

      ENDIF
 
      IF ( EE.LT.0.0 ) EE = 0.0d+00

      IF ( EE .LE. Emax1) THEN               ! align with energy grains
        Igg = NINT( EE/Egrain1 )
        EE = Egrain1*Igg
        IF ( DENVR(Mol,EE) .LT. Statest ) THEN  ! if a state is not present, move up or down one grain
           Tiggp = DENVR(Mol,EE+Egrain1)
           Tiggm = DENVR(Mol,EE-Egrain1)
           IF ( Tiggp .GT. Tiggm ) THEN
               EE = EE + Egrain1
             ELSE
               EE = EE - Egrain1
           ENDIF
        ENDIF

        IF ( EE.LT.0.0 ) EE = 0.0d+00

      ENDIF
        
      COLSTEP = EE - E
      
      RETURN
      END
**==datetime.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
      SUBROUTINE DateTime(KUNIT)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Date & time subroutine
      CHARACTER*24 DateT
      REAL*4 tarray(2) , total
 
      CALL FDATE(DateT)
      WRITE (KUNIT,99001) DateT
      total = ETIME(tarray)
      WRITE (KUNIT,99002) tarray(1)
      WRITE (KUNIT,99003) tarray(2)
      WRITE (KUNIT,99004) total/60.0
      RETURN
 
99001 FORMAT (//,60x,A24)
99002 FORMAT (60x,'Elapsed User Time:  ',f12.3,' sec')
99003 FORMAT (60x,'Elapsed System Time:',f12.3,' sec')
99004 FORMAT (60x,'Total Elapsed Time: ',f12.3,' min',/)
      END
 
      SUBROUTINE DensArray
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c	Generates densities of states in 'double' array
c
c	Options: Whitten-Rabinovitch, or read densities from external file
c
c	SMol = 0 : A product molecule and do not calculate densities
c
c	Note that File name and path in OPEN statements may need revision
c		for non-Macintosh
c
c***********************************************************************
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER i , II , JJ , j , k , lenstr
      DOUBLE PRECISION E , D , WRSUM , A , B , Test1 , Test2
      CHARACTER TITLE*100
      EXTERNAL lenstr
      SAVE  
 
 
      DO 100 i = 1 , NWells
         Viblo(i) = 100.                        ! lowest vibrational frequency
            OPEN (12,FILE='DensData/'//
     &      MolName(i)(1:lenstr(MolName(i)))//'.dens',STATUS='OLD')

            READ (12,99030) dumcut                 ! Check to see if file format includes input data summary
            IF ( dumcut .EQ. cut ) THEN            ! Format starting with v.2008.1
               dumcut = 'x'
               DO WHILE (dumcut .NE. cut)
                 READ (12,99030) dumcut
               END DO
            ELSE
               BACKSPACE 12                        ! If format from prior to v.2008.1
            ENDIF
99030 FORMAT (A46)

            READ (12,*) TITLE
            READ (12,*) TITLE
            READ (12,*) A , JJ , B , k , Viblo(i)
            READ (12,*) TITLE
            Test1 = ABS(A-Egrain1)/1.D-06
            Test2 = ABS(B-Emax2)/1.D-06
            IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. Test2.LT.1.0 .AND. 
     &           k.EQ.Isize ) THEN              ! Test for proper array bounds, etc.
               DO 10 k = 1 , Isize
                  READ (12,*) j , A , Dens(i,k) , B
                  IF ( Dens(i,k).GT.0.0 ) THEN
                     Dens(i,k) = LOG(Dens(i,k)) ! Store natural log.
                  ELSE
                     Dens(i,k) = -70.
                  ENDIF
 10            CONTINUE
            ELSE                        ! If grain or energy limit does not match
               WRITE (KSTD,99002) MolName(i)
               STOP
            ENDIF
            CLOSE (12)
 100  CONTINUE
 
      RETURN
 
99001 FORMAT (A100)
99002 FORMAT (/'Energy grain or energy limit does not match: ',
     &        A10/'****** FATAL ERROR: Run Terminated *******')

      END
**==densarray.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
      SUBROUTINE DensArrayTS(Mol,i)

c same code as densarray.f except this version generates the double
c array for the transition state

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c***********************************************************************
c
c	Generates densities of states in 'double' array for the transition state
c       to be used later in the subroutine ECKART to calculate the tunneling
c	corrected sum of states for the transition state
c
c***********************************************************************
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER i , II , JJ , j , k , lenstr, Mol, it
      DOUBLE PRECISION E , D , WRSUM , A , B , Test1 , Test2
      CHARACTER TITLE*100
      EXTERNAL lenstr
      SAVE  
            OPEN (12,FILE='DensData/'//
     &    TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.dens'
     &             ,STATUS='OLD')
               DO 10 k = 1 , Isize
                  READ (12,*) j , A , DensTS(i,k) , B
                  IF ( DensTS(i,k).GT.0.0 ) THEN
                     DensTS(i,k) = LOG(DensTS(i,k)) ! Store natural log.
                  ELSE
                     DensTS(i,k) = -70.
                  ENDIF
 10            CONTINUE
      RETURN
 
      END
      DOUBLE PRECISION FUNCTION DENTS(Mol,i,E)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Interpolates 2-D double array DensTS(Nchan,Isize) to determine
c	density of states of the transtition state at energy E. 
c       Note that the densities are stored as natural logs.
c
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol,i
      DOUBLE PRECISION E , DD , DOUBNTERP2
      EXTERNAL DOUBNTERP2
      SAVE 

 
      DD = DOUBNTERP2(E,Mol,DensTS,i)
 
      DENTS = exp(DD)   ! Densities stored as logs
      RETURN
      END
**==denvr.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      DOUBLE PRECISION FUNCTION DENVR(Mol,E)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Interpolates 2-D double array Dens(Nmols,Isize) to determine
c		density of states at energy E. Note that the densities
c		are stored as natural logs.
c
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol
      DOUBLE PRECISION E , DD , DOUBNTERP
      EXTERNAL DOUBNTERP
      SAVE 
 
      DD = DOUBNTERP(E,Mol,Dens)
 
      DENVR = exp(DD)   ! Densities stored as logs
 
      RETURN
      END
**==doubnterp.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      DOUBLE PRECISION FUNCTION DOUBNTERP2(E,Mol,ARRAY,i)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Interpolates 2-Dim double array ARRAY to determine
c		interpolated value at energy E.
c
c	Note that this subroutine differs from doubnterp.f only in
c	the size of ARRAY; the size of ARRAY had to be increased so that
c	tunneling corretions can be calculated for all possible
c	channels out of all possible wells

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER IE , Mol, i
      DOUBLE PRECISION E , X , DD , ARRAY(MaxWell*MaxChan,Imax) , y1 
      DOUBLE PRECISION y2 , y3
      SAVE 
 
       IF ( E.LE.Emax1 ) THEN
         IE = 1 + INT(E/Egrain1)
         DD = ARRAY(i,IE)
       ELSE
         CALL NTERP(E,IE,X)
         IF ( ABS(X) .LE. 1.D-06 ) THEN
           DD = ARRAY(i,IE)
         ELSE
           y1 = ARRAY(i,IE-1)
           y2 = ARRAY(i,IE)
           y3 = ARRAY(i,IE+1)
           CALL QUDINT(y1,y2,y3,X,DD)
         ENDIF
      ENDIF
      DOUBNTERP2 = DD

      RETURN
      END
**==doubnterp.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      DOUBLE PRECISION FUNCTION DOUBNTERP(E,Mol,ARRAY)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Interpolates 2-Dim double array ARRAY to determine
c		interpolated value at energy E.
c
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER IE , Mol
      DOUBLE PRECISION E , X , DD , ARRAY(MaxWell,Imax) , y1 , y2 , y3
      SAVE 
 
       IF ( E.LE.Emax1 ) THEN
         IE = 1 + INT(E/Egrain1)
         IF (IE .LT. 1) IE = 1
         IF (IE .GT. Imax1) IE = Imax1
         DD = ARRAY(Mol,IE)
       ELSE
         CALL NTERP(E,IE,X)
         IF ( ABS(X) .LE. 1.D-06 ) THEN
           DD = ARRAY(Mol,IE)
         ELSE
           y1 = ARRAY(Mol,IE-1)
           y2 = ARRAY(Mol,IE)
           y3 = ARRAY(Mol,IE+1)
           CALL QUDINT(y1,y2,y3,X,DD)
         ENDIF
      ENDIF
      DOUBNTERP = DD

      RETURN
      END
        SUBROUTINE ECKART(Mol,i,EE,psum,j)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c This subroutine calculates the sum of states of the transition state
c modified to take into account the tunneling transmission coefficient.
c This modified sum of states is defined as psum below. psum is 
c calculated as the convolution of the density of states of the transition 
c state and the tunneling transmission coefficient. The program returns 
c psum to ratearray for calculation of k(E) corrected for tunneling
c
c-----------------------------------------------------------------------------

        IMPLICIT NONE
        INCLUDE 'declare1.inc'
        INTEGER Mol , i,  j , istep , ij , ijmax
        DOUBLE PRECISION pi , e2 , e3 , econstant , vh , pre
        DOUBLE PRECISION E1 , EE , E , tstep, rij, Egrain
        DOUBLE PRECISION a1 , b1 , c1 , c2 , c3 , c4 
        DOUBLE PRECISION ptop , pbot , tprob , psum , DENTS 
        DOUBLE PRECISION a5,b5,c5,d5
        DOUBLE PRECISION term1,term2,term3,term4,Echeck
        SAVE 
     


!	constants used in applying the tunneling correction

        pi=4.0D+00*DATAN(1.0D+00)
        e2=Eor(Mol,i)**(-0.5d0) 
        e3=V1(Mol,i)**(-0.5d0)
        econstant=(e2+e3)**(-1.0d0)
        vh=plank*avogadro*clight*jtocm/1000.0d0
        pre=(vimag(Mol,i)*vh)**(-1.0d0)

        c1=2.0d0*pi
        c2=(Eor(Mol,i)*V1(Mol,i))*pre**(2.0d0)
        c3=1.0d0/16.0d0
        c4=c1*(c2-c3)**(0.5d0)

        psum=0.0d0                 !zero out sum of states (ss) array

!	Egrain will be the array (lower or upper) dependent integration step size

        if(j.le.imax1*2)then       !these lines set the integration step size
          Egrain=Egrain1           !j is the index from ratearray.f which tells
          tstep=Egrain1            !us at which energy EE we are at; for j.le.imax1*2
        else                       !we use Egrain1 to perform the integration
          Egrain=Egrain2           
          tstep=Egrain2
        end if

        ijmax = 1 + EE/Egrain
        do ij = 1 , ijmax          !only calculate ss at grain energies
          rij = (ij-1)*Egrain

          E=rij                    !energy in the molecule's other degrees of freedom

          E1=EE-E-Eor(Mol,i)       !energy in the rxn coordinate

          if(EE.lt.0.0d0)goto 18   !EE is the energy from the zpe of the reactant it 
                                   !can never be negative

          if(Eor(Mol,i).ge.V1(Mol,i))then  !endothermic and barrier-less rxn condition
            Echeck=Eor(Mol,i)-V1(Mol,i)

            if((EE.ge.Echeck).and.(E1.ge.-V1(Mol,i)))then    !tunneling corrections are undefined if
                                                             ![EE < (Eor-V1)] for endothermic reactions
              continue
            else
              goto 18
            end if

          else                              !exothermic rxn condition

            if((EE.ge.0).and.(E1.ge.-Eor(Mol,i)))then
              continue         
            else
              goto 18
            end  if
                            
          end if
  

          a1=4.0d0*pi*pre*((E1+Eor(Mol,i))**(0.5d0))*econstant
          b1=4.0d0*pi*pre*((E1+V1(Mol,i))**(0.5d0))*econstant

!	approximate sinh(x) and cosh(x) by exp(x); this approximation
!	is valid since x is the value of a1, b1 and c4, and in this case
!	the values for these variables are large 

          a5=a1
          b5=b1
          c5=((a1+b1)/2.0d0)
          d5=c4

          term1=b5-d5
          term2=c5-a5
          term3=d5-a5
          term4=c5-d5

          tprob=exp(term1)/(exp(term2)*exp(term4)+exp(term3)) ! tunneling probability

          if(tprob.le.1.0d00)then  ! all tunneling probabilities should be 1 or less
            continue
          else
            tprob=1.0d00             ! this covers the case that the exponential terms
          end if                     ! in the above expression become too large; when this
                                     ! happens (and it sometimes happens for energies much
                                     ! greater then the reaction threshold and for small imaginary
                                     ! frequency) tprob=NAN when in fact it should be equal to one.
                                     ! Setting tprob=1 takes care of this problem

        if(tprob.lt.(tunthresh))then ! if tprob is less than tunthresh the contribution to the sum
          return                     ! of states of the transition state is very small and we stop
        else                         ! peforming the convolution
          continue
        end if
          psum=psum+tstep*(tprob*DENTS(Mol,i,E)) !density of states of TS is interpolated at energy E
 18    CONTINUE
       end do 
      RETURN
      END

      DOUBLE PRECISION FUNCTION Estart(Iflag,Icalc,IR,Molinit,Einit,T)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2007 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c		Selects initial energy for trial
c
c	Iflag	= 0	Initialize Pstart
c		= 1	select Estart from Pstart
c	Icalc	= type of initial distribution
c		= 1	Monoenergetic at energy Einit
c		= 2	Thermal with energy offset Einit
c		= 3	Chemical activation from 'product' #IR
c		= 4	Read Pstart from file 'multiwell.pstart'
c	IR	= index of 'product' from which chemical activation originates
c	Molinit	= index of initial molecule (well)
c	Einit	= initial energy (relative to ZPE of Molinit; units: cm-1)
c	T	= Temperature (vibrational)
c       Edels   = lowest energy (cm-1) of distribution (e.g. energy offset)
c       Hstart  = energy step-size (cm-1)
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
 
      INTEGER i , Iflag , ichan , Icalc , Molinit , IR , IE , IRUN , 
     &        itest , IA , Igg
 
      DOUBLE PRECISION AA , RX , E , T , RAN1 , DENVR , Einit , 
     &                 X , Y , Y1 , Y2 , Y3 , Term , Tlast , Etherm ,
     &                 RVR , CC, DOUBNTERP , Statest , Tiggp , Tiggm
 

      EXTERNAL Etherm , RVR
 
      SAVE 

      Statest = 0.1/Egrain1

      IF ( Iflag.EQ.0 ) THEN                    ! Initialize Pstart
 
         IF ( Icalc.EQ.1 ) THEN                 ! DELTA FUNCTION: No need for initialization
            RETURN

         ELSEIF ( Icalc.EQ.2 ) THEN                     ! THERMAL DISTRIBUTION
            Edels = Einit                               ! Offset

         ELSEIF ( Icalc.EQ.3 ) THEN                     ! CHEMICAL ACTIVATION
            itest = 0
            DO 40 i = 1 , Nchan(Molinit)                ! Find reaction index
               IF ( Jto(Molinit,i).EQ.IR ) THEN
                  ichan = i
                  itest = 1
               ENDIF
 40         CONTINUE
            IF ( itest.EQ.0 ) THEN
               WRITE (6,99001) Molinit , IR
               STOP
            ENDIF
            Edels = Eor(Molinit,ichan)                   ! Offset (incorporates centrifugal correction)
            Einit = Edels
 
         ELSEIF ( Icalc.EQ.4 ) THEN             ! READ NORMALIZED CUMULATIVE DISTRIBUTION FROM FILE
            OPEN (12,FILE='DensData/'//'multiwell.pstart',STATUS='OLD')
            READ (12,*) Jsize , Hstart , Edels     ! Edels and Hstart entered in cm-1
            IF ( Jsize .GT. Imax ) THEN
               write(*,*) '*** FATAL: Pstart array is too long ***'
               STOP
            ENDIF
            Einit = Edels
            DO 20 i = 1 , Jsize
               READ (12,*) AA , Pstart(i)       ! AA is dummy real variable
 20         CONTINUE
            CLOSE (12)
            RETURN
 
         ENDIF
 
         Hstart = 4.d+00*Etherm(Molinit,T)/Isize        ! Set step size
         Hstart = MAX( Egrain1 , Hstart )               ! Make sure step size not too small
         Jsize = 20.d+00*Etherm(Molinit,T)/Hstart       ! Set array length
         IF ( Jsize .GT. Imax ) Jsize = Imax

         Tlast = 0.0D+00
         DO 50 i = 1 , Jsize
            IF ( Icalc.EQ.3 ) THEN                         ! CHEMICAL ACTIVATION
               if(itun( Molinit , ichan ) .eq. 1 ) then    ! with tunneling
                 Einit = Edels - Emax1                     ! start at Emax1 below the barrier
                 IF ( Einit .LT. 0.0 ) Einit = 0.0
                 E = Einit + (i-1)*Hstart
                 CALL NTERP2((E-Edels),IE,X)
                 Y1 = TRate(Molinit,ichan,IE-1)
                 Y2 = TRate(Molinit,ichan,IE)
                 Y3 = TRate(Molinit,ichan,IE+1)
                 CALL QUDINT(Y1,Y2,Y3,X,Y)
                 Y = exp(Y)                                     ! Rate constant k(E) [stored as log]
                 Term = Y*DENVR(Molinit,E)*exp(-hor*E/T)
               else
                 E = Edels + (i-1)*Hstart                       ! Chemical activation without tunneling
                 Y = RVR( Molinit , ichan , E-Edels , IE )      ! Interpolated k(E)
                 Term = Y*DENVR(Molinit,E)*exp(-hor*E/T)
               end if
            ELSEIF ( Icalc.EQ.2 ) THEN                          ! THERMAL, WITH OR WITHOUT OFFSET
               E = (i-1)*Hstart
               Term = DENVR( Molinit , E )*exp(-hor*E/T)
            ENDIF
            IF ( Hstart .EQ. Egrain1 ) THEN
               Pstart(i) = Hstart*Term                     ! Constant within each grain at low energies
            ELSE
               Pstart(i) = 0.5D+00*Hstart*(Term+Tlast)     ! Trapezoidal rule for larger steps
            ENDIF
            Tlast=Term
 50      CONTINUE
 
         DO 100 i = 2 , Jsize
            Pstart(i) = Pstart(i) + Pstart(i-1)          ! Cumulative
 100     CONTINUE
 
         DO 150 i = 1 , Jsize
            Pstart(i) = Pstart(i)/Pstart(Jsize)          ! Normalize Cumulative distribution
 150     CONTINUE
 
c ---------------------------------------------------------------------------------
c ---------------------------------------------------------------------------------
c ---------------------------------------------------------------------------------

      ELSEIF ( Iflag.EQ.1 ) THEN                ! Select initial energy from Pstart
 
         IF ( Icalc.EQ.1 ) THEN
            Estart = Einit                              ! For delta function
            IF (Estart .LE. Emax1) THEN                 ! Align Estart with grains
                Igg = INT( Estart/Egrain1 + 0.5d+00 )
                Estart = Egrain1*Igg
                IF (Estart .LT. 0.0d+00) Estart = 0.0d+00
            ENDIF
            RETURN
         ELSE                                           ! For THERMAL, CHEMACT, etc.
 
            RX = RAN1(IDUM)
            i = 0
            IRUN = 1
            DO WHILE ( IRUN .EQ. 1 )
               i = i + 1
               IF ( Pstart(i) .GT. RX ) IRUN = 0
            ENDDO
 
            Nstart(i) = Nstart(i) + 1           ! Record selected distribution for posterity

            IF ( i .GT. 1 ) THEN
               IF ( Hstart .EQ. Egrain1 ) THEN                          ! for Egrain1 steps
                  Estart = Einit + Hstart*(i-1)
                ELSE
                  Estart = Einit + Hstart*( (i-1) +                     ! interpolate for other steps
     &               (RX - Pstart(i-1))/(Pstart(i) - Pstart(i-1)) )

                  Igg = NINT( Estart/Egrain1 )                    ! align with energy grains at low energy
                  Estart = Egrain1*Igg
                  IF ( DENVR(Molinit,Estart) .LT. Statest ) THEN  ! if a state is not present, move up or down one grain
                    Tiggp = DENVR(Molinit,Estart+Egrain1)
                    Tiggm = DENVR(Molinit,Estart-Egrain1)
                    IF ( Tiggp .GT. Tiggm ) THEN
                      Estart = Estart + Egrain1
                    ELSE
                      Estart = Estart - Egrain1
                    ENDIF
                  ENDIF
               ENDIF
             ELSE
                 Estart = Einit
            ENDIF

         ENDIF  ! Icalc
      ENDIF     ! Iflag

      IF ( Estart.LT.0.0 ) Estart = 0.0d+00

      IF(DOUBNTERP(Estart,Molinit,CNORM) .LT. 0.001) write(20,*) Estart       ! Statement for testing purposes

      RETURN
 
99001 FORMAT (/'Data file error: CHEMACT rxn does not exist:',I3,' to',
     &        I3/'****** Hit RETURN to terminate run *******')
      END
**==etherm.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
      DOUBLE PRECISION FUNCTION Etherm(Mol,T)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2005 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol
      DOUBLE PRECISION Q , T , E , ET , Term , Test , H , DENVR , 
     &                 Tlast , Elast
      EXTERNAL DENVR
      SAVE 
c
c	Average thermal energy of Mol at temperature T
c
c	Uses numerical integration of densities of states.
c
         ET = 0.0D+00
         Q = 0.0D+00
         Elast = 0.0D+00
         Tlast = 0.0D+00
         H = Egrain1
         E = -H
         Test = 1.0
         DO WHILE ( Test.GT.1.0E-08 )
            E = E + H
            Term = DENVR(Mol,E)*exp(-hor*E/T)
            Q = Q + 0.5*H*(Tlast+Term)                  ! Trapezoidal rule
            ET = ET + 0.5*H*(Elast*Tlast+E*Term)        ! Trapezoidal rule
            Tlast = Term
            Elast = E
            IF ( E.GT.10.*T ) Test = E*Term/ET
         ENDDO
 
         Etherm = ET/Q
         RETURN
      END
 
      SUBROUTINE INITIAL
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Initialize bookkeeping arrays between trials
c
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
      INTEGER i , j , k , n
      SAVE 
 
      DO 200 i = 1 , NWells
 
         DO 50 j = 1 , Ntime
            EMol(i,j) = 0.0D+00 ! average vibrational energy
            Num(i,j) = 0        ! number of samples for molecular properties
            Nuni(i,j) = 0       ! number of samples for reaction rates
            DO 20 n = 1 , Nchan(i)
               kuni(i,n,j) = 0.0D+00
                                ! average unimolecular rates
 20         CONTINUE
 50      CONTINUE
 
         DO 100 j = 1 , Ndist
            DO 60 k = 1 , Mtime
               EDIST(i,j,k) = 0 ! vibrational distribution
 60         CONTINUE
 100     CONTINUE
 
         DO 110 j = 1 , Mtime
            Mum(i,j) = 0
 110     CONTINUE

 200  CONTINUE
 
      DO 300 i = 1 , Isize
         Nstart(i) = 0          ! distribution of selected starting energies
 300  CONTINUE
 
 
      IF ( NProds.GT.0 ) THEN
         DO 350 i = NWells + 1 , NWells + NProds
            DO 320 j = 1 , Ntime
               Num(i,j) = 0     ! number of samples for molecular properties
 320        CONTINUE
 350     CONTINUE
      ENDIF
 
      RETURN
      END
c
      function lenstr(string)
c
      implicit      none
c
      character*(*) string,  !input string
     2              null*1   !null character
c
      integer*4     i,       !string index
     2              iend,    !candidate for string length
     3              j,       !index used for stepping through string
     4              length,  !defined length of input
     5              lenstr   !length of string
c
      length=len(string)
c
c     Look for the existence of nulls; if there
c     are any, the first one marks one character past the
c     end of the defined string. If there are no nulls,
c     pick up the final nonspace character, and call that
c     the defined length of the string.
c
      null=char(0)
c
c     Look for nulls
c
      iend=index(string(1:length),null)
c
c     No nulls, look for the location of the last nonspace.
c
      if(iend.eq.0)then
          do i=1,length
              j=length+1-i
              if(string(j:j).ne.' ')then
                  lenstr=j
                  return
              endif
          enddo
c
c         all spaces
c
          lenstr=0
          return
      else
c
c         first null-1
c
          lenstr=iend-1
          return
      endif
      end

      PROGRAM MultiWell
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c
c	VERSION HISTORY
c
c Subsequent: see HISTORY file in main directory
c
c 10/03  v1.3.3   (JP) trimmed defined-length character strings with lenstr
c 08/03  v1.3.3b -BUG FIX: fixed problem in subroutine colstep.f that caused
c                 population to become trapped in energy grains that contained 
c                 no states.
c                -BUG FIX: fixed problem in estart.f, in which the lower array
c                 bounds were violated.
c 04/03  v1.3.2  -New Feature: optionally enter total concentration
c                 (molecule/cc) in place of total pressure [use Punit key 
c                 word 'MCC'].
c 01/03  V1.3.1  -Small changes in format of output files
c 08/02  v1.2.3  -Bug Fix: in subroutine book1.f, fixed bug that gave 
c                 erroneously large yields when the time scale was much shorter
c                 than the inverse of the sum of the rates (k(E) + collision
c                 frequency).
c 01/02  v1.2.0  -New Feature: can optionally choose to apply centrifugal
c                 corrections for any choice of model. Thus can apply them
c                 even when using Inverse Laplace Transform form of k(E).
c                 NOTE: this new feature affects the data file format.
c 11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                 inconsistently when calculating Arrhenius parameters 
c                 (k(inf) was correct, however).
c 11/01   v1.1.3 -Bug fix in DenSum and in Thermo, not in MultiWell itself.
c 09/01   v1.1.2 -New feature: time limit (Tlim) set by specifying max time, 
c                 or by specifying max number of collisions. Makes fall-off 
c                 calculations more efficient.
c                -New feature: short summary output file (MultiWell.SUM)
c                 designed to make fall-off calculations more efficient.
c                -Fixed bug in centrifugal corrections to rate constants.
c                 Bug was most evident when the moment of inertia ratio 
c                 (I+/I) was significantly different from unity.
c 04/01   v1.1.1 -Maintenance revision; fixed energy distribution (EDIST)
c                 so that it is normalized according to Ntrials; EDIST is
c                 also reported as a "snapshot", instead of a time average.
c 03/01   v1.1   -New release with many changes. In particular, error in
c                     chemical activation option was fixed.
c 11/99   v1.01  -Revised to compile on LINUX and UNIX; no functional changes.
c 09/99   v1.0   -Macintosh version for Absoft F77 compiler.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
 
      CHARACTER TITLE*100 , FROMTO*10 , TS*10 , Punits*3 , bar*3 , 
     &          atm*3 , tor*3 , POUT*4 , Eunits*4 , kcal*4 , kjou*4 ,
     &          cmm1*4 , Tspec*4 , mcc*3 , Rotatunits*4
      CHARACTER*120  NameFile, NameOUT, NameSMM, NameARY, NameDIST, 
     & NameFLX, NameRAT


      CHARACTER AVERSION*6, ADATE*8
      PARAMETER ( AVERSION='2010', ADATE='Jan 2010' )
 
      PARAMETER (FROMTO=' From...To ',bar='BAR',atm='ATM',tor='TOR',
     &           mcc='MCC',cmm1='CM-1',kcal='KCAL',kjou='KJOU')
 
      INTEGER i , j , k , l , Nforward , M , Ntrials , Igo , IPP , II , 
     &        Mol , Nreac , ito , Jr , Icalc , IR , Molinit , JJ , ki ,
     &        Mdum , Np , Ip , NumTot , IDUM0 , ncen , lenstr , KK ,
     &        kkey , ntunflag , nivrflag , Pressflag , NSTOP , iflag
 
      DOUBLE PRECISION Temp , P(50) , PP(50) , Sig , Eps , Amu , Sigma ,
     &                 Epsil , Mass , pi , Time , E , XRANST , AA , EE ,
     &                 Einit , RR , SigM , EpsM , AmuM , CollFreq , EM
 
      DOUBLE PRECISION FracMol(MaxWell+MaxProd) , AveE(MaxWell+MaxProd)
     &                 , Test , SCALE , numbdens , DistStep , Tvib , 
     &                 klj , kqm ,Tread , kcol , DD , DR , DOUBNTERP
 
      DOUBLE PRECISION RAN1 , Etherm , PDOWN , Estart , 
     &                 Kinf(MaxWell,MaxChan) , Ainf(MaxWell,MaxChan) , 
     &                 Einf(MaxWell,MaxChan) , Var(MaxWell+MaxProd) , 
     &                 Pfact , Rob_1 , Delt , 
     &                 Flux(MaxWell,MaxChan,Ntime),
     &                 kosc(MaxWell,MaxChan) , qele

      CHARACTER        KEYWORD*8 , KEYTEMP*8 , DUM*4
      DIMENSION        KEYWORD(5) , DUM(3)
 
      DOUBLE PRECISION  vivr , vave , civr , pivr , tivr
      INTEGER     jivr , jtun ,  iivr
      INTEGER     III, Iwell , ichan
      INTEGER     iwellindex, Nreaindex(MaxWell*MaxChan)
      INTEGER     Nproindex(MaxWell*MaxChan)
      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            vave( MaxWell , MaxChan ) ,
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

      EXTERNAL RAN1 , Etherm , PDOWN , Estart
 
      SAVE 
 
      pi = 4.0D+00*DATAN(1.0D+00)

      call GETARG(1,NameFile)
      NameFile=NameFile(1:lenstr(NameFile))
      IF(NameFile.eq.'') NameFile="multiwell.dat"
      NameOUT=NameFile(1:lenstr(NameFile)-4)//".out"
      NameSMM=NameFile(1:lenstr(NameFile)-4)//".sum"
      NameARY=NameFile(1:lenstr(NameFile)-4)//".array"
      NameDIST=NameFile(1:lenstr(NameFile)-4)//".dist"
      NameRAT=NameFile(1:lenstr(NameFile)-4)//".rate"
      NameFLX=NameFile(1:lenstr(NameFile)-4)//".flux"
c
c	I/O Unit numbers
c
      KIN   = 2  ! Data file
      KSTD  = 6  ! standard output (screen)
      KSMM  = 7  ! Short summary of output file 
      KARY  = 8  ! Summary of array variables
      KOUT  = 9  ! General time-dependent output
      KRAT  = 10 ! average rate 'constants'
      KFLX  = 11 ! relative reactive flux via a given path
      KDIS  = 12 ! vib distribution

      OPEN (KIN,FILE=NameFile,STATUS='OLD')          ! Data file
      OPEN (KOUT,FILE=NameOUT,STATUS='UNKNOWN')      ! General time-dependent output
      OPEN (KSMM,FILE=NameSMM,STATUS='UNKNOWN')      ! Short summary output for fall-off calculations
      OPEN (KARY,FILE=NameARY,STATUS='UNKNOWN')      ! Summary of input, output, and intermediate array variables
      OPEN (KRAT,FILE=NameRAT,STATUS='UNKNOWN')      ! average rate 'constants'
      OPEN (KFLX,FILE=NameFLX,STATUS='UNKNOWN')      ! relative reactive flux via a given reaction path
      OPEN (KDIS,FILE=NameDIST,STATUS='UNKNOWN')     ! vib distributions vs. time
  
      READ (KIN,99003) TITLE

      DO ki = 7 , 11
        WRITE (ki,99001) AVERSION , ADATE , AVERSION , ADATE
        CALL DateTime(ki)
        WRITE (ki,99003) TITLE
      END DO

      READ (KIN,*) Egrain1 , imax1 , Isize , Emax2 , IDUM

      IF ( Isize .GT. Imax ) THEN
         WRITE (KSTD,99002) Isize
         WRITE (KOUT,99002) Isize
         STOP
      ENDIF
      
      Emax1 = Egrain1*(imax1-1)
      Egrain2 = Emax2/(Isize-imax1-1)
c.......................................................................
c	Energy grain parameters for interpolation 'double arrays'
c
c	'Double arrays' have two sections:
c		segment 1 consists of equally spaced (Egrain1) values ranging
c			from E=0 to Emax1
c		segment 2 consists of equally spaced values from E=0 to Emax2;
c			the spacing depends on the number of array elements
c			remaining: Isize-Emax1/Egrain1-1
c
c	Egrain1	= energy grain of lower segments in 'double array' (units: cm-1)
c	Emax1	= maximum energy of 1st segment of double array (units: cm-1)
c	Emax2	= maximum energy of 2nd segment of double array (units: cm-1)
c	IDUM	= random number seed (positive, non-zero integer)
c.......................................................................
c
 
      IF ( IDUM.EQ.0 ) IDUM = 1990339   ! Random number seed
      IDUM0 = IDUM
      IDUM = -ABS(IDUM)
      XRANST = RAN1(IDUM)    ! initialize RAN1

      DO ki = 7 , 11
        WRITE (ki,99044) Egrain1 , Emax1 , imax1 , Egrain2 , Emax2 , 
     &                   (Isize-imax1)
      END DO
      
c.......................................................................
c	Key words for units
c    READ Punits , Eunits , Rotatunits
c
c	Punits	  = pressure units key word [use upper case CHARACTER*3]
c		       = 'BAR', 'ATM', 'TOR' [also will accept 'TORR'], or 'MCC'
c	Eunits	  = energy units keyword [use upper case CHARACTER*4]
c		       = 'CM-1','KCAL', or 'KJOU' (for cm-1, kcal/mole, or kJ/mole)
c    Rotatunits  = units for moment of inertia 
c                = 'AMUA', 'GMCM', 'CM-1', MHz, 'GHz'
c.......................................................................
      READ (KIN,*) DUM(1) , DUM(2) , DUM(3)
      CALL UnitTest ( DUM , Punits , Eunits , Rotatunits , KOUT )
c.......................................................................
c	Physical parameters
c
c	Temp	= translational temperature (units: degrees Kelvin)
c	Tvib	= vibrational temperature (units: degrees Kelvin)
c	Np	= number of pressures
c	PP	= vector of Np pressures
c.......................................................................
      READ (KIN,*) Temp , Tvib
      READ (KIN,*) Np
      READ (KIN,*) (PP(i),i=1,Np)
      IF ( Np.GT.50 ) THEN
         write(*,*) 'FATAL: Only 50 pressures allowed....'
         write(KOUT,*) 'FATAL: Only 50 pressures allowed....'
         STOP
      ENDIF
c.......................................................................
c
c	Convert pressures to bars
c
c.......................................................................
      IF ( Punits.EQ.atm ) THEN
         POUT = 'atm '
         Pfact = 1.01325
      ELSEIF ( Punits.EQ.tor ) THEN
         POUT = 'Torr'
         Pfact = 1.01325/760.
      ELSEIF ( Punits.EQ.bar ) THEN
         POUT = 'bar '
         Pfact = 1.0
      ELSEIF ( Punits.EQ.mcc ) THEN
         POUT = '1/cc '
         Pfact = Temp/7.2428D+21
      ELSE
         WRITE(*,*) 'FATAL: Pressure units not specified properly'
         WRITE(KOUT,*) 'FATAL: Pressure units not specified properly'
         STOP
      ENDIF
 
      DO 100 i = 1 , Np
         P(i) = PP(i)*Pfact
 100  CONTINUE
 
      READ (KIN,*) NWells , NProds
      DO 200 i = 1 , NWells
         READ (KIN,*) IMol(i) , MolName(i) , HMol(i) , MolMom(i) , 
     &                Molsym(i) , Molele(i) , Molopt(i)
c         MolName(i) = ADJUSTR( MolName(i) )

         IF ( Eunits.EQ.kcal ) THEN
            HMol(i) = HMol(i)*caltocm           ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            HMol(i) = HMol(i)*jtocm           ! Convert kJ/mole to cm-1
         ENDIF
         Nchan(i) = 0                           ! Initialize Nchan
         
         call rotunits( Rotatunits , MolMom(i) )     ! unit conversion for moments of inertia
200   CONTINUE
 
      DO 300 i = NWells + 1 , NWells + NProds
         READ (KIN,*) IMol(i) , MolName(i) , Hmol(i)
c         MolName(i) = ADJUSTR( MolName(i) )
         IF ( Eunits.EQ.kcal ) THEN
            HMol(i) = HMol(i)*caltocm           ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            HMol(i) = HMol(i)*jtocm           ! Convert kJ/mole to cm-1
         ENDIF
 300  CONTINUE
c.......................................................................
c	Reactant/Product parameters
c
c	NWells	= number of 'wells'
c	NProds	= number of entrance/exit species
c
c	IMol	= index number for molecule
c	MolName	= name (10 characters) of molecule
c	HMol	= enthalpy of formation at 0K
c	MolMom	= moment of inertia for 2-dim external rotor (amu Ang^2), 
c                gram*cm^2, cm-1, MHz, or GHz
c	Molsym	= external symmetry number for molecule
c	Molele	= electronic partition function for molecule
c	Molopt	= number of optical isomers for molecule
c	Iread	=1: read densities from file
c		  *** filename: same name as MolName +'.dens'
c		  ***    e.g. 'Benzene.dens'
c.......................................................................
      CALL DensArray                            ! Calculate or read densities
 
      DO ki = 7 , 9
        WRITE (ki,99031) Temp , Tvib , NWells , NProds
        WRITE (ki,99036) (i,i=1,NWells+NProds)
        WRITE (ki,99035) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
        WRITE (ki,99023) (HMol(i),i=1,NWells+NProds)
        WRITE (ki,99025) (HMol(i)/jtocm,i=1,NWells+NProds)
        WRITE (ki,99024) (HMol(i)/caltocm,i=1,NWells+NProds)
        WRITE (ki,99033) (Viblo(i),i=1,NWells)
        WRITE (ki,99045) (MolMom(i),i=1,NWells)
        WRITE (ki,99046) (Molsym(i),i=1,NWells)
        WRITE (ki,99047) (Molele(i),i=1,NWells)
        WRITE (ki,99048) (Molopt(i),i=1,NWells)
        WRITE (ki,99041) (Etherm(i,Temp),i=1,NWells)
        WRITE (ki,99009)
      END DO
 
      WRITE (KARY,99010) (ADJUSTR(MolName(i)),i=1,NWells)

      DO 400 II = 1 , Isize
         IF ( II.LE.imax1 ) THEN
            E = (II-1)*Egrain1
         ELSE
            E = (II-imax1-1)*Egrain2
         ENDIF
         WRITE (KARY,99011) E , (exp(Dens(i,II)),i=1,NWells)
                                                        ! Densities stored as logs
 400  CONTINUE
 
      READ (KIN,*) SigM , EpsM , AmuM , Amu
c.......................................................................
c	Collision parameters - collider
c
c	SigM	L-J sigma (Ang) for collider
c	EpsM	L-J Eps (Kelvins) for collider
c	AmuM	Molecular weight (g/mole) of collider

      DO ki = 7 , 9
        WRITE(ki,99073) NWells
        WRITE (ki,99055)
        Mdum = 0
        WRITE (ki,99056) Mdum , SigM , EpsM , AmuM
      END DO
      
      DO 500 j = 1 , NWells
         READ (KIN,*) Mol , Sig , Eps , ITYPE(Mol) , (DC(Mol,i),i=1,8)
         READ (KIN,*) LJQM(Mol)
c.......................................................................
c	Collision parameters
c
c	Sig	L-J sigma (Ang) for molecule
c	Eps	L-J Eps (Kelvins) for molecule
c	Amu	Molecular weight (g/mole) of molecule
c	ITYPE	selects model type in Subroutine PDOWN
c	DC	coefficients for collision model
c	LJQM	= 'LJ' for Lennard-Jones collision frequency, ALL collisions inelastic
c		= 'QM' for total cross section collision frequency
c.......................................................................
c
         Sigma = 0.5*(SigM+Sig)
         Epsil = SQRT(EpsM*Eps)
         Mass = AmuM*Amu/(AmuM+Amu)
 
         klj = pi*1.E-16*(Sigma**2)*1.45525E+04*SQRT(Temp/Mass)
     &         /(0.636+0.246*LOG(Temp/Epsil))   ! L-J collision integral
 
         kqm = 7.64E-12*((Epsil*Sigma**6)**0.4)*(Temp/Mass)**0.3
         QCollK( Mol ) = kqm                                          ! Max quantum rate constant

         IF ( LJQM(Mol).EQ.LJ ) THEN
            CollK(Mol) = klj
         ELSE
            CollK(Mol) = kqm
         ENDIF
c.......................................................................
c
c	klj = L-J collision rate constant (s-1)
c	kqm = collision rate constant based on the total quantum cross section
c		[Durant and Kaufman, Chem. Phys. Letters, 142, 246 (1987)]
c.......................................................................
c
      DO ki = 7 , 9
         WRITE (ki,99056) Mol , Sig , Eps , Amu , Sigma , Epsil , 
     &                      Mass , klj , kqm , CollK(Mol)
      END DO
      
 500  CONTINUE
c
c	Call PDOWN WITH E=-1.0 in order to store LI(Mol,i) in COMMON for print-out
c
      WRITE (KOUT,99030)
      WRITE (KSMM,99030)
      WRITE (KARY,99030)
 
      DO Mol = 1 , NWells
         CALL COLNORM(Mol,Temp)         ! Calculate COLLUP and CNORM arrays
      END DO

 
      READ (KIN,*) Nforward
      Nreac = Nforward
 
      IF ( Nforward .GT. 0 ) WRITE(KOUT,99074) Nforward

      ntunflag = 0                     ! flag for tunneling
      nivrflag = 0                     ! flag for slow IVR
      Pressflag = 0                    ! flag for reactant [B]


      DO 800 i = 1 , Nforward
 
         READ (KIN,*) Mol , ito , TS , RR , j , qele ,
     &                 l , AA , EE , (KEYWORD(II),II=1,5)
         WRITE (KOUT,99075) i , Mol , ito , TS , RR ,  
     &             j , qele , l , AA , EE , (KEYWORD(II),II=1,5)
     
         call rotunits( Rotatunits , RR )     ! unit conversion for moments of inertia

         Nreaindex(i)=Mol ! keep track of reactants and products so that
         Nproindex(i)=ito ! the initial distribution and Eor can be printed
                          ! correctly in multiwell.array
c.......................................................................
c	Reaction parameters
c
c	Nforward= number of reaction parameters to be input (forward rxns, only)
c	Mol	= index of reactant molecule
c	ito	= Jto	 = index of product molecule
c	TS	= TSname = Name of transition state (up to 10 characters)
c	RR	= TSmom  = moment of inertia (amu.ang^2, gram.cm^2, cm-1, MHz, GHz)
c	j	= TSsym  = external symmetry number for TS
c	qele	= TSele	 = electronic partition function for TS
c	l	= TSopt	 = number of optical isomers for TS
c	AA	= Afact	= A-factor for reaction (units: s-1)
c	EE	= Eo	= critical energy, relative to ZPE of reactant
c
c       KEYWORDS (5 are expected):
c
c         'CENT'     for quasi-diatomic centrifugal correction    [NCENT=1]
c         'NOCENT'   for no quasi-diatomic centrifugal correction [NCENT=0]
c
c	  'NOREV'    for neglecting the reverse reaction        [Jrev = Jr]
c         'REV'      for calculating reverse rxn rate; after calculation,
c		        Jrev=1 is changed to Jrev=2
c
c         'FAST'     for neglecting limitations due to IVR
c         'SLOW'     for including IVR limitations; a following line contains 
c                        'SLOW', pivr(Mol,i), vivr(Mol,i), vave(Mol,i), 
c                             tivr(Mol,i), civr(Mol,i,1), civr(Mol,i,2), 
c                             civr(Mol,i,3)
c
c         'NOTUN'    for neglecting tunneling
c         'TUN'      for including tunneling via unsymmetrical Eckart barrier;
c                       a following line contains: 'TUN', tunnel(Mol,i)
c
c          'ILT'     Inverse laplace transform method for k(E)
c          'SUM'     File containing sums of states (usually generated by Densum)
c          'RKE'     File containing k(E), generated externally
c          'REVRKE'  k(E) for reverse reaction calculated from k(E) file for forward rxn
c
c                Jread = 0 for ILT method
c                      = 1 for reading sums of states from external file
c                        *** filename: same name as TSame +'.dens'
c                        ***   e.g. 'TS-1.dens'
c                      = 2 for reading k(E) from external file
c		         *** filename: same name as TSame +'.rke'
c		         ***   e.g. 'TS-1.rke'
c                      = -2 for calculating reverse k(E) from external file for forward k(E)
c		         *** filename: same name as TSame +'.rke'
c		         ***   e.g. 'TS-1.rke'
c.......................................................................
c
c
c	Nchan(Mol) = number of rxn channels originating from a given molecule
c.......................................................................
 
         Nchan(Mol) = Nchan(Mol) + 1
         Jto(Mol,Nchan(Mol)) = ito
         TSname(Mol,Nchan(Mol)) = TS
         TSmom(Mol,Nchan(Mol)) = RR
         TSsym(Mol,Nchan(Mol)) = j
         TSele(Mol,Nchan(Mol)) = qele
         TSopt(Mol,Nchan(Mol)) = l
         Afac(Mol,Nchan(Mol)) = AA
         IF(Afac(Mol,Nchan(Mol)).lt.0) THEN
           Press(Mol,Nchan(Mol))=ABS(Afac(Mol,Nchan(Mol)))
           Pressflag=Pressflag+1
          ELSE
           Press(Mol,Nchan(Mol))=1
         ENDIF
         IF ( Eunits.EQ.cmm1 ) THEN
            Eo(Mol,Nchan(Mol)) = EE                     ! cm-1 units
         ELSEIF ( Eunits.EQ.kcal ) THEN
            Eo(Mol,Nchan(Mol)) = EE*caltocm             ! Convert kcal/mole to cm-1
         ELSEIF ( Eunits.EQ.kjou ) THEN
            Eo(Mol,Nchan(Mol)) = EE*jtocm             ! Convert kJ/mole to cm-1
         ENDIF
         Hts(Mol,ito) = HMol(Mol) + Eo(Mol,Nchan(Mol))  ! Rxn barrier enthalpy
         Hts(ito,Mol) = Hts(Mol,ito)                    ! Rxn barrier enthalpy
c
c      KEYWORDS
c
c
c        DEFAULTS
c
         NCENT(Mol,Nchan(Mol)) = 1
         Jr = 0
         jtun = 0
         jivr = 0
         itun(Mol,Nchan(Mol)) = 0
         Jread(Mol,Nchan(Mol)) = 1

         kkey = 0

         DO 701 KK = 1 , 5

           IF     (KEYWORD(KK) .EQ. 'ILT') THEN   ! k(E) from Inverse Laplace Transform method
              Jread(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'ilt') THEN   ! k(E) from Inverse Laplace Transform metho Include reverse reaction
              Jread(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'SUM') THEN   ! external sums of states file
              Jread(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'sum') THEN   ! external sums of states file
              Jread(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'RKE') THEN  ! external k(E) file
              Jread(Mol,Nchan(Mol)) = 2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'rke') THEN  ! external k(E) file
              Jread(Mol,Nchan(Mol)) = 2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'REVRKE') THEN  ! external k(E) file used to calculate REVERSE reaction
              Jread(Mol,Nchan(Mol)) = -2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'revrke') THEN  ! external k(E) file used to calculate REVERSE reaction
              Jread(Mol,Nchan(Mol)) = -2
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'REV') THEN   ! Include reverse reaction
              Jr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'rev') THEN   ! Include reverse reaction
              Jr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOREV') THEN
              Jr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'norev') THEN
              Jr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'TUN') THEN   ! Include tunneling
              ntunflag = 1
              jtun = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'tun') THEN   ! Include tunneling
              ntunflag = 1
              jtun = 1  
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOTUN') THEN  ! No tunneling
              jtun = 0  
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'notun') THEN  ! No tunneling
              jtun = 0  
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'SLOW') THEN   ! Include slow IVR
              nivrflag = 1
              jivr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'slow') THEN   ! Include slow IVR
              nivrflag = 1
              jivr = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'FAST') THEN   ! Fast IVR
              jivr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'fast') THEN   ! Fast IVR
              jivr = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'CENT') THEN   ! Include centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'cent') THEN   ! Include centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 1
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'NOCENT') THEN   ! No centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ELSEIF (KEYWORD(KK) .EQ. 'nocent') THEN   ! No centrifugal correction
              NCENT(Mol,Nchan(Mol)) = 0
              kkey = kkey + 1
           ENDIF

 701     CONTINUE

           NSTOP = 0
           IF ( kkey .NE. 5 ) THEN
             write(6,*)'*** LESS THAN 5 REQUIRED KEYWORDS ***'
             write(KOUT,*)'*** LESS THAN 5 REQUIRED KEYWORDS ***'
             NSTOP = 1
           ELSE IF ((Jread(Mol,Nchan(Mol)).eq.0).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: ILT CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: ILT CANNOT BE USED WITH TUN'
             NSTOP = 1
           ELSE IF ((Jread(Mol,Nchan(Mol)).eq.0).AND.
     &           (NCENT(Mol,Nchan(Mol)).EQ.1))THEN
             write(6,*)'KEYWORDS: ILT CANNOT BE USED WITH CENT'
             write(KOUT,*)'KEYWORDS: ILT CANNOT BE USED WITH CENT'
             NSTOP = 1
           ELSE IF ( ( NCENT(Mol,Nchan(Mol)) .EQ. 1 ) .AND.
     &           (TSmom(Mol,Nchan(Mol)) .LE. 0.0))THEN
             write(6,*)'CENT KEYWORD REQUIRES TSmom >0'
             write(KOUT,*)'CENT KEYWORD REQUIRES TSmom >0'
             NSTOP = 1
           ELSE IF((Jread(Mol,Nchan(Mol)).eq.2).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             NSTOP = 1
           ELSE IF((NCENT(Mol,Nchan(Mol)).eq.1).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: CENT CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: CENT CANNOT BE USED WITH TUN'
             NSTOP = 1
           ELSE IF((Jread(Mol,Nchan(Mol)).eq.-2).AND.(jtun.EQ.1))THEN
             write(6,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             write(KOUT,*)'KEYWORDS: RKE CANNOT BE USED WITH TUN'
             NSTOP = 1
           END IF

           IF ( NSTOP .NE. 0 ) THEN
             write(*,*)'***** FATAL: NSTOP .NE. 0'
             write(KOUT,*)'***** FATAL: NSTOP .NE. 0'
             STOP
           END IF
             
          IF (jivr .EQ. 1 ) THEN
             iivr(Mol,Nchan(Mol)) = 1
             READ(KIN,*) KEYTEMP, vivr(Mol,Nchan(Mol)), 
     &          vave(Mol,Nchan(Mol)) ,
     &          pivr( Mol , Nchan(Mol) ) , tivr( Mol , Nchan(Mol) ) ,
     &          ( civr( Mol , Nchan(Mol) , ii ) , ii = 1 , 3 )
          ENDIF

          IF (jtun .EQ. 1 ) THEN
             itun(Mol,Nchan(Mol)) = 1
             READ(KIN,*) KEYTEMP , vimag(Mol,Nchan(Mol))
          ENDIF

         IF ( Jr.EQ.1 ) THEN                    ! Jr=1 for rev rxn
            Jrev(Mol,Nchan(Mol)) = 1
            IF ( ito .GT. NWells ) THEN
               Jrev(Mol,Nchan(Mol)) = 0         ! If 'ito' is an irreversible product molecule
            ELSE
               Nreac = Nreac + 1
               Nchan(ito) = Nchan(ito) + 1
               Jto(ito,Nchan(ito)) = Mol
               TSname(ito,Nchan(ito)) = TS
               TSmom(ito,Nchan(ito)) = RR
               NCENT(ito,Nchan(ito)) = NCENT(Mol,Nchan(Mol))
               TSsym(ito,Nchan(ito)) = j
               TSele(ito,Nchan(ito)) = qele
               TSopt(ito,Nchan(ito)) = l
               Jrev(ito,Nchan(ito)) = 0
               iivr(ito,Nchan(ito)) = iivr(Mol,Nchan(Mol))
               vivr(ito,Nchan(ito)) = vivr(Mol,Nchan(Mol))
               vimag(ito,Nchan(ito)) = vimag(Mol,Nchan(Mol))
               iivr(ito,Nchan(ito)) = iivr(Mol,Nchan(Mol))
               pivr(ito,Nchan(ito)) = pivr(Mol,Nchan(Mol))
               tivr(ito,Nchan(ito)) = tivr(Mol,Nchan(Mol))
               civr(ito,Nchan(ito),1) = civr(Mol,Nchan(Mol),1)
               civr(ito,Nchan(ito),2) = civr(Mol,Nchan(Mol),2)
               civr(ito,Nchan(ito),3) = civr(Mol,Nchan(Mol),3)
               itun(ito,Nchan(ito)) = itun(Mol,Nchan(Mol))
              
               IF ( Jread(Mol,Nchan(Mol)).EQ.2 ) THEN
                  Jread(ito,Nchan(ito)) = -2
                                        ! RateArray will read k(E) for forward rxn and then calculate reverse k(E)
               ELSE
                  Jread(ito,Nchan(ito)) = Jread(Mol,Nchan(Mol))
               ENDIF
               Afac(ito,Nchan(ito)) = AA
               Eo(ito,Nchan(ito)) = Hts(ito,Mol) - HMol(ito)
            ENDIF
         ENDIF
           

 800  CONTINUE

c------------------------------------------------------------------------------------------
c   COMPUTE RATE CONSTANTS, ETC
c------------------------------------------------------------------------------------------

      IF (Nforward .GT. 0 ) CALL RateArray(Temp)       ! Calculate rate constants

      EM = (Isize-imax1-2)*Egrain2           ! Top energy in double array
 
      DO j = 1 , NWells
         IF ( Nchan(j) .EQ. 0 ) THEN         ! New up-collision method: normalization at imax1 (if no rxn channel)
           CNORMref(j) = CNORM(j,imax1)
           Eref(j) = (imax1-1)*Egrain1
         ELSE
           CNORMref(j) = CNORM(j,imax1)
           Eref(j) = Emax1
           DD = EM                           ! Top energy in double array
           DO i = 1 , Nchan(j)
             IF ( Eor(j,i) .LT. DD .AND. Eor(j,i) .GT. Emax1 ) THEN ! New up-collision method: normalization at Eor
                DD = Eor(j,i)            ! The lowest reaction threshold that is higher than Emax1
                Eref(j) = DD
                CNORMref(j) = DOUBNTERP( DD , j , CNORM )
             ENDIF
           END DO
         ENDIF
      END DO

      DO 900 j = 1 , NWells
         IF ( Nchan(j) .NE. 0 ) THEN
           DO 850 i = 1 , Nchan(j)            
             kcol = CollK(j)
             if(itun(j,i).eq.1)then
               CALL QKinftun(j,i,Temp,Kinf(j,i),Ainf(j,i),Einf(j,i))    ! high-pressure rate constant w/ tunneling effects
             else
               CALL QKinf(j,i,Temp,Kinf(j,i),Ainf(j,i),Einf(j,i),       ! high-pressure rate constant w/o tunneling effects
     &                   kcol,kosc)
             end if
!           IF(Press(j,i).ne.1) Kinf(j,i)=Kinf(j,i)*Press(j,i)
 850       CONTINUE
         END IF
 900  CONTINUE

c------------------------------------------------------------------------------------------
c   WRITE OUT COLLISIONAL NORMALIZATION, UP-TRANSITION PROBABILITIES, ETC
c------------------------------------------------------------------------------------------

      DO 600 Mol = 1 , NWells
         Rob_1 = -1.0
         XRANST = PDOWN(Rob_1,Rob_1,Mol,Temp,SCALE)
         DO ki = 7 , 9
           WRITE (ki,99057) Mol , ADJUSTR(MolName(Mol)),
     &      Eref(Mol), (DC(Mol,i),i=1,8)
           WRITE (ki,99027) (LI(Mol,i),i=1,5)
         END DO
 600  CONTINUE
 
      WRITE (KARY,99030)
      WRITE (KARY,99012) (Mol,Mol,Mol=1,NWells)
      DO 700 II = 1 , Isize
         IF ( II.LE.imax1 ) THEN
            E = (II-1)*Egrain1
         ELSE
            E = (II-imax1-1)*Egrain2
         ENDIF
         WRITE (KARY,99011) E , 
     &        (CNORM(Mol,II),COLLUP(Mol,II),Mol=1,NWells)
 700  CONTINUE
 
c------------------------------------------------------------------------------------------
c   WRITE OUT RATE CONSTANTS, ETC
c------------------------------------------------------------------------------------------
      IF ( Nreac .GT. 0 ) THEN
      DO ki = 7 , 9
       WRITE (ki,99013)  Nreac
       WRITE (ki,99121) (FROMTO,i=1,Nreac)
       WRITE (ki,99028) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99034) ((ADJUSTR(TSname(j,i)),i=1,Nchan(j)),
     &   j=1,NWells)
       WRITE (ki,99045) ((TSmom(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99046) ((TSsym(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99047) ((TSele(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99048) ((TSopt(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99026) ((Path(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99023) ((Hts(j,Jto(j,i)),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99025) ((Hts(j,Jto(j,i))/jtocm,i=1,Nchan(j)),j=1,
     &                   NWells)
       WRITE (ki,99024) ((Hts(j,Jto(j,i))/caltocm,i=1,Nchan(j)),j=1,
     &                   NWells)
       WRITE (ki,99021) ((Eo(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99022) ((Eo(j,i)/jtocm,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99020) ((Eo(j,i)/caltocm,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99029) ((Kinf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99049) ((Ainf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99050) ((Einf(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99051) ((Einf(j,i)*jtocal,i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99086) ((kosc(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99115) ((NCENT(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99114) ((Eor(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99052) ((Jread(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99088) ((vimag(j,i),i=1,Nchan(j)),j=1,NWells)
       IF ( nivrflag .EQ. 1 ) THEN
c        WRITE (ki,99080) ((iivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99081) ((vivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99087) ((vave(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99082) ((pivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99186) ((tivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99083) ((civr(j,i,1),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99084) ((civr(j,i,2),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99085) ((civr(j,i,3),i=1,Nchan(j)),j=1,NWells)
       ENDIF
       IF ( Pressflag .GT. 0 ) WRITE (ki,99089)
     &        ((Press(j,i),i=1,Nchan(j)),j=1,NWells)
       WRITE (ki,99053)
      END DO

      WRITE (KARY,99013)  Nreac
      WRITE (KARY,99113) ((Eor(j,i),i=1,Nchan(j)),j=1,NWells)
      WRITE (KARY,99121) (FROMTO,i=1,Nreac)
      WRITE (KARY,99016) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)

!	combine tunneling and nontunneling k(E) into KRate for output purposes

        do i=1,NWells
          do k=1,Nchan(i)
            do II=1,Isize+imax1
              if(II.le.imax1*2)then
                E=(II-1)*Egrain1-(imax1*Egrain1)
              else
                E=(II-imax1*2-1)*Egrain2
              end if
              if(itun(i,k).eq.1)then
                KRate(i,k,II)=TRate(i,k,II)
              else
                if(E.lt.0.0d0)then
                  KRate(i,k,II)=-5000.0d0     !ensures log[KRate(i,k,II)] is 0 below the rxn barrier
                else
                  III=II-imax1                !III index ensures Rate array for the nontunneling case
                  KRate(i,k,II)=Rate(i,k,III) !is stored correctly in the KRate array
                end if
              end if
            end do
          end do
        end do

!	output all k(E)

      IF ( ntunflag .EQ. 1 ) THEN                ! IF TUNNELING, print k(E) below critical energy Eo

        do II=1,Isize+imax1
          if(II.le.imax1*2)then
            E=(II-1)*Egrain1-(imax1*Egrain1)
          else
            E=(II-imax1*2-1)*Egrain2
          end if
          WRITE(KARY,99018)E,
     &         ((EXP(KRate(j,i,II)),i=1,Nchan(j)),j=1,NWells)
        end do

      ELSE                                       ! IF NO TUNNELING, print k(E) starting at critical energy Eo

        do II= imax1+1 , Isize+imax1
          if(II.le.imax1*2)then
            E=(II-1)*Egrain1-(imax1*Egrain1)
          else
            E=(II-imax1*2-1)*Egrain2
          end if
          WRITE(KARY,99018)E,
     &         ((EXP(KRate(j,i,II)),i=1,Nchan(j)),j=1,NWells)
        end do

      ENDIF
      
      ENDIF     ! End printing when Nreac .GT. 0

c-------------------------------------------------------------------------------------
c-------------------------------------------------------------------------------------
c.......................................................................
c	Ntrials	= number of trials
c	Tspec	= time specification (CHARACTER*4)
c		= 'TIME': Tread = Tlim (max time)
c		= 'COLL': Tread = max collisions, from which Tlim is calculated 
c	Tread	= maximum simulated time
c	KEYTYPE	= KEYWORD  for type of initial distribution
c		= DELTA    for Monoenergetic at energy Einit          (Icalc=1)
c		= THERMAL  for Thermal with energy offset Einit       (Icalc=2)
c		= CHEMACT  for Chemical activation from 'product' #IR (Icalc=3)
c               = EXTERNAL for read from external file                (Icalc=4)
c	Molinit	= index of initial molecule
c	IR	= origin of initial molecule, if produced via chemical activation
c	Einit	= initial energy (relative to ZPE of Molinit)
c.......................................................................
c
      READ (KIN,*) Ntrials , Tspec , Tread , KEYTEMP , Molinit , IR , 
     &                Einit
      IF ( KEYTEMP.EQ.      'delta'   ) THEN
         Icalc = 1
        ELSEIF ( KEYTEMP.EQ.'DELTA'   ) THEN
         Icalc = 1
        ELSEIF ( KEYTEMP.EQ.'thermal' ) THEN
         Icalc = 2
        ELSEIF ( KEYTEMP.EQ.'THERMAL' ) THEN
         Icalc = 2
        ELSEIF ( KEYTEMP.EQ.'chemact' ) THEN
         Icalc = 3
        ELSEIF ( KEYTEMP.EQ.'CHEMACT' ) THEN
         Icalc = 3
        ELSEIF ( KEYTEMP.EQ.'external' ) THEN
         Icalc = 4
        ELSEIF ( KEYTEMP.EQ.'EXTERNAL' ) THEN
         Icalc = 4
        ELSE
         write(6,*)'UNKNOWN KEYWORD USED TO SPECIFY INITIAL DIST.'
         write(KOUT,*)'UNKNOWN KEYWORD USED TO SPECIFY INITIAL DIST.'
         
         STOP
      ENDIF

      IF ( Eunits.EQ.kcal ) THEN
         Einit = Einit*caltocm          ! Convert kcal/mole to cm-1
        ELSEIF ( Eunits.EQ.kjou ) THEN
         Einit = Einit*jtocm          ! Convert kJ/mole to cm-1
      ENDIF
  
         WRITE (KSMM,99059) Ntrials
         IF ( Icalc.EQ.1 ) THEN
          WRITE (KSMM,99165) Temp , ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.2 ) THEN
          WRITE (KSMM,99166) Tvib , Temp , ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.3 ) THEN
          WRITE (KSMM,99167) Tvib , Temp , ADJUSTR(MolName(Molinit)) ,
     &                         MolName(IR) , IDUM0 , IDUM
         ELSEIF ( Icalc.EQ.4 ) THEN
          WRITE (KSMM,99168) Temp , ADJUSTR(MolName(Molinit)) ,
     &                         IDUM0 , IDUM
         ENDIF
 
         WRITE (KSMM,99063) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
         WRITE (KSMM,99060) (HMol(i),i=1,NWells)
         WRITE (KSMM,99161) POUT , (i,i,i=1,NWells+NProds)

c
c	Initialize Estart to calculate cumulative distribution
c
      E = Estart(0,Icalc,IR,Molinit,Einit,Tvib)

      Iwell=Molinit    ! save index for the well that the reaction will be initiated
                       ! from so that the correct Eor(Iwell,1) can be printed before the
                       ! cumulative distribution function info in multiwell.array
 
c @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
c
c	START PRESSURE STEPS & TRIALS
c
c
      DO 1200 Ip = 1 , Np               ! Pressure steps
         numbdens = P(Ip)*7.2428D+21/Temp
         CollFreq = CollK(Molinit)*numbdens

         IF ( Tspec.EQ.'TIME' .OR. Tspec.EQ.'time' ) THEN           ! Specify max time
            Tlim = Tread
         ELSEIF ( Tspec.EQ.'COLL' .OR. Tspec.EQ.'coll' ) THEN       ! Specify max number of collisions
           IF (CollFreq .GT. 0.0d+00) THEN
              Tlim = Tread/CollFreq
           ELSE
              Tlim = 100.
           ENDIF
         ENDIF

         Tstep = Tlim/(Ntime-1)
         IDUM0 = IDUM
 
         CALL INITIAL                   ! Initialize bookkeeping arrays
 
         M = 0
         WRITE (*,99164) PP(Ip)
         WRITE (*,99064) M , M
 
         DO 1050 M = 1 , Ntrials
            E = Estart(1,Icalc,IR,Molinit,Einit,Tvib)
            Igo = 1
            Mol = Molinit
            Time = 0.0D+00               ! Initialize
            DO WHILE ( Igo.EQ.1 )
               CALL STEPPER(Mol,E,Temp,numbdens,IPP,Delt)
               CALL BOOK1(Mol,E,Temp,IPP,Time,Delt,Igo)
               IF ( Time.GT.Tlim ) Igo = 0
            ENDDO
            Test = 20.*M/Ntrials - (20*M/Ntrials)
            IF ( ABS(Test).LT.1.E-06 ) WRITE (*,99064) M , 
     &           5*(20*M/Ntrials)
 1050    CONTINUE
c
c
c	Massage booked results and store output
c
         CALL DateTime(KOUT)
         CALL DateTime(KARY)
 
         DO ki = 8 , 11
          WRITE (ki,99059) Ntrials
          IF ( Icalc.EQ.1 ) THEN
            WRITE (ki,99065) Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                    Einit , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.2 ) THEN
            WRITE (ki,99066) Tvib , Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                         Einit , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.3 ) THEN
            WRITE (ki,99067) Tvib , Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) , 
     &                    ADJUSTR(MolName(IR)) , IDUM0 , IDUM
            ELSEIF ( Icalc.EQ.4 ) THEN
            WRITE (ki,99868) Temp , PP(Ip) , POUT , numbdens , 
     &                    CollFreq , Eref(Molinit), 
     &                    ADJUSTR(MolName(Molinit)) ,
     &                    IDUM0 , IDUM
         ENDIF
        END DO
 
       DO ki = 10, 11
         WRITE (ki,99030)
         WRITE (ki,99034) ((ADJUSTR(TSname(j,i)),i=1,Nchan(j)),
     &       j=1,NWells)
         WRITE (ki,99045) ((TSmom(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99046) ((TSsym(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99047) ((TSele(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99048) ((TSopt(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99026) ((Path(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99023) ((Hts(j,Jto(j,i)),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99024) ((Hts(j,Jto(j,i))/caltocm,i=1,Nchan(j)),j=1,
     &                      NWells)
         WRITE (ki,99021) ((Eo(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99022) ((Eo(j,i)/jtocm,i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99020) ((Eo(j,i)/caltocm,i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99029) ((Kinf(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99049) ((Ainf(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99050) ((Einf(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99051) ((Einf(j,i)*jtocal,i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99086) ((kosc(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99115) ((NCENT(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99114) ((Eor(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99052) ((Jread(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99081) ((vivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99087) ((vave(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99082) ((pivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99186) ((tivr(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99083) ((civr(j,i,1),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99084) ((civr(j,i,2),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99085) ((civr(j,i,3),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99088) ((vimag(j,i),i=1,Nchan(j)),j=1,NWells)
        IF ( Pressflag .GT. 0 ) WRITE (ki,99089)
     &        ((Press(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (ki,99053)
       END DO 
c
c	Calculate populations of each molecule
c
         WRITE (KOUT,99063) (ADJUSTR(MolName(i)),i=1,NWells+NProds)
         WRITE (KOUT,99060) (HMol(i),i=1,NWells)
         WRITE (KOUT,99061) (i,i,i=1,NWells+NProds)
         WRITE (KFLX,99015)  Nreac , (FROMTO,i=1,Nreac)
         WRITE (KFLX,99017) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)
         WRITE (KRAT,99014) Nreac , (FROMTO,i=1,Nreac)
         WRITE (KRAT,99017) ((j,Jto(j,i),i=1,Nchan(j)),j=1,NWells)
         DO 1100 j = 1 , Ntime          ! Start of time-loop
            Time = (j-1)*Tstep
            NumTot = 0
            DO 1060 i = 1 , NWells + NProds
               FracMol(i) = 0.0D+00
               AveE(i) = 0.0D+00
               NumTot = NumTot + Num(i,j)
 1060       CONTINUE
 
            DO 1080 Mol = 1 , NWells + NProds
               FracMol(Mol) = DBLE(Num(Mol,j))/DBLE(NumTot)             ! Average population in each well
               Var(Mol) = SQRT(FracMol(Mol)*(1.0-FracMol(Mol))/DBLE(
     &                    NumTot))
               IF ( Mol.LE.NWells .AND. Num(Mol,j).GT.0 ) THEN
                  AveE(Mol) = EMol(Mol,j)/Num(Mol,j)
                                               ! Average E in each well
                  DO 1065 k = 1 , Nchan(Mol)
                    IF (Nuni(Mol,j).GT.0) THEN
                         kuni(Mol,k,j) = kuni(Mol,k,j)/Nuni(Mol,j)  ! average rate constants
                         Flux(Mol,k,j) = kuni(Mol,k,j)*FracMol(Mol) ! relative reaction flux via a given path
                      ELSE
                         kuni(Mol,k,j) = 0.0d+00
                         Flux(Mol,k,j) = 0.0d+00
                    ENDIF
 1065             CONTINUE
               ENDIF
 1080       CONTINUE
 
            WRITE (KOUT,99062) Time , Time*CollFreq , 
     &                         (FracMol(Mol),Var(Mol),AveE(Mol),Mol=1,
     &                         NWells+NProds)
            WRITE (KRAT,99019) Time , 
     &                         ((kuni(Mol,k,j),k=1,Nchan(Mol)),Mol=1,
     &                         NWells)  ! average rate constants
            WRITE (KFLX,99019) Time , 
     &                         ((Flux(Mol,k,j),k=1,Nchan(Mol)),Mol=1,
     &                         NWells)  ! average rate constants
 1100    CONTINUE                       ! End of time-loop
 
         WRITE (KSMM,99062) PP( Ip ) , numbdens ,
     &                      (FracMol(Mol),Var(Mol),AveE(Mol),Mol=1,
     &                      NWells+NProds)
      
         OPEN (KDIS,FILE=NameDIST,STATUS='UNKNOWN')     ! vib distributions vs. time

         WRITE (KDIS,99001) AVERSION , ADATE , AVERSION , ADATE
         CALL DateTime(KDIS)
         WRITE (KDIS,99003) TITLE
         WRITE (KDIS,99044) Egrain1 , Emax1 , imax1 , Egrain2 , Emax2 , 
     &                   (Isize-imax1)
         WRITE (KDIS,99070) Temp , Tvib , P(Ip)     ! Write out energy distributions

         Tstep = Tlim/(Mtime-1)
         DistStep = Emax2/(Ndist-1) 
         DO 1150 Mol = 1 , NWells
          DO 1120 j = 1 , Mtime
           DO 1110 k = 1 , Ndist
            IF ( EDIST(Mol,k,j).GT.0.0D+00 ) THEN
             WRITE(KDIS,99071) Mol, (j-1)*Tstep, (j-1)*Tstep*Collfreq, 
     &            (k-0.5)*DistStep , DBLE(EDIST(Mol,k,j))/DBLE(Ntrials)    ! write non-zero elements
            ENDIF
 1110      CONTINUE
 1120     CONTINUE
 1150    CONTINUE

! this do loop ensures that the correct Eor and itun information are used when
! printing out the distributions below

        do i=1,Nchan(Molinit)
          IF ( Jto(Molinit,i).EQ.IR ) ichan = i
        end do

         IF ( Icalc.GT.1 ) THEN                              ! If not a delta function
            WRITE (KARY,99068)                               ! chemical activation or thermal distribution
            DO 1160 II = 1 , Jsize
              if (Icalc.eq.2 .OR. Icalc.eq.4) then
                 E = Einit + (II-1)*Hstart                   ! start at Einit  for THERMAL and EXTERNAL
              else if (Icalc.eq.3)then
                if(itun(Molinit,ichan).eq.0)then
                  E = Eor(Molinit,ichan) + (II-1)*Hstart     ! start at reaction threshold for CHEMACT w/o tunneling 
                else
                  Einit = Eo(Molinit,ichan) - Emax1
                  IF ( Einit .LT. 0.0 ) Einit = 0.0
                  E = Einit + (II-1)*Hstart                  ! start at Egrain1*imax1 below the reaction threshold for 
                end if                                       ! for CHEMACT with tunneling
              else
                 continue
              end if
              IF ( II.GT.1 ) Nstart(II) = Nstart(II) + Nstart(II-1)
              WRITE (KARY,99069) E , Pstart(II) , 
     &            DBLE(Nstart(II))/DBLE(Ntrials)
 1160       CONTINUE
         ENDIF
 1200 CONTINUE                          ! End pressure step loop
c
c	FORMAT STATEMENTS
c
99001 FORMAT (
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'//8x,
     &'                                John R. Barker'/8x,
     &'      MultiWell-',A6,'          University of Michigan'/8x,
     &'                                Ann Arbor, MI 48109-2143'/8x,
     &'        ',A8,'                jrbarker@umich.edu'/8x,
     &'                                (734) 763 6239'//8x,
     &'      http://aoss.engin.umich.edu/multiwell/'//,
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'//8x,
     &'Suggested Literature Citation:'//8x,
     &'a) MultiWell-',A6,' Software, ',A8,', designed and'/11x,
     &'maintained by J.R. Barker with contributors N.F. Ortiz,'/11x,
     &'J.M. Preses, L.L. Lohr, A. Maranzana, P.J. Stimac, T. L.'/11x,
     &'Nguyen, and T. J. Dhilip Kumar, University of Michigan,'/11x,
     &'Ann Arbor, MI; http://aoss.engin.umich.edu/multiwell/',//8x,
     &'b) J.R. Barker, Int. J. Chem. Kinetics, 33, 232-45 (2001)',//8x,
     &'c) J.R. Barker, Int. J. Chem. Kinetics, 41, 748-763 (2009)',//8x,
     &//,
     &'%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     &%%%%%%%%%%%%%'/)
99002 FORMAT (/'Isize greater than maximum allowed: ',I5,
     &        ' elements.'/'****** Hit RETURN to terminate run *******')
99003 FORMAT (A100)
99004 FORMAT (60x,'DATE: ',I2.2,'/',I2.2,'/',I4.4,/,60x,'TIME: ',I2.2,
     &        ':',I2.2,':',I2.2//)
99005 FORMAT (//I2,1x,'Stable Molecules',//,
     &'No. Name             delH   2D-moment Symm Vibs Rots     ZPE     
     &  beta    <freq>  Thermal<E>')
99006 FORMAT (I2,2x,A10,1x,f10.1,2x,f10.1,2x,I2,2x,I2,2x,I2,1x,0pf10.1,
     &        1x,f10.4,1x,f10.3,1x,f10.1)
99007 FORMAT (I2,2x,A10,1x,f10.1,2x,f10.1,2x,I2,2x,
     &        'Densities from an external file',9x,f10.1)
99008 FORMAT (I2,2x,A10,1x,'      Irreversible product species')
99009 FORMAT (/'Units: 2D-moment : amu Ang^2',
     &         '       Thermal<E>: cm-1 (at Translational T)')
99010 FORMAT (/,5X,'Densities of States',/'     Energy',1x,75(1x,A10))
99011 FORMAT (1x,f10.1,75(1x,1pe10.3,1x,1pe10.3))
99012 FORMAT (/'     Energy',75('   Norm(',I2.2,') UpProb(',I2.2,')'))
99013 FORMAT (/,5X,I3,' k(E) RATE CONSTANTS')
99113 FORMAT (7X,'(with centrifugal corrections) vs. active energy',/,/,
     &           'Eo(cent):',75(1x,f10.1),/)
99114 FORMAT ('Eocent/cm-1:',75(1x,f10.1))
99115 FORMAT ('Centr.Corr.:',75(1x,I10))
99014 FORMAT (/,5X,I3,' Average Rate Constants',/11X,75(1X,A10))
99015 FORMAT (/,5X,I3,' Relative Reactive Flux',/12X,75(1X,A10))
99016 FORMAT (' Energy',3x,75(4X,I2.2,'...',I2.2))
99017 FORMAT ('   Time',4x,75(4X,I2.2,'...',I2.2))
99018 FORMAT (1x,f10.1,75(1x,1pe10.3))
99019 FORMAT (2x,1pe10.3,75(1x,1pe10.3))
99020 FORMAT ('   Eo(kcal):',75(1x,f10.2))
99121 FORMAT (11X,75(1X,A10))
99021 FORMAT ('   Eo(cm-1):',75(1x,f10.1))
99022 FORMAT ('     Eo(kJ):',75(1x,f10.2))
99023 FORMAT (' delH(cm-1):',75(1x,f10.1))
99024 FORMAT (' delH(kcal):',75(1x,f10.1))
99025 FORMAT ('   delH(kJ):',75(1x,f10.1))
99026 FORMAT ('Rx_Path*Qel:',75(1x,f10.1))
99027 FORMAT (A100)
99028 FORMAT ('       ',3x,75(4X,I2.2,'...',I2.2))
99029 FORMAT ('     k(inf):',75(1x,1pe10.2))
99030 FORMAT (/)
99031 FORMAT (/,'TEMPERATURES:',/,
     &          '        Translational Temperature =',
     &        f8.1,1x,'K'/,    
     &          '  Initial Vibrational Temperature =',
     &        f8.1,1x,'K'///,I3,' WELLS and',I3,' PRODUCTS'/)
99032 FORMAT (/,I5,' PRODUCT SETS',/)
99033 FORMAT (' lowest_vib:',75(1x,f10.1))
99034 FORMAT ('    TS_Name:',75(1x,A10))
99035 FORMAT (' Molec_Name:',75(1x,A10))
99036 FORMAT ('  Molec_No.:',75(1x,I10))
99041 FORMAT (' Thermal<E>:',75(1x,f10.1))
99044 FORMAT (/'PARAMETERS FOR DOUBLE-ARRAYS:'/
     &    '        Grain (cm-1)      Maximum Energy    Number of grains'
     &    /2(2F20.3,i20/))
99045 FORMAT ('  2D-moment:',75(1x,f10.1))
99046 FORMAT ('   ext_symm:',75(1x,I10))
99047 FORMAT ('Qel_partfxn:',75(1x,F10.2))
99048 FORMAT ('   opt_isom:',75(1x,I10))
99049 FORMAT ('     A(inf):',75(1x,1pe10.2))
99050 FORMAT (' E(inf)kcal:',75(1x,f10.2))
99051 FORMAT ('   E(inf)kJ:',75(1x,f10.2))
99052 FORMAT ('     Source:',75(1x,I10))
99053 FORMAT (
     &'Source:  0 => ILT method; 1 => sums of states file;',
     &' 2 => k(E) file; -2 => k(E) file for reverse rxn')
99054 FORMAT ('Temperature (K) =',f10.2,/,'Pressure (bar) =',1pe10.3/)
99055 FORMAT ('Lennard-Jones Parameters & Pdown Models    ',
     &        '[rate constants k(LJ) and k(QM) units: cc/sec]',//,
     &'Mol   Sig(Ang)      Eps/k MolWt(amu)     NetSig   NetEps/k    Red
     &Mass     k(LJ)    k(QM)     Used')
99056 FORMAT (1x,I2,6(1X,f10.3),3(1x,1pe10.3))
99057 FORMAT (/,'Well number: ',I2,5x,'Name: ',A10,/,
     &        'Reference energy for collision frequency:',0pf8.1,' cm-1',
     &        /,'Collision parameters:',/,8(1X,1pe10.3))
99058 FORMAT ('Inelastic collision parameters:',3(1X,1pe10.3))
99059 FORMAT (/'RESULTS FOR',I8,' TRIALS')
99060 FORMAT ('        deltaH(cm-1): ',75(5x,0pf10.1,18x))
99061 FORMAT ('   Time  Collisns(Mol=1)',
     &        75(3x,'Fract(',I2.2,') +/-Error  <Evib(',I2.2,')>'))
99161 FORMAT (3X , A4 , '   molecules/cc  ',
     &        75(3x,'Fract(',I2.2,') +/-Error  <Evib(',I2.2,')>'))
99062 FORMAT (2(1x,1pe10.3),75(3x, 1pe11.3 , 1pe9.1 , 0pf8.0,2x))
99063 FORMAT ('                Name:    ',75(5x,A10,18x))
99064 FORMAT (1x,I10,'  Trials = ',I5,' %')
99164 FORMAT (/,1x,'Pressure = ',1pe10.3)
99065 FORMAT ('****Monoenergetic****'/'    Ttrans  = ',f6.1,
     &        ' K',/,'    Press   = ',1pe10.3,1x,A4,/,'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Initial Molecule: ',A10,/,
     &       'E(initial) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99066 FORMAT ('****Shifted Thermal (at Tvib)****'/'     Tvib   = ',f6.1,
     &        ' K',/,'     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,
     &        1x,A4,/,'          N = ',1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Initial Molecule: ',A10,/,'E(shift) =',
     &        0pf10.1,' cm-1'/'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99067 FORMAT ('****Chemical Activation (at Tvib)****'/'     Tvib   = ',
     &        f6.1,' K',/,'     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,1x,A4,/'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf8.1,' cm-1',/,
     &        'Excited Molecule: ',
     &        A10,/,'   Produced from: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99868 FORMAT ('**** ENERGY DISTRIBUTION FROM EXTERNAL FILE ****'/,
     &        '     Ttrans = ',f6.1,' K',/,'     Press  = ',
     &        1pe10.3,1x,A4,/'          N = ',
     &        1pe10.3,' /cm3',/,
     &        'Coll. Freq. = ',1pe10.3,' /s at E =',0pf10.1,' cm-1',/,
     &        'Initial Molecule: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99165 FORMAT (/,/,'****Monoenergetic****'/'    Ttrans  = ',f6.1,
     &        ' K',/,'Initial Molecule: ',A10,/,'E(initial) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99166 FORMAT (/,/,'****Shifted Thermal (at Tvib)****'/'     Tvib   = ',
     &        f6.1,
     &        ' K',/,'     Ttrans = ',f6.1,' K',/,'Initial Molecule: ',
     &        A10,/,'E(shift) =',
     &        0pf10.1,' cm-1',/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99167 FORMAT (/,/,'****Chemical Activation (at Tvib)****'
     &        /'     Tvib   = ',
     &        f6.1,' K',/,'     Ttrans = ',f6.1,' K',/,
     &        'Excited Molecule: ',
     &        A10,/,'   Produced from: ',
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99168 FORMAT (/,/,'**** Energy Distribution from an EXTERNAL File ****',
     &        /,'     Ttrans = ',f6.1,' K',/,
     &        A10,/,'Starting random number seed = ',
     &        I10,/,'Ending   random number seed = ',I10,/)
99068 FORMAT (/'CUMULATIVE DISTRIBUTION FXNS FOR STARTING ENERGIES'//
     &        '    ENERGY   P(calc)    P(actual)')
99069 FORMAT (1x,f10.1,2(1x,1pe10.3))
99070 FORMAT (/'Energy Distribution (non-zero elements)',/,
     &        '  (Energy at center of energy bin)',/,' Ttrans=',f6.1,
     &        1x,'K',5x,'Tvib=',f6.1,1x,'K',5x,'P= ',1PE10.3,1x,'bar',/,
     &        ' Molecule  Time  Collisions VibEnergy  Distrib')
99071 FORMAT (1x,I3,1x,1pe10.3,2(1x,0pf10.1),1x,1pe10.3)
99072 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' cm-1')
99076 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' kJ/mol')
99077 FORMAT (8x,I3,2x,A10,2x,'delH =',f10.1,' kcal/mol')
99073 FORMAT (//,'COLLISION PARAMETERS FOR ',I2,' WELLS')
99074 FORMAT (/,'INPUT DATA FILE CHECK: ',I2,' REACTION CHANNELS'/
     &'#  Mol ito TS          RR       j       Qel   l   AA         ',
     &'EE       KEYWORDS')
99075 FORMAT(3(I2,1X),A10,1X,1PE10.2,1X,I2,1X,0PF10.2,1X,I2,
     &  2(1PE10.2,1X),5(A7,1x))
99080 FORMAT ('IVR: 0 or 1:',75(1x,I10))
99081 FORMAT ('IVR  v(IVR):',75(1x,f10.1),' cm-1')
99087 FORMAT ('IVR  v(ave):',75(1x,f10.1),' cm-1')
99082 FORMAT ('IVR  coll k:',75(1x,1pe10.3))
99186 FORMAT ('IVR  thresh:',75(1x,f10.1))
99083 FORMAT ('IVR civr(1):',75(1x,1pe10.3))
99084 FORMAT ('IVR civr(2):',75(1x,1pe10.3))
99085 FORMAT ('IVR civr(3):',75(1x,1pe10.3))
99086 FORMAT (' koSC(cc/s):',75(1x,1pe10.2))
99088 FORMAT ('TUN   vimag:',75(1x,0pf10.1))
99089 FORMAT ('React. [B] :',75(1x,1pe10.2))
      END
**==nterp.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      SUBROUTINE NTERP2(E,IE,X)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c***********************************************************************
C
C	Calculate nearest element in Double Array of size Isize
c
c	'Double arrays' have two sections:
c		segment 1 consists of equally spaced (Egrain1) values ranging
c			from E=0 to Emax1
c		segment 2 consists of equally spaced values from E=0 to Emax2;
c			the spacing depends on the number of arrays elements
c			remaining: Isize-Emax1/Egrain1-1
c
c	E	= energy (units: cm-1)
c	Egrain1	= energy grain of lower segments in 'double arrays' (units: cm-1)
c	Egrain2	= energy grain of upper segments in 'double arrays' (units: cm-1)
c	Emax1	= maximum energy of 1st segment of double arrays (units: cm-1)
c	Emax2	= maximum energy of 2nd segment of double arrays (units: cm-1)
c	IE	= nearest element index
c	X	= fraction of element difference from IE (units: dimensionless)
c
c***********************************************************************
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      DOUBLE PRECISION E , X
      INTEGER IE
      SAVE 

! code modified to correctly interpolate k(E) array with Isize+imax1
! elements; imax1 of which are below the barrier

        if(E.le.Emax1)then
          IE=1+INT(E/Egrain1)+Imax1
          if(IE.lt.2) IE=2
          if(IE.gt.(2*Imax1-1)) IE=2*Imax1-1
          X=E/Egrain1-(IE-1.0)+Imax1
        else  
         IE = 2*imax1 + 1 + NINT(E/Egrain2)       ! Nearest integer
          if(IE.ge.Isize+Imax1) IE=Isize+Imax1-1
          if(IE.le.(2*Imax1+1)) IE=2*Imax1+2
         X = E/Egrain2 - (IE-2*imax1-1.0)
      end if
      RETURN
      END
**==nterp.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      SUBROUTINE NTERP(E,IE,X)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c***********************************************************************
C
C	Calculate nearest element in Double Array of size Isize
c
c	'Double arrays' have two sections:
c		segment 1 consists of equally spaced (Egrain1) values ranging
c			from E=0 to Emax1
c		segment 2 consists of equally spaced values from E=0 to Emax2;
c			the spacing depends on the number of arrays elements
c			remaining: Isize-Emax1/Egrain1-1
c
c	E	= energy (units: cm-1)
c	Egrain1	= energy grain of lower segments in 'double arrays' (units: cm-1)
c	Egrain2	= energy grain of upper segments in 'double arrays' (units: cm-1)
c	Emax1	= maximum energy of 1st segment of double arrays (units: cm-1)
c	Emax2	= maximum energy of 2nd segment of double arrays (units: cm-1)
c	IE	= nearest element index
c	X	= fraction of element difference from IE (units: dimensionless)
c
c***********************************************************************
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      DOUBLE PRECISION E , X
      INTEGER IE
      SAVE 
 
 
      IF ( E.LE.Emax1 ) THEN
         IE = 1 + INT(E/Egrain1)                
         IF ( IE.LT.2 ) IE = 2
         IF ( IE.GT.(imax1-1) ) IE = imax1 - 1
c         X = E/Egrain1 - (IE-1)
         X = E/Egrain1 - (IE-1.0)
      ELSE
         IE = imax1 + 1 + NINT(E/Egrain2)       ! Nearest integer
         IF ( IE.GE.Isize ) IE = Isize - 1
         IF ( IE.LE.imax1+1 ) IE = imax1 + 2
c         X = E/Egrain2 - (IE-imax1-1)
         X = E/Egrain2 - (IE-imax1-1.0)
      ENDIF
 
      RETURN
      END
 
      DOUBLE PRECISION FUNCTION PDOWN(E,EE,Mol,TEMPI,Hs)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2005 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c      REVISIONS
c
c      12/2004  Added two new energy transfer models (#12 and #13)
c      12/2004  General overhaul to improve speed and accuracy, especially at 
c               low energies.
c
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c	PDOWN	= step-size probability density distribution, not normalized,
c		for DOWN-steps:  E > EE
c	E	= initial energy
c	EE	= final energy
c	Mol	= index number for a molecule
c	TEMPI	= translational temperature
c	Hs	= characteristic 'average' step size corresponding to PDOWN;
c		used in SUBROUTINEs COLNORM and COLSTEP to determine integration
c		step-sizes.
c
c	ITYPE (stored in a COMMON) is a flag used to designate collision model:
c
c	ITYPE = 1  for Biexponential Model
c	ITYPE = 2  for Density-weighted Biexponential Model
c         ITYPE = 3  for Off-set Gaussian with constant offset and E-dependent width
c	ITYPE = 4  for Biexponential Model with energy-dependent fraction
c	ITYPE = 5  for Generalized Gaussian with energy-dependent exponent
c	ITYPE = 6  for Generalized Gaussian plus Exponential term
c	ITYPE = 7  for Weibull Model
c	ITYPE = 8  for Lorentzian Step-Ladder Model
c	ITYPE = 9  for Exponential+Elastic Model
c	ITYPE = 10 for Klaus Luther's empirical function
c	ITYPE = 11 for Triplet state empirical function
c	ITYPE = 12 for Exponential Model with Alpha(E)=linear + exponential
c	ITYPE = 13 for Exponential Model with alpha(E) switching function
c
      IMPLICIT NONE
 
      DOUBLE PRECISION DENVR , TEMPI , tfac
      INCLUDE 'declare1.inc'
 
      INTEGER Mol
 
      DOUBLE PRECISION B , DENE , E , EE , Hs
 
      DOUBLE PRECISION TERM1 , TERM11 , TERM2 , TERM22 , ALPHA , 
     &                 ALPHA1 , ALPHA2 , EN , Fractt , A , BET , BETA , 
     &                 X , ARG , ARG1 , ARG2 , WIDTH , F , TT , Rob_3
 
      SAVE 
 
      IF ( E.LT.-0.9 ) THEN     ! Indicates that we just want the title lines
 
         LI(Mol,1) = '  '
         LI(Mol,2) = '  '
         LI(Mol,3) = '  '
         LI(Mol,4) = '  '
         LI(Mol,5) = '  ' 

         IF ( ITYPE(Mol).NE.1 ) THEN
            IF ( ITYPE(Mol).EQ.2 ) THEN
 
               LI(Mol,1) = ' Density-weighted Biexponential Model'
 
               LI(Mol,2) = 
     &                   '  Alpha1 = C(1) + [E*C(2) + E*E*C(3)]*T**C(8)'
               LI(Mol,3) = 
     &                   '  Alpha2 = C(5) + [E*C(6) + E*E*C(7)]*T**C(8)'
               LI(Mol,4) = 
     &' Pdown = rho*((1-C(4))*EXP(-(E-EE)/Alpha1) + C(4)*EXP(-(E-EE)/Alp
     &ha2))'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.3 ) THEN
 
               LI(Mol,1) = 
     &                ' Constant off-set, variable width Gaussian Model'
 
               LI(Mol,2) = 
     &                   '  Alpha1 = C(1) + [E*C(2) + E*E*C(3)]*T**C(8)'
               LI(Mol,3) = 
     &              '  C(4) = constant off-set, Alpha1 is the std. dev.'
               LI(Mol,4) = ' Pdown = EXP(-(0.5*(E-EE-C(4))/Alpha1)**2)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.4 ) THEN
 
               LI(Mol,1) = 
     &             ' Biexponential Model with Energy-Dependent Fraction'
 
               LI(Mol,2) = '  Alpha1 = C(1) + [E*C(2) + E*E*C(3)]'
               LI(Mol,3) = '  Alpha2 = C(5) + [E*C(6) + E*E*C(7)]'
               LI(Mol,4) = 
     &' Pdown = (1-[C(4)+E*C(8)])*EXP(-(E-EE)/Alpha1) + [C(4)+E*C(8)]*EX
     &P(-(E-EE)/Alpha2)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.5 ) THEN
 
               LI(Mol,1) = 
     &            'Generalized Gaussian Model with E-Dependent Exponent'
 
               LI(Mol,2) = '  Alpha = C(1) + [E*C(2) + E*E*C(3)]'
               LI(Mol,3) = '  exponent = C(5) + [E*C(6) + E*E*C(7)]'
               LI(Mol,4) = ' Pdown = EXP(-[(E-EE)/Alpha]**Exponent)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.6 ) THEN
 
               LI(Mol,1) = ' Generalized Gaussian + Exponential'
 
               LI(Mol,2) = 
     &'  Alpha1   = C(1) + [E*C(2) + E*E*C(3)]    Alpha2 = C(7)+ E*C(8)'
               LI(Mol,3) = '  Exponent = C(4) + E*C(5)'
               LI(Mol,4) = '  F        = C(6)'
               LI(Mol,5) = 
     &' Pdown     = (1-C(6))*EXP(-[(E-EE)/Alpha1]**Exponent) + C(6)*EXP(
     &-(E-EE)/Alpha2)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.7 ) THEN
 
               LI(Mol,1) = ' Weibull Model'
 
               LI(Mol,2) = 
     &                    '  Alpha = {C(1) + [E*C(2) + E*E*C(3)]}**Beta'
               LI(Mol,3) = '  Beta  = C(4) + E*C(5)'
               LI(Mol,4) = 
     &' Pdown  = (Beta/Alpha)*{(E-EE)**(beta-1)}*exp(-((E-EE)/Alpha)**Be
     &ta)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.8 ) THEN
 
               LI(Mol,1) = ' Lorentzian Step-Ladder Model'
 
               LI(Mol,2) = '  Alpha = C(1) + [E*C(2) + E*E*C(3)]'
               LI(Mol,3) = '  Width = C(5) + [E*C(6) + E*E*C(7)]'
               LI(Mol,4) = ' Pdown = 1 / [(E-EE-Alpha)^2 + Width^2]'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.9 ) THEN
 
               LI(Mol,1) = ' Exponential+Elastic Model'
 
               LI(Mol,2) = 
     &                    '  Alpha = C(1) + [E*C(2) + E*E*C(3)]*T**C(8)'
               LI(Mol,3) = 
     &            '      F = C(5) + [(E/C(6))**C(7)]*T**C(8); when E=EE'
               LI(Mol,4) = 
     &              ' Pdown = [F/(F+C(4))]*EXP(-(E-EE)/Alpha) + elastic'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.10 ) THEN
 
               LI(Mol,1) = ' Klaus Luther s P(EE,E) Empirical Function'
 
               LI(Mol,2) = '  Alpha = C(1) + E*C(2)'
               LI(Mol,3) = '  Beta  = C(3)'
               LI(Mol,4) = ' Pdown = EXP {-[(E-EE)/Alpha]^Beta}'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.11 ) THEN
 
               LI(Mol,1) = ' Triplet State P(EE,E) Empirical Function'
 
               LI(Mol,2) = 
     &                   '  Alpha = C(1)*{1-exp[-(E/C(2))^C(3)] + C(4)}'
               LI(Mol,3) = '  Pdown = EXP [-(E-EE)/Alpha]'
               LI(Mol,4) = ' '
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.12 ) THEN
 
               LI(Mol,1) = 
     &             'Exponential Model with alpha(E)= Linear+exponential'
 
               LI(Mol,2) = ' Alpha1 = C(1) + E*C(2) + C(3)*exp(-E/C(4))'
               LI(Mol,3) =  ' Pdown = EXP(-(E-EE)/Alpha1)'
c        1         2         3         4         5         6         7         8
               RETURN
            ELSEIF ( ITYPE(Mol).EQ.13 ) THEN
 
               LI(Mol,1) = ' Exponential Model with Switching Function'
 
               LI(Mol,2) = 
     &           '  Alpha1 = C(1) + E*C(2);      Alpha2 = C(3) + E*C(4)'
               LI(Mol,3) = 
     &' Alp = Alpha1 + 0.5*(Alpha2 - Alpha1)*(1. - TANH((C(5)-E)/C(6)))'
               LI(Mol,4) =  ' Pdown = EXP(-(E-EE)/Alp)'
c        1         2         3         4         5         6         7         8
               RETURN
            ENDIF
         ENDIF
 
         LI(Mol,1) = ' Biexponential Model'
 
         LI(Mol,2) = '  Alpha1 = C(1) + [E*C(2) + E*E*C(3)]*T**C(8)'
         LI(Mol,3) = '  Alpha2 = C(5) + [E*C(6) + E*E*C(7)]*T**C(8)'
         LI(Mol,4) = 
     &' Pdown = (1-C(4))*EXP(-(E-EE)/Alpha1) + C(4)*EXP(-(E-EE)/Alpha2)'
c        1         2         3         4         5         6         7         8
         RETURN
 
      ENDIF
 
 
 
      IF ( ITYPE(Mol).NE.1 ) THEN
         IF ( ITYPE(Mol).EQ.2 ) THEN
 
            ALPHA1 = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3)))
     &               *TEMPI**DC(Mol,8))
 
 
 
            ALPHA2 = ABS(DC(Mol,5)+(E*(DC(Mol,6)+E*DC(Mol,7)))
     &               *TEMPI**DC(Mol,8))
 
            IF ( ALPHA1.GT.0.0 ) THEN
               TERM1 = EXP(-ABS(E-EE)/ALPHA1)
               TERM11 = TERM1/ALPHA1
            ELSE
               TERM1 = 0.0D+00
               TERM11 = 0.0D+00
            ENDIF
 
            IF ( ALPHA2.GT.0.0 ) THEN
               TERM2 = EXP(-ABS(E-EE)/ALPHA2)
               TERM22 = TERM2/ALPHA2
            ELSE
               TERM2 = 0.0D+00
               TERM22 = 0.0D+00
            ENDIF
 
            DENE = DENVR(Mol,E)
            PDOWN = DENE*((1.0D00-DC(Mol,4))*TERM1+DC(Mol,4)*TERM2)
            Hs = DENE*((1.0D00-DC(Mol,4))*TERM11+DC(Mol,4)*TERM22)
            Hs = PDOWN/Hs
            IF ( Hs.GT.MAX(ALPHA1,ALPHA2) ) THEN
               Hs = MAX(ALPHA1,ALPHA2)
c            ELSEIF ( Hs.LT.MIN(ALPHA1,ALPHA2) ) THEN
c               Hs = MIN(ALPHA1,ALPHA2)
c               Hs = MAX(Hs,Egrain1)
            ENDIF
 
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.3 ) THEN
 
            ALPHA1 = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3)))
     &               *TEMPI**DC(Mol,8))
 
 
 
            IF ( ALPHA1.GT.0.0 ) THEN
               TERM1 = EXP(-(0.5*(E-EE-DC(Mol,4))/ALPHA1)**2)
               TERM11 = -(E-EE-DC(Mol,4))*TERM1/(ALPHA1**2)
            ELSE
               TERM1 = 0.0D+00
               TERM11 = 0.0D+00
            ENDIF
 
            PDOWN = TERM1
            Hs = ABS(TERM1/TERM11)
            IF ( Hs.GT.0.5*ALPHA1 ) Hs = 0.5*ALPHA1
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.4 ) THEN
 
            ALPHA1 = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3))))
 
 
            ALPHA2 = ABS(DC(Mol,5)+(E*(DC(Mol,6)+E*DC(Mol,7))))
 
            IF ( ALPHA1.GT.0.0 ) THEN
               TERM1 = EXP(-ABS(E-EE)/ALPHA1)
               TERM11 = TERM1/ALPHA1
            ELSE
               TERM1 = 0.0D+00
               TERM11 = 0.0D+00
            ENDIF
 
            IF ( ALPHA2.GT.0.0 ) THEN
               TERM2 = EXP(-ABS(E-EE)/ALPHA2)
               TERM22 = TERM2/ALPHA2
            ELSE
               TERM2 = 0.0D+00
               TERM22 = 0.0D+00
            ENDIF
 
            Fractt = DC(Mol,4) + DC(Mol,8)*E
 
            PDOWN = (1.0D00-Fractt)*TERM1 + Fractt*TERM2
            Hs = PDOWN/((1.0D00-Fractt)*TERM11+Fractt*TERM22)
            IF ( Hs.GT.MAX(ALPHA1,ALPHA2) ) THEN
               Hs = MAX(ALPHA1,ALPHA2)
c            ELSEIF ( Hs.LT.MIN(ALPHA1,ALPHA2) ) THEN
c               Hs = MIN(ALPHA1,ALPHA2)
c               Hs = MAX(Hs,Egrain1)
            ENDIF
 
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.5 ) THEN
 
            ALPHA = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3))))
 
 
 
            EN = ABS(DC(Mol,5)+(E*(DC(Mol,6)+E*DC(Mol,7))))
 
            ARG = (ABS(E-EE)/ALPHA)**EN
 
            IF ( ARG.GT.100 ) THEN
               PDOWN = 0.
               Hs = 10000.      ! Take giant steps, because PDOWN=0, anyway
            ELSE
               PDOWN = EXP(-ARG)
               IF ( ALPHA.GT.0.0 ) THEN
                  Hs = ALPHA/EN
               ELSE
                  Hs = Egrain1
               ENDIF
            ENDIF
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.6 ) THEN
 
            ALPHA1 = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3))))
 
 
 
            EN = ABS(DC(Mol,4)+E*(DC(Mol,5)))
            Fractt = DC(Mol,6)
            ALPHA2 = ABS(DC(Mol,7)+E*(DC(Mol,8)))
 
 
            IF ( ALPHA1.GT.0.0 ) THEN
               ARG = (ABS(E-EE)/ALPHA1)**EN
               IF ( ARG.GT.100. ) ARG = 100.
               TERM1 = EXP(-ARG)
               TERM11 = EN/ALPHA1
            ELSE
               TERM11 = 1.00D+00/Egrain1
            ENDIF
 
            IF ( ALPHA2.GT.0.0 ) THEN
               TERM2 = EXP(-ABS(E-EE)/ALPHA2)
               TERM22 = TERM2/ALPHA2
            ELSE
               TERM2 = 0.0D+00
               TERM22 = 0.0D+00
            ENDIF
 
            A = 1.0/TERM11
            B = 1.0/TERM22
 
            PDOWN = (1.0D00-Fractt)*TERM1 + Fractt*TERM2
            Hs = PDOWN/((1.0D00-Fractt)*TERM11+Fractt*TERM22)
            IF ( Hs.GT.MAX(A,B) ) THEN
               Hs = MAX(A,B)
c            ELSEIF ( Hs.LT.MIN(A,B) ) THEN
c               Hs = MIN(A,B)
c               Hs = MAX(Hs,Egrain1)
            ENDIF
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.7 ) THEN
 
            ALPHA = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3))))  ! Energy units
 
 
            BET = DC(Mol,4) + E*DC(Mol,5)
 
            TT = ALPHA*BET
            IF ( TT.LE.0.0 ) THEN
               WRITE(*,*) 
     &          '*** Weibull Model requires ALPHA & BETA > 0 ***'
               WRITE(KOUT,*)  
     &          '*** Weibull Model requires ALPHA & BETA > 0 ***'
               STOP
            ENDIF
 
            Rob_3 = 1.0
 
            Hs = MAX(ALPHA/BET,Rob_3)   ! Energy units
            ALPHA = ALPHA**BET          ! units for Weibull Distribution
            X = ABS(E-EE)
 
            IF ( X.GT.0.0 ) THEN        ! if X >0
               ARG2 = (X**(BET-2.))/ALPHA
               ARG1 = X*ARG2
               ARG = X*ARG1
               IF ( ARG.GT.100. ) THEN  ! then ARG1 and ARG2 must be even bigger
                  PDOWN = 0.0
                  Hs = 10000.   ! Take giant steps, because PDOWN = 0 anyway
                  RETURN
               ELSE
                  PDOWN = (BET/ALPHA)*ARG1*EXP(-ARG)
                                                ! This is General Weibull Model
                  RETURN
               ENDIF
            ELSEIF ( ABS(BET-1.0).LT.1.0E-06 ) THEN     ! if X=0 and BET=1, exponential model
               PDOWN = 1.0
               RETURN
            ELSE                        ! if X=0 and BET1
               PDOWN = 0.0
               RETURN
            ENDIF
         ELSEIF ( ITYPE(Mol).EQ.8 ) THEN
 
            ALPHA = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3))))
 
            WIDTH = ABS(DC(Mol,5)+(E*(DC(Mol,6)+E*DC(Mol,7))))
 
            X = (E-EE-ALPHA)
            PDOWN = WIDTH*WIDTH/(X*X+WIDTH*WIDTH)
 
            Hs = MIN(WIDTH/PDOWN,0.1*ALPHA)
            Hs = MAX(Hs,Egrain1)
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.9 ) THEN
 
            ALPHA = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3)))
     &              *TEMPI**DC(Mol,8))
 
            F = DC(Mol,5) + ((E/DC(Mol,6))**DC(Mol,7))*TEMPI**DC(Mol,8)
 
            IF ( ALPHA.GT.0.0 ) THEN
               TERM1 = EXP(-ABS(E-EE)/ALPHA)
            ELSE
               TERM1 = 0.0D+00
            ENDIF
 
            IF ( ABS(EE-E).LT.1.0E-06 ) THEN                    ! Elastic collisions
               PDOWN = F*TERM1/(DC(Mol,4)+F) + DC(Mol,4)/(DC(Mol,4)+F)
            ELSE
               PDOWN = F*TERM1/(DC(Mol,4)+F)
            ENDIF
 
            Hs = MAX(ALPHA,Egrain1)
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.10 ) THEN
 
            ALPHA = ABS( DC(Mol,1) + E*DC(Mol,2) )
 
 
            BETA = ABS( DC(Mol,3) )
 
            IF ( ALPHA.GT.0.0 ) THEN
               PDOWN = EXP( -( ABS(E-EE)/ALPHA )**BETA )
               Hs = ALPHA
            ELSE
               PDOWN = 0.0D+00
               Hs = 1.0
            ENDIF
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.11 ) THEN
             ALPHA = ABS(DC(Mol,1)*(1-EXP(-((E/DC(Mol,2))**DC(Mol,3))))
     &        +DC(Mol,4))
 
            IF ( ALPHA.GT.0.0 ) THEN 
              TERM1 = EXP(-ABS(E-EE)/ALPHA)
              PDOWN = TERM1
              Hs = ALPHA
             ELSE
              PDOWN = 0.0D+00
              Hs = 1.0
            ENDIF
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.12 ) THEN
 
            ALPHA1 = ABS( DC(Mol,1) + E*DC(Mol,2) + 
     &                DC(Mol,3)*EXP(-E/DC(Mol,4)) )

            IF ( ALPHA1.GT.0.0 ) THEN
               TERM1 = EXP(-ABS(E-EE)/ALPHA1)
               TERM11 = TERM1/ALPHA1
            ELSE
               TERM1 = 0.0D+00
               TERM11 = 0.0D+00
            ENDIF
 
            PDOWN = TERM1

            Hs = PDOWN/TERM11
            IF ( Hs.GT.ALPHA1 )  Hs = ALPHA1
c            IF (E .GT. Emax1+5.*Hs ) Hs = 5.*Hs    ! Big steps for exponential model at high E
 
            RETURN
         ELSEIF ( ITYPE(Mol).EQ.13 ) THEN
 
            ALPHA1 = ABS(DC(Mol,1) + E*DC(Mol,2))  ! Energy units
            ALPHA2 = ABS(DC(Mol,3) + E*DC(Mol,4))  ! Energy units
            BET = ALPHA1 + 0.5d+00*(ALPHA2 - ALPHA1)*(1.d+00 - 
     &             TANH((DC(Mol,5)-E)/DC(Mol,6)))                    ! Switching function

            TERM1 = EXP(-ABS(E-EE)/BET)
            TERM11 = TERM1/BET
 
            PDOWN = TERM1

            Hs = PDOWN/TERM11
            IF ( Hs.GT.BET )  Hs = BET
c            IF (E .GT. Emax1+5.*Hs ) Hs = 5.*Hs    ! Big steps for exponential model at high E

            RETURN
         ENDIF
      ENDIF
 
      tfac = TEMPI**DC(Mol,8)

      ALPHA1 = ABS(DC(Mol,1)+(E*(DC(Mol,2)+E*DC(Mol,3)))*tfac)
 
      ALPHA2 = ABS(DC(Mol,5)+(E*(DC(Mol,6)+E*DC(Mol,7)))*tfac)
 
      IF ( ALPHA1.GT.0.0 ) THEN
         TERM1 = EXP(-ABS(E-EE)/ALPHA1)
         TERM11 = TERM1/ALPHA1
      ELSE
         TERM1 = 0.0D+00
         TERM11 = 0.0D+00
      ENDIF
 
      IF ( ALPHA2.GT.0.0 ) THEN
         TERM2 = EXP(-ABS(E-EE)/ALPHA2)
         TERM22 = TERM2/ALPHA2
      ELSE
         TERM2 = 0.0D+00
         TERM22 = 0.0D+00
      ENDIF
 
      PDOWN = (1.0D00-DC(Mol,4))*TERM1 + DC(Mol,4)*TERM2
      Hs = PDOWN/((1.0D00-DC(Mol,4))*TERM11+DC(Mol,4)*TERM22)


c      IF ( Hs.GT.MAX(ALPHA1,ALPHA2) ) THEN
c         Hs = MAX(ALPHA1,ALPHA2)
c      ELSEIF ( Hs.LT.MIN(ALPHA1,ALPHA2) ) THEN
c         Hs = MIN(ALPHA1,ALPHA2)
c      ENDIF
 
c      IF (E .GT. Emax1+5.*Hs .AND. DC(Mol,4) .LT. 1.e-6 ) THEN
c         Hs = 5.*Hs                   ! Big steps for exponential model at high E
c      ENDIF

      RETURN
  
      END
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2007 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

      SUBROUTINE QKinf(Mol,nchann,tx,Kinf,Ainf,Einf,kcol,kosc)

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , i , IE , Itop , mm, i2
      DOUBLE PRECISION Q , RK , TT , E , D , DENVR , Kinf , Ainf , 
     &                 Einf , K , tx , Et , arg , Y1 , Y2 , Y3 , Y , 
     &                 X , Estep, RR, QQ , Qlast , Rlast , TTSTEP , 
     &                 TS , EC 
      DOUBLE PRECISION kcol, Qts, QQts, Qtslast, DENTS, E2, Qcheck
      DOUBLE PRECISION kosc(MaxWell,MaxChan)
      DIMENSION K(3), TS(3)
      SAVE 
c
c	Calculates high pressure limiting unimolecular rate constant according to
c         Equation (4.30) of Robinson & Holbrook, "Unimolecular Reactions"
c         (Wiley-Interscience, London, 1972), p. 90.
c        This is the same as equation 3.30 in Holbrook, Pilling, and Robertson,
c         "Unimolecular Reactions. Second Edition" (John Wiley & Sons, Chichester, 
c         1996), p. 64.
c
c	Mol	= index of molecule
c	nchann	= index of rxn channel
c	tx	= translational temperature
c	Kinf	= infinite pressure rate constant [Robinson & Holbrook Eq. 4.30]
c	Ainf	= A-factor for kinf
c	Einf	= Activation energy for kinf
c
c
c     Centrifugal correction at temperature = tx
c

      EC =  Eor(Mol,nchann)                       ! Includes centrifugal correction

      TTSTEP = 0.001d+00/tx                       ! Step size in 1/Temp
      TT = 1.0d+00/tx-2.d+00*TTSTEP               ! 1/Temp

      DO 1000 mm = 1,3
        K(mm) = 0.0d+00
        RK = 0.0d+00
        Q  = 0.0d+00
        Qts = 0.0d+00
        Qlast = exp(Dens(Mol,1))
        Qtslast = DENVR(Mol,EC)
        Rlast = 0.0d+00

        TT = TT + TTSTEP                          ! 1/T
        TS(mm) = TT                               ! 1/T

        Estep = Egrain1
        E = 0.0d+00
      DO 100 i = 2 , imax1                        
         E = E + Estep                            ! summation continues till end of first part of double array
         QQ = exp(Dens(Mol,i)-E*hor*TT)           ! molecule partition fxn;  density stored as log
         arg = E + EC
         D = DENVR(Mol,arg)
         RR = D*exp(Rate(Mol,nchann,i)-E*hor*TT)  ! thermal rate constant; k(E) stored as log
         QQts=D*exp(-E*hor*TT)                    ! partition function for active energy;
         Q  = Q  + 0.5*Estep*(QQ + Qlast)         ! Trapezoidal Rule
         RK = RK + 0.5*Estep*(RR + Rlast)
         Qts = Qts + 0.5*Estep*(QQts+Qtslast)     ! for use in strong collider calculation
         Qlast = QQ
         Rlast = RR
         Qtslast=QQts
 100  CONTINUE
c
c          NOW CONTINUE TO HIGHER ENERGIES
c 
      Estep = 10.*Egrain1                          ! Step size (cm-1)
      Itop = ( 25.0/(hor*TT))/Estep               ! Integral to be extended to 200 kT; however the evaluation
      E2=E                                         ! will stop when Qcheck is less than Qthresh
      DO i = 1 , Itop
         E = E + Estep
         D = DENVR(Mol,E)
         QQ = D*exp(-E*hor*TT)                     ! continuation of molecule partition function
         Q  = Q  + 0.5*Estep*(QQ + Qlast)          ! Trapezoidal Rule
         Qlast = QQ
         Qcheck=dabs(QQ/Q)
         if(Qcheck.lt.Qthresh)then
           goto 17
         else
           continue
         end if

         if(E.gt.Emax2)then
           WRITE(*,*) 'FATAL: Emax2 exceeded; extend Emax2 in Densum'
           WRITE(KOUT,*) 'FATAL: Emax2 exceeded; extend Emax2 in Densum'
           STOP
         end if
       end do

 17     continue

       arg=E2+EC
       do i2=1,Itop
         arg = arg+Estep
         E2=E2+Estep
         D = DENVR(Mol,arg)
         CALL NTERP(E2,IE,X)
         Y1 = Rate(Mol,nchann,IE-1)
         Y2 = Rate(Mol,nchann,IE)
         Y3 = Rate(Mol,nchann,IE+1)
         CALL QUDINT(Y1,Y2,Y3,X,Y)                 ! Y = interpolated ln k(E)
         RR = D*exp(Y-E2*hor*TT)                   ! k(E) stored as log
         QQts = D*exp(-E2*hor*TT)                  ! continuation of partition function for active energy;
         RK = RK + 0.5*Estep*(RR + Rlast)
         Qts = Qts + 0.5*Estep*(QQts+Qtslast)      ! for use in strong collider calculation
         Rlast = RR
         Qtslast=QQts
         Qcheck=dabs(QQts/Qts)
          if(Qcheck.lt.Qthresh)then    
            goto 18
          else
            continue
          end if

          if(E2.gt.Emax2)then
            WRITE(*,*) 'FATAL: Emax2 exceeded; extend Emax2 in Densum'
            WRITE(KOUT,*)'FATAL: Emax2 exceeded; extend Emax2 in Densum'
            STOP
          end if
        end do

 18    Continue

       K(mm) = RK*exp(-Eo(Mol,nchann)*hor*TT)/Q    
       IF ( (Jread(Mol,nchann).GT.0) . AND.                            ! not ILT method
     &    (MolMom(Mol) .GT. 0.0)  .AND. (TSmom(Mol,nchann) .GT. 0.0) )
     &    K(mm) = K(mm)*(TSmom(Mol,nchann)/MolMom(Mol))     ! kinf with adiabatic Qrot ratio: equal to TST result
         

1000  CONTINUE

      Kinf = K(2)
      Et = -(LOG(K(3)/K(1)))/(TS(3)-TS(1))            ! Ea/R
      Ainf = K(2)*exp(Et*TS(2))                       ! 1/s
      Einf = (1.9872065d-03)*Et                       ! kcal/mole
C                                                                       strong collider low pressure rate constant
      kosc(Mol,nchann) = kcol*Qts*exp(-Eo(Mol,nchann)*hor*TT)/Q        ! Based on Eq. 4.30 in Robinson & Holbrook
       IF ( (Jread(Mol,nchann).GT.0) . AND.                            ! not ILT method
     &    (MolMom(Mol) .GT. 0.0)  .AND. (TSmom(Mol,nchann) .GT. 0.0) )
     &           kosc(Mol,nchann) =  kosc(Mol,nchann)*
     &                         (TSmom(Mol,nchann)/MolMom(Mol))

      RETURN
      END
      SUBROUTINE QKinftun(Mol,nchann,tx,Kinf,Ainf,Einf)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
c
c     01/02  v1.2.0  -New Feature: can optionally choose to apply centrifugal
c                     corrections for any choice of model. Thus can apply them
c                     even when using Inverse Laplace Transform form of k(E).
c                     NOTE: this new feature affects the data file format.
c     11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                     inconsistently when calculating Arrhenius parameters 
c                     (k(inf) was correct, however).
c     09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                     See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c
c***********************************************************************

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , i , IE , Itop , mm, i2
      DOUBLE PRECISION Q , RK , TT , E , D , DENVR , Kinf , Ainf , 
     &                 Einf , K , tx , Et , arg , Y1 , Y2 , Y3 , Y , 
     &                 X , Estep, RR, QQ , Qlast , Rlast , TTSTEP , 
     &                 TS , EC, E2, Qcheck, Qts, QQts, Qtslast
      DIMENSION K(3), TS(3)
      SAVE 
c
c	Calculates high pressure limiting unimolecular rate constant according to
c         Equation (4.30) of Robinson & Holbrook, "Unimolecular Reactions"
c         (Wiley-Interscience, London, 1972), p. 90.
c
c	Mol	= index of molecule
c	nchann	= index of rxn channel
c	t	= translational temperature
c	Kinf	= infinite pressure rate constant [Robinson & Holbrook Eq. 4.30]
c	Ainf	= A-factor for kinf
c	Einf	= Activation energy for kinf
c
c
c     Centrifugal correction at temperature = tx
c
      EC =  Eor(Mol,nchann)                 ! Includes centrifugal correction

      TTSTEP = 0.001d+00/tx                  ! Step size in 1/Temp
      TT = 1.0d+00/tx-2.d+00*TTSTEP          ! 1/Temp

      DO 1000 mm = 1,3
        K(mm) = 0.0d+00
        RK = 0.0d+00
        Q  = 0.0d+00
        RR = 0.0d+00
        QQ = 0.0d+00
        Qlast = 0.0d+00
        Rlast = 0.0d+00
        QQts=0.0d00
        Qtslast=0.0d0
        Qts=0.0d0

        TT = TT + TTSTEP                     ! 1/T
        TS(mm) = TT                          ! 1/T

        Estep = Egrain1
        E = -Estep*(imax1+1)        

      DO 100 i = 1 , 2*imax1                       ! start at imax1*Egrain1 below the reaction threshold
         E = E + Estep                             ! summation continues till end of first part of double array

        if(i.gt.imax1)then                         ! allows Q(Trans) to be calculated from 0-->inf
          QQ=exp(Dens(Mol,i-imax1)-(E*hor*TT))     ! molecule partition function;correct E and Dens
                                                   ! both start at 0 Energy and index=0 i.e. (i-imax1)=0
          Q  = Q  + 0.5*Estep*(QQ + Qlast)         ! Trapezoidal Rule
          Qlast = QQ
          QQts=D*exp(-E*hor*TT)
          Qts=Qts+0.5*Estep*(QQts+Qtslast)
          Qtslast=QQts
        else
          continue
        end if

         arg = E + EC                              ! arg is E + barrier height; starts at -Egrain1*imax1
                                                   ! below the barrier

        if(arg.ge.0)then
	    
          D = DENVR(Mol,arg)                        ! density of states of molecule evaluated at
                                                    ! the (top of the barrier - Egrain1*imax1)



	
          RR = D*exp(TRate(Mol,nchann,i)-(E*hor*TT))! this corresponds to Eq. 12 in MW pub. and is
                                                   ! the integral from (Eo-(Egrain1*imax1)) to inf
          RK = RK + 0.5*Estep*(RR + Rlast)
          Rlast = RR
        end if


 100  CONTINUE
c
c          NOW CONTINUE TO HIGHER ENERGIES
c 
      Estep = 10.*Egrain1                          ! Step size (cm-1)
      Itop = ( 25.0/(hor*TT))/Estep                ! Integral to be extended to 25 kT
      E2=E
      DO i = 1 , Itop                              ! Continuing the integral to higher energies
         E = E + Estep                             ! last energy from above loop + new Estep is 1st energy evaluated
         D = DENVR(Mol,E)
         QQ = D*exp(-E*hor*TT)
         Q  = Q  + 0.5*Estep*(QQ + Qlast)           ! Trapezoidal Rule
         Qlast = QQ
         Qcheck = dabs(QQ/Q)
         if(Qcheck.lt.Qthresh)then
           goto 17
         else
           continue
         end if

         if(E.gt.Emax2)then
           write(*,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           write(KOUT,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           STOP
          else
            continue
          end if
        end do

 17     continue

        arg=E2+EC
        do i2=1,Itop
           arg=arg+Estep
           E2=E2+Estep
           D = DENVR(Mol,arg)

           CALL NTERP2(E2,IE,X)                    ! NTERP2 determines IE and X for a k(E)
                                                   ! array with Isize+imax1 elements; of which
                                                   ! imax1 elements are below the reaction threshold
           Y1 = TRate(Mol,nchann,IE-1)
           Y2 = TRate(Mol,nchann,IE)
           Y3 = TRate(Mol,nchann,IE+1)
           CALL QUDINT(Y1,Y2,Y3,X,Y)                  ! Y = interpolated ln k(E)
           RR = D*exp(Y-E2*hor*TT)                    ! k(E) stored as log
           RK = RK + 0.5*Estep*(RR + Rlast)
           Rlast = RR
           QQts=D*exp(-E2*hor*TT)              ! partition function evaluated to determine
           Qts=Qts+0.5*Estep*(QQts+Qtslast)    ! when the calculation of RK can be stopped
           Qtslast=QQts
           Qcheck=dabs(QQts/Qts)
           if(Qcheck.lt.Qthresh)then
             goto 18
           else
             continue
           end if

           if(E2.gt.Emax2)then
           write(*,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           write(KOUT,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           STOP
           else
             continue
           end if
         end do

 18      continue    
       RK = RK*exp(-Eo(Mol,nchann)*hor*TT)/Q
       IF ( NCENT(Mol,nchann).EQ.1 ) THEN               ! If Centrifugal Correction
          K(mm) = RK*(TSmom(Mol,nchann)/MolMom(Mol))    ! kinf with adiabatic Qrot ratio
       ELSE
          K(mm) = RK                                    ! Adiabatic Qrot ratio = 1.0
       ENDIF
1000  CONTINUE

      Kinf = K(2)
      Et = -(LOG(K(3)/K(1)))/(TS(3)-TS(1))                  ! Ea/R
      Ainf = K(2)*exp(Et*TS(2))                             ! 1/s
      Einf = (1.9872065d-03)*Et                             ! kcal/mole
      RETURN
      END
**==qudint.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
      SUBROUTINE QUDINT(Y1,Y2,Y3,X,Y)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2001 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the "ReadMe" file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c***********************************************************************
C
C     LINEAR INTERPOLATION FROM EQUALLY-SPACED DATA, RETURNING
C	VALUE OF Y AT VALUE OF X:
C
C       Y1 AT X = -1
C       Y2 AT X =  0
C       Y3 AT X = +1
c
c***********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE 
 
      IF ( X .LE. 0.0) THEN
           Y = Y2 + X*(Y2-Y1)
      ELSEIF ( X .GT. 0.0) THEN
           Y = Y2 + X*(Y3-Y2)
      ENDIF
 
      RETURN

      END
**==ran1.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
 
 
      DOUBLE PRECISION FUNCTION RAN1(idum)
C---------------------------------------------------------------------------------
C	"Minimal" random number generator of Park and Miller with Bays-Durham shuffle
C	and added safeguards.  Returns a uniform random deviate between 0.0 and 1.0
C	(exclusive of the endpoint values).  Call with idum a negative integer to
C	initialize; thereafter do not alter idum between successive deviates in a
C	sequence.  RNMX should approximate the largest floating value that is less
C	than 1.
C
C	W. H. Press and S. A. Teukolsky, Computers in Physics, 6(5), 522-4 (1992).
C---------------------------------------------------------------------------------
      IMPLICIT NONE
      INTEGER idum , IA , IM , IQ , IR , NTAB , NDIV
      DOUBLE PRECISION AM , EPS , RNMX , T
      PARAMETER (IA=16807,IM=2147483647,AM=1.0D+00/IM,IQ=127773,IR=2836,
     &        NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=2.3D-16,RNMX=1.0D+00-EPS)
      INTEGER j , k , iv(NTAB) , iy
      SAVE iv , iy
      DATA iv/NTAB*0/ , iy/0/
      IF ( idum.LE.0 .OR. iy.EQ.0 ) THEN
                                        ! Initialize
         idum = max(-idum,1)            ! Be sure to prevent idum=0
         DO 50 j = NTAB + 8 , 1 , -1    ! Load shuffle table after 8 warm-ups
            k = idum/IQ
            idum = IA*(idum-k*IQ) - IR*k
            IF ( idum.LT.0 ) idum = idum + IM
            IF ( j.LE.NTAB ) iv(j) = idum
 50      CONTINUE
         iy = iv(1)
      ENDIF
      k = idum/IQ                       ! Start here when not initializing
      idum = IA*(idum-k*IQ) - IR*k      ! Compute idum=mod(IA*idum,IM) without
      IF ( idum.LT.0 ) idum = idum + IM !    overflows by Schrage's Method
      j = 1 + iy/NDIV                   ! Will be in range 1:NTAB
      iy = iv(j)                        ! Output previously stored value
      iv(j) = idum                           ! Refill shuffle table
      T = AM*iy
      RAN1 = min(T,RNMX)        ! Because users don't expect endpoint values
c11	format(d90.50)
c	write(6,11)RAN1
      RETURN
      END
      SUBROUTINE RateArray(T)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c	k(E) calculated and stored in double array 'Rate'.  Note that
c	reverse rxns are calculated with same sums of states as
c	forward rxns.
c
c	T	= translational temperature
c	Mol	= index of reactant molecule
c	Nchan	= number of channels originating from Mol
c	Jread	= 0  use inverse Laplace transform method for channel
c		= 1  read sums of states for channel from external file
c		= 2  read k(E) for this channel from external file
c	Jto	= index of product molecule
c	Jrev	= 0 for neglecting the reverse reaction
c		= 1 for calculating reverse rxn rate; after calculation,
c			Jrev=1 is changed to Jrev=2
c	Afact	= A-factor for reaction (units: s-1) [for ILT method]
c	Eact	= activation energy (units: cm-1) [for ILT method]
c
c	Note that File name and path in OPEN statements may need revision
c		for non-Macintosh
c
c***********************************************************************
c
c 10/03  v1.3.3  (JP) trimmed defined-length character strings with lenstr.
c 01/02  v1.2.0  -New Feature: can optionally choose to apply centrifugal
c                 corrections for any choice of model. Thus can apply them
c                 even when using Inverse Laplace Transform form of k(E).
c                 NOTE: this new feature affects the data file format.
c 11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                 inconsistently when calculating Arrhenius parameters 
c                 (k(inf) was correct, however).
c 09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                 See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c 07/00  v1.1    -All k(E)'s now calculated including Path(Mol,i)
c 07/00  v1.1    -For k(E) read from file, can calculate reverse rxn k(E)
c
c***********************************************************************
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER i , j , Mol , JJ , jdum , lenstr
      DOUBLE PRECISION DENVR , E , EE , DD , A , B , TSsum , T , DUM , 
     &                 ER , Test1 , Test2 
      DOUBLE PRECISION psum , DENTS ,absbar, DScrit
      CHARACTER TITLE*100
      SAVE 

      
        DO 100 Mol = 1 , NWells

 
         IF ( Mol .LE. Nwells ) THEN     ! Ensure that Mol has a density of states array
            DO 20 i = 1 , Nchan(Mol)
 
               Path(Mol,i) = (Molsym(Mol)*TSele(Mol,i)*TSopt(Mol,i))
     &                       /(TSsym(Mol,i)*Molele(Mol)*Molopt(Mol))
                          ! Rxn path degeneracy X electronic partition fxn ratio used for all k(E)

               IF ( (NCENT(Mol,i) .EQ. 1) .AND. (MolMom(Mol) .GT. 0.0)  
     &                              .AND. (TSmom(Mol,i) .GT. 0.0) ) THEN
                   Eor(Mol,i) =  Eo(Mol,i)
     &                   + (1.0-TSmom(Mol,i)/MolMom(Mol))*T/hor ! Centrifugal correction
               ELSE
                   Eor(Mol,i) =  Eo(Mol,i)
               ENDIF                     
              

               IF ( Jread(Mol,i).EQ.0 ) THEN            ! Inverse-Laplace-Transform !!!!!!!!!!!!!!!!!!!!!!!!!!!
                   
c                  IF ( NCENT(Mol,i) .EQ.1 )THEN         ! If centrifugal correction, modify A-factor
c                      Afac(Mol,i) = Afac(Mol,i)*MolMom(Mol)/TSmom(Mol,i)
c                  ENDIF                     

                  DO 5 j = 1 , Isize
                     IF ( j.LE.imax1 ) THEN             ! E = energy from top of barrier
                        E = (j-1)*Egrain1
                     ELSE
                        E = (j-imax1-1)*Egrain2
                     ENDIF
                     EE = E + Eor(Mol,i)                ! Includes centrifugal correction
                     IF ( EE .LE. Emax2) THEN           ! calculate k(E) only up to Emax2, since dens is undefined for higher E
                       DD = DENVR(Mol,EE)
                       Rate(Mol,i,j) = Afac(Mol,i)*DENVR(Mol,E)/DD  ! k(E)
                       Rate(Mol,i,j) = LOG(Rate(Mol,i,j))           ! k(E) stored as log
                     ELSE
                       Rate(Mol,i,j) = LOG( 1.e-20 )                ! k(E) stored as log
                     ENDIF
                       
 5                CONTINUE
 
               ELSEIF ( Jread(Mol,i).EQ.1 ) THEN        ! Read sums of states from external file !!!!!!!!!!!!!!!!!!!!!!!!!!
                  OPEN (12,FILE='DensData/'//
     &            TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.dens',
     &            STATUS='OLD')                         ! TST sum of states

                  READ (12,99030) dumcut                ! Check to see if file format includes input data summary
                  IF ( dumcut .EQ. cut ) THEN           ! Format starting with v.2008.1
                    dumcut = 'x'
                    DO WHILE (dumcut .NE. cut)
                      READ (12,99030) dumcut
                    END DO
                 ELSE
                    BACKSPACE 12                        ! If format from prior to v.2008.1
                 ENDIF
99030 FORMAT (A46)

                  READ (12,*) TITLE
                  READ (12,*) TITLE
                  READ (12 , * )  A , JJ , B , j , DUM
                  READ (12,*) TITLE
                  Test1 = ABS(A-Egrain1)/1.D-06
                  Test2 = ABS(B-Emax2)/1.D-06
                  IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. ! Test for proper array bounds, etc.
     &                 Test2.LT.1.0 .AND. j.EQ.Isize ) THEN

                          DScrit=0.00001d0/Egrain1     ! If the density of states is less than DScrit
                                                       ! then k(E) is set to zero for the tunneling and
                                                       ! non-tunneling cases below 

                          absbar=HMol(Mol)+Eor(Mol,i)   ! absolute energy of barrier needed to determine
                                                        ! back reaction barrier

                          V1(Mol,i)=absbar-HMol(Jto(Mol,i))   ! HMol(Jto(Mol,i)) is the enthalpy of the product


                          if(itun(Mol,i).eq.1)then     ! check to make sure the back reaction barrier is
                                                       ! greater than zero, which must be the case if tunneling
                                                       ! is to be considered
                            if(V1(Mol,i).lt.0)then
                              write(6,99005)Jto(Mol,i),Mol
                              itun(Mol,i)=0
                            end if
                          end if

                          if(itun(Mol,i).eq.1)then   ! tunneling condition


                            write(*,99003) Mol        ! Entering tunneling routine write statement

                            CALL DensArrayTS(Mol,i)   ! set up the densTS array so that the density of states
                                                      ! of the transition state can be interpolated later
                            do j=1,Isize+imax1        ! loop over energies for which k(E) will be calculated

                              IF ( j.LE.imax1*2 ) THEN                
                                E = (j-1)*Egrain1               
                                EE=Eor(Mol,i)-((imax1-1)*Egrain1)+E  ! Includes centrifugal correction
                              ELSE 
                                E = (j-imax1*2-1)*Egrain2
                                EE=Eor(Mol,i)+E  ! Includes centrifugal correction; ensures 2nd part
                              ENDIF              ! of double array starts at 0 cm-1 instead of below the barrier


                              if((Jrev(Mol,Jto(Mol,i))).eq.1)then  ! if the reverse rxn from well Mol to product Jto(Mol,i)
                                                                   ! is considered than store the density of states of the 
                                                                   ! product calculated for the forward rxn in TSdensBK  and 
                                                                   ! use this quantity to calculate the reverse rxn rate 

                                iset(Mol)=Jto(Mol,i)           ! the reverse rxn will have iset(Mol) as the well and 
                                iset2(Mol)=Mol                 ! iset2(Mol) as the product

                              else if((Mol.eq.iset(Mol)).and.    
     +                              (Jto(Mol,i).eq.iset2(Mol)))then

                                if(TSdensBK(iset2(Mol),j).lt.1e-30)then
                                  TRate(Mol,i,j)=-70.0d0       
                                  goto 19                      
                                end if

                                if(DENVR(Mol,EE).lt.DScrit)then
                                  TRate(Mol,i,j)=-70.0d0
                                  goto 19
                                end if

                                IF ( EE .LE. Emax2 ) THEN               ! If molecular energy is not too high, claculate k(E)
                                TRate(Mol,i,j)=TSdensBK(iset2(Mol),j)*
     &                                  clight*Path(Mol,i)/DENVR(Mol,EE)!    k(E) with tunneling correction
                                TRate(Mol,i,j) = LOG(TRate(Mol,i,j))    !    k(E) with tunneling correction stored as log
                                goto 19
                                ELSE
                                 TRate(Mol,i,j) = -70.0                 ! Do not calculate k(E) if molecular energy >Emax2
                                ENDIF

                              end if

                            
                              CALL ECKART(Mol,i,EE,psum,j)  ! calculate tunneling correction using 
                                                            ! unsymmetrical Eckart potential


                              if((Jrev(Mol,Jto(Mol,i))).eq.1)then   ! if the reverse rxn  Jto(Mol,i)-->Mol is considered 
                                TSdensBK(Mol,j)=psum                ! save the denisty of states of the TS for the forward
                              end if                                ! rxn and use them to calculate the reverse rxn rate, i.e.
                                                                    ! don't perform the calculation twice

                              if(psum.lt.1e-30)then        ! this if statement is necessary because if the sum of 
                                TRate(Mol,i,j)=-70.0d0     ! is zero taking LOG(TRate(Mol,i,j) below results in NAN 
                                goto 19                    ! this circumvents that problem
                              end if

                              if(DENVR(Mol,EE).lt.DScrit)then
                                TRate(Mol,i,j)=-70.0d0
                                goto 19
                              end if

                              IF ( EE .LE. Emax2 ) THEN                 ! If molecular energy is not too high, claculate k(E)
                              TRate(Mol,i,j) = psum*clight*Path(Mol,i)
     &                                       /DENVR(Mol,EE)             ! k(E) with tunneling correction
                              TRate(Mol,i,j) = LOG(TRate(Mol,i,j))      ! k(E) with tunneling correction stored as log
                              ELSE
                              TRate(Mol,i,j) = -70.0                    ! Do not calculate k(E) if molecular energy >Emax2
                              ENDIF

19                         CONTINUE
                         end do

                           write(*,99004)

                       else

                          DO 6 j = 1 , Isize                        ! **** NO TUNNELING *************************************
                            READ (12,*) jdum , A , B , TSsum        ! Read sum of states  (here, A and B are dummy variables)
                            IF ( j.LE.imax1 ) THEN                  ! E = active energy from top of barrier
                              E = (j-1)*Egrain1
                            ELSE
                              E = (j-imax1-1)*Egrain2
                            ENDIF
                            EE = E + Eor(Mol,i)                     ! Includes centrifugal correction

                            IF ( EE .LE. Emax2) THEN           ! calculate k(E) only up to Emax2, since dens is undefined for higher E
                              DD = DENVR(Mol,EE)
                              if ( DD .lt. DScrit ) then
                                Rate(Mol,i,j)= LOG( 1.d-20 )   ! the density of states of the reactant is less
                                goto 6                         ! than DScrit, set the rate constant equal to zero.
                              end if
                              DD = TSsum/DD 
                              Rate(Mol,i,j) = clight*DD*Path(Mol,i)     ! k(E) w/o tunneling correction
                              Rate(Mol,i,j) = LOG(Rate(Mol,i,j)+1.e-20) ! k(E) w/o tunneling correction stored as log
                            ELSE
                              Rate(Mol,i,j) = LOG( 1.d-20 )             ! k(E) w/o tunneling correction stored as log
                            ENDIF
 6                       CONTINUE
                       end if

                  ELSE                               ! If grain or energy limit does not match
                    WRITE (KSTD,99002) TSname(Mol,i)
                    WRITE (KOUT,99002) TSname(Mol,i)
                    STOP
                  ENDIF
                  CLOSE (12)
 
               ELSEIF ( Jread(Mol,i).EQ.2 ) THEN        ! Read k(E)'s (not including path degeneracy) from external file !!!!!!!!!!!!!!!!!!!!!!!!!!!
                  OPEN (12,FILE='DensData/'//
     &            TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.rke',
     &            STATUS='OLD')                   ! k(E)'s
                  READ (12,*) TITLE
                  READ (12,*) TITLE
                  READ (12 , * )  A , JJ , B , j , DUM
                  READ (12,*) TITLE
                  Test1 = ABS(A-Egrain1)/1.D-06
                  Test2 = ABS(B-Emax2)/1.D-06
                  IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. 
     &                 Test2.LT.1.0 .AND. j.EQ.Isize ) THEN     ! Test for proper array bounds, etc.
                     DO 8 j = 1 , Isize
                        READ (12,*) jdum , A , B , Rate(Mol,i,j)    ! Read k(E)'s
                        IF(Afac(Mol,i).lt.0) Rate(Mol,i,j)=Rate(Mol,i,j)
     &                                                * ABS(Afac(Mol,i))
                        IF ( Rate(Mol,i,j).GT.1.D-20 ) THEN
                           Rate(Mol,i,j) = LOG(Rate(Mol,i,j))       ! k(E) stored as log
                        ELSE
                           Rate(Mol,i,j) = LOG(1.D-20)              ! k(E) stored as log
                        ENDIF
 8                   CONTINUE
                  ELSE                  ! If grain or energy limit does not match
                     WRITE (KSTD,99002) TSname(Mol,i)
                     WRITE (KOUT,99002) TSname(Mol,i)
                     STOP
                  ENDIF
                  CLOSE (12)
 
               ELSEIF ( Jread(Mol,i).EQ.-2 ) THEN       ! Read k(E)'s (not including path degeneracy) for REVERSE rxn !!!!!!!!!!!!!!!!!!!!!!!!!!!
                  OPEN (12,FILE='DensData/'//
     &            TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.rke',
     &            STATUS='OLD')                   ! k(E)'s for REVERSE rxn
                  READ (12,*) TITLE
                  READ (12,*) TITLE
                  READ (12 , * )  A , JJ , B , j , DUM
                  READ (12,*) TITLE
                  Test1 = ABS(A-Egrain1)/1.D-06
                  Test2 = ABS(B-Emax2)/1.D-06
                  IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. 
     &                 Test2.LT.1.0 .AND. j.EQ.Isize ) THEN
                                                ! Test for proper array bounds, etc.
                     DO 10 j = 1 , Isize
                        READ (12,*) jdum , A , B , Rate(Mol,i,j)
                                                                ! Read k(E)'s (not including path degeneracy) for REVERSE reaction
 
                        IF ( j.LE.imax1 ) THEN                  ! E = energy from top of barrier
                           E = (j-1)*Egrain1                    ! Energy in forward direction
                        ELSE
                           E = (j-imax1-1)*Egrain2              ! Energy in forward direction
                        ENDIF
                        E = E + Eor(Mol,i)                      ! With rotational correction
                        ER = E - (HMol(Jto(Mol,i))-HMol(Mol))   ! Energy in reverse direction
                        DD = DENVR(Jto(Mol,i),ER)/DENVR(Mol,E)  ! Density of states ratio
                        Rate(Mol,i,j) = Rate(Mol,i,j)*DD*Path(Mol,i)
                                                                ! k(E) in forward direction
                        IF ( Rate(Mol,i,j).GT.1.D-20 ) THEN
                           Rate(Mol,i,j) = LOG(Rate(Mol,i,j))
                                                        ! k(E) stored as log
                        ELSE
                           Rate(Mol,i,j) = LOG(1.D-20)  ! k(E) stored as log
                        ENDIF

 10                  CONTINUE
                  ELSE                  ! If grain or energy limit does not match
                     WRITE (KSTD,99002) TSname(Mol,i)
                     WRITE (KOUT,99002) TSname(Mol,i)
                     STOP
                  ENDIF
                  CLOSE (12)
 
               ENDIF
 20         CONTINUE
         ENDIF

 100    CONTINUE
 
        RETURN
 
99001   FORMAT (A100)
99002   FORMAT (/'Energy grain or energy limit does not match: ',
     &        A10/'****** Run Terminated *******')
99003   FORMAT ('Entering Tunneling Subroutine: Molecule #',I2)
99004   FORMAT ('Leaving Tunneling Subroutine')
99005   FORMAT (/'The reaction barrier from product',i3,' to well', i3,
     &          /' is less than the enthalpy of this product.',
     &   /'Calculations will proceed as if tunneling were not invoked.')
 
        END
 
      FUNCTION rivr( Mol , i , IE , numbdens )
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2005 John R. Barker
c
c John R. Barker
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c jrbarker@umich.edu
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
      IMPLICIT NONE

      INCLUDE 'declare1.inc'

      INTEGER          Mol , i , IE

      DOUBLE PRECISION EE , colls , rativr , numbdens , rivr , vfrq ,
     &                rmax

      DOUBLE PRECISION vivr , vave , civr , pivr , tivr
      INTEGER     iivr
      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            vave( MaxWell , MaxChan ) ,
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

c
c     vivr(Mol,i)   : characteristic reaction frequency (as in RRK theory) (cm-1)
c     vave(Mol,i)   : average molecular frequency (cm-1)
c     pivr(Mol,i)   : bimolecular collision rate constant for collision-induced IVR
c     tivr(Mol,i)   : IVR threshold energy (cm-1), measured from the  reaction threshold (Eor)
c     civr(Mol,i,j) : coefficients for 2nd order fit of 
c                     k(ivr) = civr(-,-,1) + civr(-,-,2)*EE + cirv(-,-,3)*EE*EE
c                     where EE = (E - Eor) is the Energy measured from the rxn threshold
c     iivr(Mol,i)   : flag to calculate IVR (iivr=1), or not (iivr=0)
c

      IF ( iivr(Mol , i) .EQ. 0 ) THEN              ! Neglect IVR

         rivr = 1.0d+00

       ELSEIF (iivr(Mol , i) .EQ. 1 ) THEN          ! Calculate IVR transmission coefficient

        rmax = 2.d+00*2.9979d+10*vave( Mol , i )    ! Upper limit to IVR rate

         IF ( IE.LE.imax1 ) THEN                    ! EE = (E - Eor) is the Energy measured from the rxn threshold
            EE = (IE-1)*Egrain1
           ELSE
            EE = (IE-imax1-1)*Egrain2
         ENDIF
c
c        Calculate IVR rate constant = f(EE)
c

         IF ( EE .LT. tivr( Mol , i ) ) THEN                          ! Energy must be .GE. IVR threshold energy
           rativr = 0.0d+00
         ELSE
           rativr = civr( Mol , i , 1 ) +                             ! IVR rate constant = f(EE) [second order fit]
     &        (civr( Mol , i , 2 ) + civr( Mol , i , 3 )*EE )*EE
           IF ( rativr .LT. 0.0d+00 ) rativr = 0.0d+00
           IF ( rativr .GT. rmax )    rativr = rmax                   ! upper limit corresponding to vave(Mol,i) cm-1
         ENDIF

         vfrq = vivr(Mol , i)*2.9979d+10                              ! Convert from cm-1 to frequency (s-1)
         colls = pivr( Mol , i )*numbdens                             ! Collision-induced IVR pseudo-first-order rate

         rivr = (rativr + colls) / ( rativr + colls + vfrq )          ! Wolynes-Leitner IVR transmission coefficient
      ENDIF

      IF(rivr.LT.1.0d-10) rivr = 1.0d-10           ! ensures that log(rivr) will not be NAN when rivr is very small

      RETURN
      END
      SUBROUTINE rotunits( VROT , X )
      
      IMPLICIT NONE
      CHARACTER VROT*4      
      DOUBLE PRECISION X
      SAVE
     
c      CONVERT UNITS TO amu*Ang^2
c      For qro and rot types [NOT hindered rotorS]:
c          VROT = 'AMUA' for moment of inertia in units of amu*Ang^2
c       or      = 'GMCM' for moment of inertia units of gram*cm^2
c       or      = 'CM-1' for rotational constant in units of cm^-1
c       or      = 'MHZ' for rotational constant in units of MHz
c       or      = 'GHZ' for rotational constant in units of GHz
c         
      IF (     VROT .EQ. 'AMUA'
     &    .OR. VROT .EQ. 'amua' 
     &   ) THEN
        RETURN
      ELSEIF ( VROT .EQ. 'GMCM'
     &    .OR. VROT .EQ. 'gmcm'
     &    .OR. VROT .EQ. 'GmCm'
     & ) THEN
        X = X / 1.660538782d-040
        RETURN
      ELSEIF ( VROT .EQ. 'CM-1'
     &    .OR. VROT .EQ. 'cm-1'
     &  ) THEN
        X = 16.85763D+00 / X
        RETURN
      ELSEIF ( VROT .EQ. 'MHZ'
     &    .OR. VROT .EQ. 'MHz' 
     &    .OR. VROT .EQ. 'mhz'
     &    .OR. VROT .EQ. 'Mhz'
     & ) THEN
        X = 5.05379D+005 / X
      ELSEIF ( VROT .EQ. 'GHZ'
     &    .OR. VROT .EQ. 'GHz' 
     &    .OR. VROT .EQ. 'ghz'
     &    .OR. VROT .EQ. 'Ghz'
     & ) THEN
        X = 5.05379D+002 / X
        RETURN
      ELSE
        write (*,*) 'FATAL: units (VROT) for rotations not recongized'
        STOP
      ENDIF

      END
      
      
**==denvr.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      DOUBLE PRECISION FUNCTION RVR(Mol , nchann, E, I)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2006 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
c
c	Interpolates 2-D double array Rate(MaxWell,MaxChan,Isize) to determine
c       k(E). Note that k(E) is stored in Rate as natural logs.
c
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , IE , I
      DOUBLE PRECISION E , X , Y1 , Y2 , Y3 , Y
      SAVE 
 
      IF ( E.LE.Emax1 ) THEN
         IE = 1 + INT(E/Egrain1)                
         Y = Rate(Mol,nchann,IE)
      ELSE
        CALL NTERP( E , IE , X )
        Y1 = Rate(Mol,nchann,IE-1)
        Y2 = Rate(Mol,nchann,IE)
        Y3 = Rate(Mol,nchann,IE+1)
        CALL QUDINT(Y1,Y2,Y3,X,Y)           ! Y = interpolated ln(sum)
      ENDIF

       RVR = EXP(Y)        ! because Rsum is stored as natural log in MultiWell   
 
      RETURN
      END
**==stepper.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      SUBROUTINE STEPPER(Mol,E,T,NumbDens,IP,Delt)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c     01/02  v1.2.1  -New Feature: can optionally choose to apply centrifugal
c                     corrections for any choice of model. Thus can apply them
c                     even when using Inverse Laplace Transform form of k(E).
c                     NOTE: this new feature affects the data file format.
c     11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                      inconsistently when calculating Arrhenius parameters 
c                     (k(inf) was correct, however).
c     09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                     See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c
c***********************************************************************
c	Carries out Gillespie Exact Stochastic procedure for one step
c
c	Mol	= molecule index
c	E	= energy (relative to ZPE of Mol)
c	T	= translational temperature
c	NumbDens= number density of collider gas
c	IP	= index of selected process
c	Delt	= time step
c***********************************************************************
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
 
      INTEGER Mol , i , IE , IP
 
      DOUBLE PRECISION Atot , RX , arg , E , X , Delt , Atemp , RAN1 , 
     &                 Y1 , Y2 , Y3 , Y , T , numbdens , Aref , CC ,
     &                 DOUBNTERP
 
      DOUBLE PRECISION vivr , civr , rivr , pivr, tivr, arg2 , RVR

      INTEGER iivr

      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

      EXTERNAL DOUBNTERP

      SAVE 
 
         i = Nchan(Mol) + 1

         CC = DOUBNTERP(E,Mol,CNORM)
         CC = CC/CNORMref(Mol)                   ! NEW up-transition probability collision treatment <<<<<<<<<<

         Rab(i) = CollK(Mol)*numbdens*CC                      ! Inelastic collision frequency: NEW up-transition probability
         Atot = Rab(Nchan(Mol)+1)                             ! Collision frequency

      IF ( Nchan(Mol).GT.0 ) THEN                             ! If reaction channels
         DO 50 i = 1 , Nchan(Mol)
            Rab(i) = 0.0D+00
            arg = E - Eor(Mol,i)                              ! Active energy in TS to determine k(E); centrifugal correction
            arg2 =-imax1*Egrain1                              ! minimum active energy in TS for the tunneling case
               if((itun(Mol,i).eq.1).and.(arg.ge.arg2))then        ! tunneling condition
                 call NTERP2(arg,IE,X)
                 Y1 = log(rivr(Mol,i,IE-1,numbdens))+TRate(Mol,i,IE-1)  ! log (k(E) * IVR transmission coefficient)
                 Y2 = log(rivr(Mol,i,IE,numbdens))+TRate(Mol,i,IE)      ! log (k(E) * IVR transmission coefficient)
                 Y3 = log(rivr(Mol,i,IE+1,numbdens))+TRate(Mol,i,IE+1)  ! log (k(E) * IVR transmission coefficient)
                 CALL QUDINT(Y1,Y2,Y3,X,Y) 
                 Rab(i) = EXP(Y)           ! reminder: k(E) stored as log [now includes IVR effects]
                 Atot = Atot + Rab(i)
               elseif ( arg .GE. 0.0D+00 ) THEN                    ! no tunneling
                 Rab(i) = RVR( Mol , i , arg , IE )                ! k(E) interpolation
     &                      * rivr( Mol , i , IE , numbdens )      ! IVR transmission coefficient
                 Atot = Atot + Rab(i)
               end if
 50      CONTINUE
      ENDIF

      IF ( Atot .GT. 0.0 ) THEN
        RX = RAN1(IDUM)
        Aref = RX*Atot
        Atemp = 0.0D+00
        IP = Nchan(Mol) + 2
        DO WHILE ( (Atemp.LE.Aref) .AND. (IP .GT. 1 ) )
         IP = IP - 1
         Atemp = Atemp + Rab(IP)
        ENDDO         
c       arg = E - Eor(1,1)
c       IF(arg .lt. 10. ) write(*,*) arg , Rab(1) , IP 
        RX = RAN1(IDUM)
        Delt = -LOG(RX)/Atot              ! Determine time step
        IF ( RX .LE. 1.00D-11 ) THEN        ! if random number too small, call another one
          RX = RAN1(IDUM)
          Delt = -LOG(RX)/Atot           ! Determine time step
        ENDIF
      ELSE
        IP =  Nchan(Mol) + 1
        Delt = Tlim
      ENDIF

      RETURN
      END

c    Checks keywords ('DUM') for validity and then assigns units or STOPS

      SUBROUTINE UnitTest ( DUM , Punits , Eunits , Rotatunits , KOUT )
      
      INTEGER   i , KOUT
      CHARACTER Punits*3, Eunits*4, Rotatunits*4, DUM*4
      DIMENSION DUM(3)
      SAVE
      
        IF (   ( DUM(1) .EQ. 'TORR' )
     &    .OR. ( DUM(1) .EQ. 'TOR' ) 
     &    .OR. ( DUM(1) .EQ. 'torr' )
     &    .OR. ( DUM(1) .EQ. 'tor' ) 
     &    .OR. ( DUM(1) .EQ. 'ATM' ) 
     &    .OR. ( DUM(1) .EQ. 'atm' ) 
     &    .OR. ( DUM(1) .EQ. 'BAR' ) 
     &    .OR. ( DUM(1) .EQ. 'bar' )
     &    .OR. ( DUM(1) .EQ. 'MCC' )
     &    .OR. ( DUM(1) .EQ. 'mcc' )
     &                           ) THEN
           Punits = DUM(1)
         ELSE
           WRITE(KOUT,*) '**** FATAL: NO PRESSURE UNITS RECOGNIZED ****'
           WRITE(*,*)    '**** FATAL: NO PRESSURE UNITS RECOGNIZED ****'
           STOP
        ENDIF

        IF (   ( DUM(2) .EQ. 'kjou' )
     &    .OR. ( DUM(2) .EQ. 'kJ' ) 
     &    .OR. ( DUM(2) .EQ. 'KJOU' )
     &    .OR. ( DUM(2) .EQ. 'KJou' ) 
     &    .OR. ( DUM(2) .EQ. 'cm-1' ) 
     &    .OR. ( DUM(2) .EQ. 'CM-1' ) 
     &    .OR. ( DUM(2) .EQ. 'Kcal' ) 
     &    .OR. ( DUM(2) .EQ. 'kcal' )
     &    .OR. ( DUM(2) .EQ. 'KCAL' )
     &                           ) THEN
           Eunits = DUM(2)
         ELSE
           WRITE(KOUT,*) '**** FATAL: NO ENERGY UNITS RECOGNIZED ****'
           WRITE(*,*)    '**** FATAL: NO ENERGY UNITS RECOGNIZED ****'
           STOP
         ENDIF
      
        IF (   ( DUM(3) .EQ. 'AMUA' )
     &    .OR. ( DUM(3) .EQ. 'amua' ) 
     &    .OR. ( DUM(3) .EQ. 'GMCM' )
     &    .OR. ( DUM(3) .EQ. 'gmcm' )
     &    .OR. ( DUM(3) .EQ. 'GmCm' )
     &    .OR. ( DUM(3) .EQ. 'CM-1' )
     &    .OR. ( DUM(3) .EQ. 'cm-1' )
     &    .OR. ( DUM(3) .EQ. 'MHZ' )
     &    .OR. ( DUM(3) .EQ. 'MHz' ) 
     &    .OR. ( DUM(3) .EQ. 'Mhz' ) 
     &    .OR. ( DUM(3) .EQ. 'mhz' ) 
     &    .OR. ( DUM(3) .EQ. 'GHZ' ) 
     &    .OR. ( DUM(3) .EQ. 'GHz' ) 
     &    .OR. ( DUM(3) .EQ. 'Ghz' ) 
     &    .OR. ( DUM(3) .EQ. 'ghz' )
     &                           ) THEN
           Rotatunits = DUM(3)
         ELSE
           WRITE(KOUT,*) '**** FATAL: NO ROTATION UNITS RECOGNIZED ****'
           WRITE(*,*)    '**** FATAL: NO ROTATION UNITS RECOGNIZED ****'
           STOP
        ENDIF
      
      RETURN
      END SUBROUTINE
